@@ -307,7 +307,7 @@ DEPS_CRITICAL_SUBPACKAGES = (
 )
 
 
-# Basic mandatory set of common libraries to build Blender, which are also available as pre-conmpiled libraries.
+# Basic mandatory set of common libraries to build Blender, which are also available as pre-compiled libraries.
 DEPS_MANDATORY_SUBPACKAGES = (
     Package(name="JPEG Library",
             distro_package_names={DISTRO_ID_DEBIAN: "libjpeg-dev",
@@ -403,7 +403,7 @@ DEPS_MANDATORY_SUBPACKAGES = (
 )
 
 
-# Basic optional set of common libraries to build Blender, which are also available as pre-conmpiled libraries.
+# Basic optional set of common libraries to build Blender, which are also available as pre-compiled libraries.
 DEPS_OPTIONAL_SUBPACKAGES = (
     Package(name="OpenJPG Library",
             distro_package_names={DISTRO_ID_DEBIAN: "libopenjp2-7-dev",
@@ -611,6 +611,22 @@ PYTHON_SUBPACKAGES = (
                                   DISTRO_ID_ARCH: ...,
                                   },
             ),
+    Package(name="fastjsonschema",
+            version="2.21.1", version_short="2.21", version_min="2.21", version_mex="3.0",
+            distro_package_names={DISTRO_ID_DEBIAN: "python-fastjsonschema",
+                                  DISTRO_ID_FEDORA: "python-fastjsonschema",
+                                  DISTRO_ID_SUSE: suse_pypackages_name_gen("fastjsonschema"),
+                                  DISTRO_ID_ARCH: "python-fastjsonschema",
+                                  },
+            ),
+    Package(name="cattrs",
+            version="25.1.1", version_short="25.1", version_min="25.1", version_mex="26.0",
+            distro_package_names={DISTRO_ID_DEBIAN: "python-cattrs",
+                                  DISTRO_ID_FEDORA: "python-cattrs",
+                                  DISTRO_ID_SUSE: suse_pypackages_name_gen("cattrs"),
+                                  DISTRO_ID_ARCH: "python-cattrs",
+                                  },
+            ),
 )
 
 
@@ -799,14 +815,6 @@ PACKAGES_ALL = (
                                   DISTRO_ID_ARCH: "usd",  # No official package, in AUR only currently.
                                   },
             ),
-    Package(name="OpenCollada Library", is_mandatory=False,
-            version="1.6.68", version_short="1.6", version_min="1.6.68", version_mex="1.7",
-            distro_package_names={DISTRO_ID_DEBIAN: "opencollada-dev",  # Useless, very old!
-                                  DISTRO_ID_FEDORA: "openCOLLADA-devel",
-                                  DISTRO_ID_SUSE: "libopenCOLLADA-devel",
-                                  DISTRO_ID_ARCH: "opencollada",
-                                  },
-            ),
     Package(name="Embree Library", is_mandatory=False,
             version="4.4.0", version_short="4.4", version_min="4.3", version_mex="5.0",
             sub_packages=(),
@@ -1082,7 +1090,7 @@ class PackageInstaller:
         return self.version_match(version, ref_version)
 
     def package_query_version_ge_lt(self, package_distro_name, ref_version_min, ref_version_mex):
-        """Check if given package name fits inbetween given minimal and maximal excluded versions."""
+        """Check if given package name fits in between given minimal and maximal excluded versions."""
         version = self.package_query_version_get(package_distro_name)
         if version is None:
             return False
@@ -1118,10 +1126,10 @@ class PackageInstaller:
         Generic heuristics to try and find 'best matching version' for a given package.
         For most packages it just ensures given package name version matches the exact version from the ``package``,
         or at least fits within the [version_min, version_mex[ range.
-        But some, like e.g. python or llvm, can have packages available for several versions,
-        with complex naming (like 'python3.10', 'llvm-9-dev', etc.).
+        But some, like e.g. Python or LLVM, can have packages available for several versions,
+        with complex naming (like ``python3.10``, ``llvm-9-dev``, etc.).
         This code attempts to find the best matching one possible, based on a set of 'possible names'
-        generated by the distro-specific ``package_name_version_gen`` generator.
+        generated by the distribution-specific ``package_name_version_gen`` generator.
         """
         # Check 'exact' version match on given name.
         if self.package_query_version_match(package_distro_name, package.version_short):
@@ -1433,7 +1441,7 @@ class PackageInstallerFedora(PackageInstaller):
         return self.package_version_get([MAYSUDO, "dnf", "info", "--installed", package_distro_name])
 
     def package_query_version_get_impl(self, package_distro_name):
-        return self.package_version_get([MAYSUDO, "dnf", "info", "--all", package_distro_name])
+        return self.package_version_get([MAYSUDO, "dnf", "info", package_distro_name])
 
     def package_name_version_gen(
             self,
