@@ -18,11 +18,7 @@
 
 #ifdef WITH_EMBREE
 
-#  if EMBREE_MAJOR_VERSION >= 4
-#    include <embree4/rtcore_geometry.h>
-#  else
-#    include <embree3/rtcore_geometry.h>
-#  endif
+#  include <embree4/rtcore_geometry.h>
 
 #  include "bvh/embree.h"
 
@@ -35,10 +31,6 @@
 #  include "util/progress.h"
 #  include "util/stats.h"
 
-#  if EMBREE_MAJOR_VERSION < 4
-#    include "kernel/device/cpu/bvh.h"
-#  endif
-
 CCL_NAMESPACE_BEGIN
 
 static_assert(Object::MAX_MOTION_STEPS <= RTC_MAX_TIME_STEP_COUNT,
@@ -73,7 +65,7 @@ static bool rtc_memory_monitor_func(void *userPtr, const ssize_t bytes, const bo
 
 static void rtc_error_func(void * /*unused*/, enum RTCError /*unused*/, const char *str)
 {
-  VLOG_WARNING << str;
+  LOG_WARNING << str;
 }
 
 static double progress_start_time = 0.0;
@@ -136,11 +128,8 @@ void BVHEmbree::build(Progress &progress,
   scene = rtcNewScene(rtc_device);
   const RTCSceneFlags scene_flags = (dynamic ? RTC_SCENE_FLAG_DYNAMIC : RTC_SCENE_FLAG_NONE) |
                                     (compact ? RTC_SCENE_FLAG_COMPACT : RTC_SCENE_FLAG_NONE) |
-                                    RTC_SCENE_FLAG_ROBUST
-#  if EMBREE_MAJOR_VERSION >= 4
-                                    | RTC_SCENE_FLAG_FILTER_FUNCTION_IN_ARGUMENTS
-#  endif
-      ;
+                                    RTC_SCENE_FLAG_ROBUST |
+                                    RTC_SCENE_FLAG_FILTER_FUNCTION_IN_ARGUMENTS;
   rtcSetSceneFlags(scene, scene_flags);
   build_quality = dynamic ? RTC_BUILD_QUALITY_LOW :
                             (params.use_spatial_split ? RTC_BUILD_QUALITY_HIGH :
@@ -242,7 +231,7 @@ void BVHEmbree::add_object(Object *ob, const int i)
   }
   else if (geom->is_hair()) {
     Hair *hair = static_cast<Hair *>(geom);
-    if (hair->num_curves() > 0) {
+    if (hair->is_traceable()) {
       add_curves(ob, hair, i);
     }
   }
@@ -289,11 +278,16 @@ void BVHEmbree::add_instance(Object *ob, const int i)
         geom_id, 0, RTC_FORMAT_FLOAT3X4_ROW_MAJOR, (const float *)&ob->get_tfm());
   }
 
-  rtcSetGeometryUserData(geom_id, (void *)instance_bvh->scene);
-  rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
-#  if EMBREE_MAJOR_VERSION >= 4
-  rtcSetGeometryEnableFilterFunctionFromArguments(geom_id, true);
+  rtcSetGeometryUserData(geom_id,
+#  if RTC_VERSION >= 40400
+                         (void *)rtcGetSceneTraversable(instance_bvh->scene)
+#  else
+                         (void *)instance_bvh->scene
 #  endif
+  );
+
+  rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
+  rtcSetGeometryEnableFilterFunctionFromArguments(geom_id, true);
 
   rtcCommitGeometry(geom_id);
   rtcAttachGeometryByID(scene, geom_id, i * 2);
@@ -338,8 +332,24 @@ void BVHEmbree::add_triangles(const Object *ob, const Mesh *mesh, const int i)
      * happen on GPU, and we cannot use standard host pointers at this point. So instead
      * of making a shared geometry buffer - a new Embree buffer will be created and data
      * will be copied. */
-    int *triangles_buffer = (int *)rtcSetNewGeometryBuffer(
-        geom_id, RTC_BUFFER_TYPE_INDEX, 0, RTC_FORMAT_UINT3, sizeof(int) * 3, num_triangles);
+    int *triangles_buffer = nullptr;
+#  if RTC_VERSION >= 40400
+    rtcSetNewGeometryBufferHostDevice(
+#  else
+    triangles_buffer = (int *)rtcSetNewGeometryBuffer(
+#  endif
+        geom_id,
+        RTC_BUFFER_TYPE_INDEX,
+        0,
+        RTC_FORMAT_UINT3,
+        sizeof(int) * 3,
+        num_triangles
+#  if RTC_VERSION >= 40400
+        ,
+        (void **)(&triangles_buffer),
+        nullptr
+#  endif
+    );
     assert(triangles_buffer);
     if (triangles_buffer) {
       static_assert(sizeof(int) == sizeof(uint));
@@ -350,12 +360,7 @@ void BVHEmbree::add_triangles(const Object *ob, const Mesh *mesh, const int i)
 
   rtcSetGeometryUserData(geom_id, (void *)prim_offset);
   rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
-#  if EMBREE_MAJOR_VERSION >= 4
   rtcSetGeometryEnableFilterFunctionFromArguments(geom_id, true);
-#  else
-  rtcSetGeometryOccludedFilterFunction(geom_id, kernel_embree_filter_occluded_func);
-  rtcSetGeometryIntersectFilterFunction(geom_id, kernel_embree_filter_intersection_func);
-#  endif
 
   rtcCommitGeometry(geom_id);
   rtcAttachGeometryByID(scene, geom_id, i * 2);
@@ -415,13 +420,24 @@ void BVHEmbree::set_tri_vertex_buffer(RTCGeometry geom_id, const Mesh *mesh, con
         /* As float3 is packed on GPU side, we map it to packed_float3. */
         /* There is no need for additional padding in rtcSetNewGeometryBuffer since Embree 3.6:
          * "Fixed automatic vertex buffer padding when using rtcSetNewGeometry API function". */
-        packed_float3 *verts_buffer = (packed_float3 *)rtcSetNewGeometryBuffer(
+        packed_float3 *verts_buffer = nullptr;
+#  if RTC_VERSION >= 40400
+        rtcSetNewGeometryBufferHostDevice(
+#  else
+        verts_buffer = (packed_float3 *)rtcSetNewGeometryBuffer(
+#  endif
             geom_id,
             RTC_BUFFER_TYPE_VERTEX,
             t,
             RTC_FORMAT_FLOAT3,
             sizeof(packed_float3),
-            num_verts);
+            num_verts
+#  if RTC_VERSION >= 40400
+            ,
+            (void **)(&verts_buffer),
+            nullptr
+#  endif
+        );
         assert(verts_buffer);
         if (verts_buffer) {
           for (size_t i = (size_t)0; i < num_verts; ++i) {
@@ -443,22 +459,34 @@ void pack_motion_verts(const size_t num_curves,
                        const Hair *hair,
                        const T *verts,
                        const float *curve_radius,
-                       float4 *rtc_verts)
+                       float4 *rtc_verts,
+                       CurveShapeType curve_shape)
 {
   for (size_t j = 0; j < num_curves; ++j) {
     const Hair::Curve c = hair->get_curve(j);
     int fk = c.first_key;
-    int k = 1;
-    for (; k < c.num_keys + 1; ++k, ++fk) {
-      rtc_verts[k].x = verts[fk].x;
-      rtc_verts[k].y = verts[fk].y;
-      rtc_verts[k].z = verts[fk].z;
-      rtc_verts[k].w = curve_radius[fk];
+
+    if (curve_shape == CURVE_THICK_LINEAR) {
+      for (int k = 0; k < c.num_keys; ++k, ++fk) {
+        rtc_verts[k].x = verts[fk].x;
+        rtc_verts[k].y = verts[fk].y;
+        rtc_verts[k].z = verts[fk].z;
+        rtc_verts[k].w = curve_radius[fk];
+      }
+      rtc_verts += c.num_keys;
+    }
+    else {
+      for (int k = 1; k < c.num_keys + 1; ++k, ++fk) {
+        rtc_verts[k].x = verts[fk].x;
+        rtc_verts[k].y = verts[fk].y;
+        rtc_verts[k].z = verts[fk].z;
+        rtc_verts[k].w = curve_radius[fk];
+      }
+      /* Duplicate Embree's Catmull-Rom spline CVs at the start and end of each curve. */
+      rtc_verts[0] = rtc_verts[1];
+      rtc_verts[c.num_keys + 1] = rtc_verts[c.num_keys];
+      rtc_verts += c.num_keys + 2;
     }
-    /* Duplicate Embree's Catmull-Rom spline CVs at the start and end of each curve. */
-    rtc_verts[0] = rtc_verts[1];
-    rtc_verts[k] = rtc_verts[k - 1];
-    rtc_verts += c.num_keys + 2;
   }
 }
 
@@ -492,26 +520,43 @@ void BVHEmbree::set_curve_vertex_buffer(RTCGeometry geom_id, const Hair *hair, c
     // handled separately. Attributes are float4s where the radius is stored in w and
     // the middle motion vector is from the mesh points which are stored float3s with
     // the radius stored in another array.
-    float4 *rtc_verts = (update) ? (float4 *)rtcGetGeometryBufferData(
-                                       geom_id, RTC_BUFFER_TYPE_VERTEX, t) :
-                                   (float4 *)rtcSetNewGeometryBuffer(geom_id,
-                                                                     RTC_BUFFER_TYPE_VERTEX,
-                                                                     t,
-                                                                     RTC_FORMAT_FLOAT4,
-                                                                     sizeof(float) * 4,
-                                                                     num_keys_embree);
+    float4 *rtc_verts = nullptr;
+    if (update) {
+      rtc_verts = (float4 *)rtcGetGeometryBufferData(geom_id, RTC_BUFFER_TYPE_VERTEX, t);
+    }
+    else {
+#  if RTC_VERSION >= 40400
+      rtcSetNewGeometryBufferHostDevice(
+#  else
+      rtc_verts = (float4 *)rtcSetNewGeometryBuffer(
+#  endif
+          geom_id,
+          RTC_BUFFER_TYPE_VERTEX,
+          t,
+          RTC_FORMAT_FLOAT4,
+          sizeof(float) * 4,
+          num_keys_embree
+#  if RTC_VERSION >= 40400
+          ,
+          (void **)(&rtc_verts),
+          nullptr
+#  endif
+      );
+    }
 
     assert(rtc_verts);
     if (rtc_verts) {
       const size_t num_curves = hair->num_curves();
       if (t == t_mid || attr_mP == nullptr) {
         const float3 *verts = hair->get_curve_keys().data();
-        pack_motion_verts<float3>(num_curves, hair, verts, curve_radius, rtc_verts);
+        pack_motion_verts<float3>(
+            num_curves, hair, verts, curve_radius, rtc_verts, hair->curve_shape);
       }
       else {
         const int t_ = (t > t_mid) ? (t - 1) : t;
         const float4 *verts = &attr_mP->data_float4()[t_ * num_keys];
-        pack_motion_verts<float4>(num_curves, hair, verts, curve_radius, rtc_verts);
+        pack_motion_verts<float4>(
+            num_curves, hair, verts, curve_radius, rtc_verts, hair->curve_shape);
       }
     }
 
@@ -544,14 +589,30 @@ void BVHEmbree::set_point_vertex_buffer(RTCGeometry geom_id,
     // handled separately. Attributes are float4s where the radius is stored in w and
     // the middle motion vector is from the mesh points which are stored float3s with
     // the radius stored in another array.
-    float4 *rtc_verts = (update) ? (float4 *)rtcGetGeometryBufferData(
-                                       geom_id, RTC_BUFFER_TYPE_VERTEX, t) :
-                                   (float4 *)rtcSetNewGeometryBuffer(geom_id,
-                                                                     RTC_BUFFER_TYPE_VERTEX,
-                                                                     t,
-                                                                     RTC_FORMAT_FLOAT4,
-                                                                     sizeof(float) * 4,
-                                                                     num_points);
+
+    float4 *rtc_verts = nullptr;
+    if (update) {
+      rtc_verts = (float4 *)rtcGetGeometryBufferData(geom_id, RTC_BUFFER_TYPE_VERTEX, t);
+    }
+    else {
+#  if RTC_VERSION >= 40400
+      rtcSetNewGeometryBufferHostDevice(
+#  else
+      rtc_verts = (float4 *)rtcSetNewGeometryBuffer(
+#  endif
+          geom_id,
+          RTC_BUFFER_TYPE_VERTEX,
+          t,
+          RTC_FORMAT_FLOAT4,
+          sizeof(float) * 4,
+          num_points
+#  if RTC_VERSION >= 40400
+          ,
+          (void **)(&rtc_verts),
+          nullptr
+#  endif
+      );
+    }
 
     assert(rtc_verts);
     if (rtc_verts) {
@@ -603,12 +664,7 @@ void BVHEmbree::add_points(const Object *ob, const PointCloud *pointcloud, const
 
   rtcSetGeometryUserData(geom_id, (void *)prim_offset);
   rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
-#  if EMBREE_MAJOR_VERSION >= 4
   rtcSetGeometryEnableFilterFunctionFromArguments(geom_id, true);
-#  else
-  rtcSetGeometryIntersectFilterFunction(geom_id, kernel_embree_filter_func_backface_cull);
-  rtcSetGeometryOccludedFilterFunction(geom_id, kernel_embree_filter_occluded_func_backface_cull);
-#  endif
 
   rtcCommitGeometry(geom_id);
   rtcAttachGeometryByID(scene, geom_id, i * 2);
@@ -639,21 +695,42 @@ void BVHEmbree::add_curves(const Object *ob, const Hair *hair, const int i)
     num_segments += c.num_segments();
   }
 
-  const enum RTCGeometryType type = (hair->curve_shape == CURVE_RIBBON ?
+  const enum RTCGeometryType type = (hair->curve_shape == CURVE_THICK_LINEAR ?
+                                         RTC_GEOMETRY_TYPE_ROUND_LINEAR_CURVE :
+                                     hair->curve_shape == CURVE_RIBBON ?
                                          RTC_GEOMETRY_TYPE_FLAT_CATMULL_ROM_CURVE :
                                          RTC_GEOMETRY_TYPE_ROUND_CATMULL_ROM_CURVE);
 
   RTCGeometry geom_id = rtcNewGeometry(rtc_device, type);
   rtcSetGeometryTessellationRate(geom_id, params.curve_subdivisions + 1);
-  unsigned *rtc_indices = (unsigned *)rtcSetNewGeometryBuffer(
-      geom_id, RTC_BUFFER_TYPE_INDEX, 0, RTC_FORMAT_UINT, sizeof(int), num_segments);
+  unsigned *rtc_indices = nullptr;
+#  if RTC_VERSION >= 40400
+  rtcSetNewGeometryBufferHostDevice(
+#  else
+  rtc_indices = (unsigned *)rtcSetNewGeometryBuffer(
+#  endif
+      geom_id,
+      RTC_BUFFER_TYPE_INDEX,
+      0,
+      RTC_FORMAT_UINT,
+      sizeof(int),
+      num_segments
+#  if RTC_VERSION >= 40400
+      ,
+      (void **)(&rtc_indices),
+      nullptr
+#  endif
+  );
+
   size_t rtc_index = 0;
   for (size_t j = 0; j < num_curves; ++j) {
     const Hair::Curve c = hair->get_curve(j);
     for (size_t k = 0; k < c.num_segments(); ++k) {
       rtc_indices[rtc_index] = c.first_key + k;
-      /* Room for extra CVs at Catmull-Rom splines. */
-      rtc_indices[rtc_index] += j * 2;
+      if (hair->curve_shape != CURVE_THICK_LINEAR) {
+        /* Room for extra CVs at Catmull-Rom splines. */
+        rtc_indices[rtc_index] += j * 2;
+      }
 
       ++rtc_index;
     }
@@ -666,19 +743,7 @@ void BVHEmbree::add_curves(const Object *ob, const Hair *hair, const int i)
 
   rtcSetGeometryUserData(geom_id, (void *)prim_offset);
   rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
-#  if EMBREE_MAJOR_VERSION >= 4
   rtcSetGeometryEnableFilterFunctionFromArguments(geom_id, true);
-#  else
-  if (hair->curve_shape == CURVE_RIBBON) {
-    rtcSetGeometryIntersectFilterFunction(geom_id, kernel_embree_filter_intersection_func);
-    rtcSetGeometryOccludedFilterFunction(geom_id, kernel_embree_filter_occluded_func);
-  }
-  else {
-    rtcSetGeometryIntersectFilterFunction(geom_id, kernel_embree_filter_func_backface_cull);
-    rtcSetGeometryOccludedFilterFunction(geom_id,
-                                         kernel_embree_filter_occluded_func_backface_cull);
-  }
-#  endif
 
   rtcCommitGeometry(geom_id);
   rtcAttachGeometryByID(scene, geom_id, i * 2 + 1);
@@ -706,7 +771,7 @@ void BVHEmbree::refit(Progress &progress)
       }
       else if (geom->is_hair()) {
         Hair *hair = static_cast<Hair *>(geom);
-        if (hair->num_curves() > 0) {
+        if (hair->is_traceable()) {
           RTCGeometry geom = rtcGetGeometry(scene, geom_id + 1);
           set_curve_vertex_buffer(geom, hair, true);
           rtcSetGeometryUserData(geom, (void *)hair->curve_segment_offset);
