@@ -57,6 +57,7 @@ NODE_DEFINE(Integrator)
   SOCKET_FLOAT(ao_distance, "AO Distance", FLT_MAX);
   SOCKET_FLOAT(ao_additive_factor, "AO Additive Factor", 0.0f);
 
+  SOCKET_BOOLEAN(volume_ray_marching, "Biased", false);
   SOCKET_INT(volume_max_steps, "Volume Max Steps", 1024);
   SOCKET_FLOAT(volume_step_rate, "Volume Step Rate", 1.0f);
 
@@ -185,6 +186,8 @@ void Integrator::device_update(Device *device, DeviceScene *dscene, Scene *scene
   device_free(device, dscene);
 
   /* integrator parameters */
+
+  /* Plus one so that a bounce of 0 indicates no global illumination, only direct illumination. */
   kintegrator->min_bounce = min_bounce + 1;
   kintegrator->max_bounce = max_bounce + 1;
 
@@ -194,7 +197,10 @@ void Integrator::device_update(Device *device, DeviceScene *dscene, Scene *scene
   kintegrator->max_volume_bounce = max_volume_bounce + 1;
 
   kintegrator->transparent_min_bounce = transparent_min_bounce + 1;
-  kintegrator->transparent_max_bounce = transparent_max_bounce + 1;
+
+  /* Unlike other type of bounces, 0 transparent bounce means there is no transparent bounce in the
+   * scene. */
+  kintegrator->transparent_max_bounce = transparent_max_bounce;
 
   kintegrator->ao_bounces = (ao_factor != 0.0f) ? ao_bounces : 0;
   kintegrator->ao_bounces_distance = ao_distance;
@@ -222,8 +228,8 @@ void Integrator::device_update(Device *device, DeviceScene *dscene, Scene *scene
     }
   }
 
+  kintegrator->volume_ray_marching = volume_ray_marching;
   kintegrator->volume_max_steps = volume_max_steps;
-  kintegrator->volume_step_rate = volume_step_rate;
 
   kintegrator->caustics_reflective = caustics_reflective;
   kintegrator->caustics_refractive = caustics_refractive;
@@ -395,7 +401,7 @@ AdaptiveSampling Integrator::get_adaptive_sampling() const
 
   if (clamped_aa_samples > 0 && adaptive_threshold == 0.0f) {
     adaptive_sampling.threshold = max(0.001f, 1.0f / (float)aa_samples);
-    VLOG_INFO << "Cycles adaptive sampling: automatic threshold = " << adaptive_sampling.threshold;
+    LOG_INFO << "Adaptive sampling: automatic threshold = " << adaptive_sampling.threshold;
   }
   else {
     adaptive_sampling.threshold = adaptive_threshold;
@@ -416,8 +422,7 @@ AdaptiveSampling Integrator::get_adaptive_sampling() const
      * in various test scenes. */
     const int min_samples = (int)ceilf(16.0f / powf(adaptive_sampling.threshold, 0.3f));
     adaptive_sampling.min_samples = max(4, min_samples);
-    VLOG_INFO << "Cycles adaptive sampling: automatic min samples = "
-              << adaptive_sampling.min_samples;
+    LOG_INFO << "Adaptive sampling: automatic min samples = " << adaptive_sampling.min_samples;
   }
   else {
     adaptive_sampling.min_samples = max(4, adaptive_min_samples);
