@@ -41,6 +41,8 @@
 #include "DEG_depsgraph.hh"
 #include "DEG_depsgraph_query.hh"
 
+using blender::Array;
+using blender::float3;
 using blender::IndexRange;
 
 static void displist_elem_free(DispList *dl)
@@ -163,9 +165,9 @@ static void curve_to_displist(const Curve *cu,
        * and resolution > 1. */
       const bool use_cyclic_sample = is_cyclic && (samples_len != 2);
 
-      DispList *dl = MEM_cnew<DispList>(__func__);
-      /* Add one to the length because of 'BKE_curve_forward_diff_bezier'. */
-      dl->verts = (float *)MEM_mallocN(sizeof(float[3]) * (samples_len + 1), __func__);
+      DispList *dl = MEM_callocN<DispList>(__func__);
+      /* Add one to the length because of #BKE_curve_forward_diff_bezier. */
+      dl->verts = MEM_malloc_arrayN<float>(3 * size_t(samples_len + 1), __func__);
       BLI_addtail(r_dispbase, dl);
       dl->parts = 1;
       dl->nr = samples_len;
@@ -218,8 +220,8 @@ static void curve_to_displist(const Curve *cu,
     }
     else if (nu->type == CU_NURBS) {
       const int len = (resolution * SEGMENTSU(nu));
-      DispList *dl = MEM_cnew<DispList>(__func__);
-      dl->verts = (float *)MEM_mallocN(len * sizeof(float[3]), __func__);
+      DispList *dl = MEM_callocN<DispList>(__func__);
+      dl->verts = MEM_malloc_arrayN<float>(3 * size_t(len), __func__);
       BLI_addtail(r_dispbase, dl);
       dl->parts = 1;
       dl->nr = len;
@@ -231,8 +233,8 @@ static void curve_to_displist(const Curve *cu,
     }
     else if (nu->type == CU_POLY) {
       const int len = nu->pntsu;
-      DispList *dl = MEM_cnew<DispList>(__func__);
-      dl->verts = (float *)MEM_mallocN(len * sizeof(float[3]), __func__);
+      DispList *dl = MEM_callocN<DispList>(__func__);
+      dl->verts = MEM_malloc_arrayN<float>(3 * size_t(len), __func__);
       BLI_addtail(r_dispbase, dl);
       dl->parts = 1;
       dl->nr = len;
@@ -240,7 +242,7 @@ static void curve_to_displist(const Curve *cu,
       dl->charidx = nu->charidx;
       dl->type = (is_cyclic && (dl->nr != 2)) ? DL_POLY : DL_SEGM;
 
-      float(*coords)[3] = (float(*)[3])dl->verts;
+      float (*coords)[3] = (float (*)[3])dl->verts;
       for (int i = 0; i < len; i++) {
         const BPoint *bp = &nu->bp[i];
         copy_v3_v3(coords[i], bp->vec);
@@ -322,7 +324,7 @@ void BKE_displist_fill(const ListBase *dispbase,
 
     const int triangles_len = BLI_scanfill_calc_ex(&sf_ctx, scanfill_flag, normal_proj);
     if (totvert != 0 && triangles_len != 0) {
-      DispList *dlnew = MEM_cnew<DispList>(__func__);
+      DispList *dlnew = MEM_callocN<DispList>(__func__);
       dlnew->type = DL_INDEX3;
       dlnew->flag = (dl_flag_accum & (DL_BACK_CURVE | DL_FRONT_CURVE));
       dlnew->rt = (dl_rt_accum & CU_SMOOTH);
@@ -330,8 +332,8 @@ void BKE_displist_fill(const ListBase *dispbase,
       dlnew->nr = totvert;
       dlnew->parts = triangles_len;
 
-      dlnew->index = (int *)MEM_mallocN(sizeof(int[3]) * triangles_len, __func__);
-      dlnew->verts = (float *)MEM_mallocN(sizeof(float[3]) * totvert, __func__);
+      dlnew->index = MEM_malloc_arrayN<int>(3 * size_t(triangles_len), __func__);
+      dlnew->verts = MEM_malloc_arrayN<float>(3 * size_t(totvert), __func__);
 
       /* vert data */
       int i;
@@ -377,9 +379,9 @@ static void bevels_to_filledpoly(const Curve *cu, ListBase *dispbase)
     if (dl->type == DL_SURF) {
       if ((dl->flag & DL_CYCL_V) && (dl->flag & DL_CYCL_U) == 0) {
         if ((cu->flag & CU_BACK) && (dl->flag & DL_BACK_CURVE)) {
-          DispList *dlnew = MEM_cnew<DispList>(__func__);
+          DispList *dlnew = MEM_callocN<DispList>(__func__);
           BLI_addtail(&front, dlnew);
-          dlnew->verts = (float *)MEM_mallocN(sizeof(float[3]) * dl->parts, __func__);
+          dlnew->verts = MEM_malloc_arrayN<float>(3 * size_t(dl->parts), __func__);
           dlnew->nr = dl->parts;
           dlnew->parts = 1;
           dlnew->type = DL_POLY;
@@ -396,9 +398,9 @@ static void bevels_to_filledpoly(const Curve *cu, ListBase *dispbase)
           }
         }
         if ((cu->flag & CU_FRONT) && (dl->flag & DL_FRONT_CURVE)) {
-          DispList *dlnew = MEM_cnew<DispList>(__func__);
+          DispList *dlnew = MEM_callocN<DispList>(__func__);
           BLI_addtail(&back, dlnew);
-          dlnew->verts = (float *)MEM_mallocN(sizeof(float[3]) * dl->parts, __func__);
+          dlnew->verts = MEM_malloc_arrayN<float>(3 * size_t(dl->parts), __func__);
           dlnew->nr = dl->parts;
           dlnew->parts = 1;
           dlnew->type = DL_POLY;
@@ -573,8 +575,7 @@ void BKE_curve_calc_modifiers_pre(Depsgraph *depsgraph,
   }
 
   float *keyVerts = nullptr;
-  float(*deformedVerts)[3] = nullptr;
-  int numVerts = 0;
+  Array<float3> deformedVerts;
   if (!editmode) {
     int numElems = 0;
     keyVerts = BKE_key_evaluate_object(ob, &numElems);
@@ -586,7 +587,7 @@ void BKE_curve_calc_modifiers_pre(Depsgraph *depsgraph,
        * tilts, which is passed through in the modifier stack.
        * this is also the reason curves do not use a virtual
        * shape key modifier yet. */
-      deformedVerts = BKE_curve_nurbs_key_vert_coords_alloc(source_nurb, keyVerts, &numVerts);
+      deformedVerts = BKE_curve_nurbs_key_vert_coords_alloc(source_nurb, keyVerts);
     }
   }
 
@@ -609,12 +610,11 @@ void BKE_curve_calc_modifiers_pre(Depsgraph *depsgraph,
 
       blender::bke::ScopedModifierTimer modifier_timer{*md};
 
-      if (!deformedVerts) {
-        deformedVerts = BKE_curve_nurbs_vert_coords_alloc(source_nurb, &numVerts);
+      if (deformedVerts.is_empty()) {
+        deformedVerts = BKE_curve_nurbs_vert_coords_alloc(source_nurb);
       }
 
-      mti->deform_verts(
-          md, &mectx, nullptr, {reinterpret_cast<blender::float3 *>(deformedVerts), numVerts});
+      mti->deform_verts(md, &mectx, nullptr, deformedVerts);
 
       if (md == pretessellatePoint) {
         break;
@@ -622,9 +622,8 @@ void BKE_curve_calc_modifiers_pre(Depsgraph *depsgraph,
     }
   }
 
-  if (deformedVerts) {
+  if (!deformedVerts.is_empty()) {
     BKE_curve_nurbs_vert_coords_apply(target_nurb, deformedVerts, false);
-    MEM_freeN(deformedVerts);
   }
   if (keyVerts) { /* these are not passed through modifier stack */
     BKE_curve_nurbs_key_vert_tilts_apply(target_nurb, keyVerts);
@@ -765,8 +764,8 @@ static void displist_surf_indices(DispList *dl)
 
   dl->totindex = 0;
 
-  int *index = dl->index = (int *)MEM_mallocN(sizeof(int[4]) * (dl->parts + 1) * (dl->nr + 1),
-                                              __func__);
+  int *index = dl->index = MEM_malloc_arrayN<int>(4 * size_t(dl->parts + 1) * size_t(dl->nr + 1),
+                                                  __func__);
 
   for (int a = 0; a < dl->parts; a++) {
 
@@ -821,8 +820,8 @@ static blender::bke::GeometrySet evaluate_surface_object(Depsgraph *depsgraph,
     if (nu->pntsv == 1) {
       const int len = SEGMENTSU(nu) * resolu;
 
-      DispList *dl = MEM_cnew<DispList>(__func__);
-      dl->verts = (float *)MEM_mallocN(len * sizeof(float[3]), __func__);
+      DispList *dl = MEM_callocN<DispList>(__func__);
+      dl->verts = MEM_malloc_arrayN<float>(3 * size_t(len), __func__);
 
       BLI_addtail(r_dispbase, dl);
       dl->parts = 1;
@@ -844,8 +843,8 @@ static blender::bke::GeometrySet evaluate_surface_object(Depsgraph *depsgraph,
     else {
       const int len = (nu->pntsu * resolu) * (nu->pntsv * resolv);
 
-      DispList *dl = MEM_cnew<DispList>(__func__);
-      dl->verts = (float *)MEM_mallocN(len * sizeof(float[3]), __func__);
+      DispList *dl = MEM_callocN<DispList>(__func__);
+      dl->verts = MEM_malloc_arrayN<float>(3 * size_t(len), __func__);
       BLI_addtail(r_dispbase, dl);
 
       dl->col = nu->mat_nr;
@@ -946,8 +945,8 @@ static void fillBevelCap(const Nurb *nu,
                          const float *prev_fp,
                          ListBase *dispbase)
 {
-  DispList *dl = MEM_cnew<DispList>(__func__);
-  dl->verts = (float *)MEM_mallocN(sizeof(float[3]) * dlb->nr, __func__);
+  DispList *dl = MEM_callocN<DispList>(__func__);
+  dl->verts = MEM_malloc_arrayN<float>(3 * size_t(dlb->nr), __func__);
   memcpy(dl->verts, prev_fp, sizeof(float[3]) * dlb->nr);
 
   dl->type = DL_POLY;
@@ -1148,8 +1147,8 @@ static blender::bke::GeometrySet evaluate_curve_type_object(Depsgraph *depsgraph
 
       /* exception handling; curve without bevel or extrude, with width correction */
       if (BLI_listbase_is_empty(&dlbev)) {
-        DispList *dl = MEM_cnew<DispList>("makeDispListbev");
-        dl->verts = (float *)MEM_mallocN(sizeof(float[3]) * bl->nr, "dlverts");
+        DispList *dl = MEM_callocN<DispList>("makeDispListbev");
+        dl->verts = MEM_malloc_arrayN<float>(3 * size_t(bl->nr), "dlverts");
         BLI_addtail(r_dispbase, dl);
 
         if (bl->poly != -1) {
@@ -1198,8 +1197,9 @@ static blender::bke::GeometrySet evaluate_curve_type_object(Depsgraph *depsgraph
 
         LISTBASE_FOREACH (DispList *, dlb, &dlbev) {
           /* For each part of the bevel use a separate display-block. */
-          DispList *dl = MEM_cnew<DispList>(__func__);
-          dl->verts = data = (float *)MEM_mallocN(sizeof(float[3]) * dlb->nr * steps, __func__);
+          DispList *dl = MEM_callocN<DispList>(__func__);
+          dl->verts = data = MEM_malloc_arrayN<float>(3 * size_t(dlb->nr) * size_t(steps),
+                                                      __func__);
           BLI_addtail(r_dispbase, dl);
 
           dl->type = DL_SURF;
@@ -1281,12 +1281,18 @@ static blender::bke::GeometrySet evaluate_curve_type_object(Depsgraph *depsgraph
 
             if ((cu->flag & CU_FILL_CAPS) && !(nu->flagu & CU_NURB_CYCLIC)) {
               if (a == 1) {
-                fillBevelCap(nu, dlb, cur_data - 3 * dlb->nr, &bottom_capbase);
-                copy_v3_v3(bottom_no, bevp->dir);
+                /* Can occur when the `bevp->vec` is NAN, see: #141612. */
+                if (len_squared_v3(bevp->dir) > 0.0f) {
+                  fillBevelCap(nu, dlb, cur_data - 3 * dlb->nr, &bottom_capbase);
+                  copy_v3_v3(bottom_no, bevp->dir);
+                }
               }
               if (a == steps - 1) {
-                fillBevelCap(nu, dlb, cur_data, &top_capbase);
-                negate_v3_v3(top_no, bevp->dir);
+                /* Can occur when the `bevp->vec` is NAN, see: #141612. */
+                if (len_squared_v3(bevp->dir) > 0.0f) {
+                  fillBevelCap(nu, dlb, cur_data, &top_capbase);
+                  negate_v3_v3(top_no, bevp->dir);
+                }
               }
             }
           }
@@ -1297,8 +1303,10 @@ static blender::bke::GeometrySet evaluate_curve_type_object(Depsgraph *depsgraph
 
         if (bottom_capbase.first) {
           BKE_displist_fill(&bottom_capbase, r_dispbase, bottom_no, false);
-          BKE_displist_fill(&top_capbase, r_dispbase, top_no, false);
           BKE_displist_free(&bottom_capbase);
+        }
+        if (top_capbase.first) {
+          BKE_displist_fill(&top_capbase, r_dispbase, top_no, false);
           BKE_displist_free(&top_capbase);
         }
       }
@@ -1324,7 +1332,7 @@ void BKE_displist_make_curveTypes(Depsgraph *depsgraph,
    * which may reset the object data pointer in some cases. */
   const Curve &original_curve = *static_cast<const Curve *>(ob->data);
 
-  ob->runtime->curve_cache = MEM_cnew<CurveCache>(__func__);
+  ob->runtime->curve_cache = MEM_callocN<CurveCache>(__func__);
   ListBase *dispbase = &ob->runtime->curve_cache->disp;
 
   if (ob->type == OB_SURF) {
