@@ -17,7 +17,6 @@
 
 #include "BLI_array.hh"
 #include "BLI_bitmap.h"
-#include "BLI_buffer.h"
 #include "BLI_function_ref.hh"
 #include "BLI_math_geom.h"
 #include "BLI_math_vector.h"
@@ -35,86 +34,56 @@
 /** \name Mesh Connectivity Mapping
  * \{ */
 
-UvVertMap *BKE_mesh_uv_vert_map_create(const blender::OffsetIndices<int> faces,
-                                       const bool *hide_poly,
-                                       const bool *select_poly,
-                                       const int *corner_verts,
-                                       const float (*mloopuv)[2],
-                                       uint totvert,
-                                       const float limit[2],
-                                       const bool selected,
-                                       const bool use_winding)
+UvVertMap *BKE_mesh_uv_vert_map_create(blender::OffsetIndices<int> faces,
+                                       blender::Span<int> corner_verts,
+                                       blender::Span<blender::float2> uv_map,
+                                       int verts_num,
+                                       const blender::float2 &limit,
+                                       bool use_winding)
 {
+  using namespace blender;
   /* NOTE: N-gon version WIP, based on #BM_uv_vert_map_create. */
-
-  UvVertMap *vmap;
-  UvMapVert *buf;
-  int i, totuv, nverts;
-
-  BLI_buffer_declare_static(blender::float2, tf_uv_buf, BLI_BUFFER_NOP, 32);
-
-  totuv = 0;
-
-  /* generate UvMapVert array */
-  for (const int64_t a : faces.index_range()) {
-    if (!selected || (!(hide_poly && hide_poly[a]) && (select_poly && select_poly[a]))) {
-      totuv += int(faces[a].size());
-    }
-  }
-
-  if (totuv == 0) {
+  if (faces.is_empty()) {
     return nullptr;
   }
+  const int corners_num = faces.total_size();
 
-  vmap = MEM_callocN<UvVertMap>("UvVertMap");
-  buf = vmap->buf = MEM_calloc_arrayN<UvMapVert>(size_t(totuv), "UvMapVert");
-  vmap->vert = MEM_calloc_arrayN<UvMapVert *>(totvert, "UvMapVert*");
+  UvVertMap *vmap = MEM_callocN<UvVertMap>("UvVertMap");
+  UvMapVert *buf = vmap->buf = MEM_calloc_arrayN<UvMapVert>(size_t(corners_num), "UvMapVert");
+  vmap->vert = MEM_calloc_arrayN<UvMapVert *>(size_t(verts_num), "UvMapVert*");
 
   if (!vmap->vert || !vmap->buf) {
     BKE_mesh_uv_vert_map_free(vmap);
     return nullptr;
   }
 
-  bool *winding = nullptr;
+  Array<bool> winding;
   if (use_winding) {
-    winding = MEM_calloc_arrayN<bool>(size_t(faces.size()), "winding");
+    winding = Array<bool>(faces.size(), false);
+    threading::parallel_for(faces.index_range(), 1024, [&](const IndexRange range) {
+      for (const int64_t face : range) {
+        const Span<float2> face_uvs = uv_map.slice(faces[face]);
+        winding[face] = cross_poly_v2(reinterpret_cast<const float (*)[2]>(face_uvs.data()),
+                                      uint(faces[face].size())) < 0.0f;
+      }
+    });
   }
 
   for (const int64_t a : faces.index_range()) {
-    const blender::IndexRange face = faces[a];
-    if (!selected || (!(hide_poly && hide_poly[a]) && (select_poly && select_poly[a]))) {
-      float(*tf_uv)[2] = nullptr;
-
-      if (use_winding) {
-        tf_uv = (float(*)[2])BLI_buffer_reinit_data(
-            &tf_uv_buf, blender::float2, size_t(face.size()));
-      }
-
-      nverts = int(face.size());
-
-      for (i = 0; i < nverts; i++) {
-        buf->loop_of_face_index = ushort(i);
-        buf->face_index = uint(a);
-        buf->separate = false;
-        buf->next = vmap->vert[corner_verts[face[i]]];
-        vmap->vert[corner_verts[face[i]]] = buf;
-
-        if (use_winding) {
-          copy_v2_v2(tf_uv[i], mloopuv[face[i]]);
-        }
-
-        buf++;
-      }
-
-      if (use_winding) {
-        winding[a] = cross_poly_v2(tf_uv, uint(nverts)) < 0;
-      }
+    const IndexRange face = faces[a];
+    for (const int64_t i : face.index_range()) {
+      buf->loop_of_face_index = ushort(i);
+      buf->face_index = uint(a);
+      buf->separate = false;
+      buf->next = vmap->vert[corner_verts[face[i]]];
+      vmap->vert[corner_verts[face[i]]] = buf;
+      buf++;
     }
   }
 
   /* sort individual uvs for each vert */
-  for (uint a = 0; a < totvert; a++) {
-    UvMapVert *newvlist = nullptr, *vlist = vmap->vert[a];
+  for (const int64_t vert : IndexRange(verts_num)) {
+    UvMapVert *newvlist = nullptr, *vlist = vmap->vert[vert];
     UvMapVert *iterv, *v, *lastv, *next;
     const float *uv, *uv2;
     float uvdiff[2];
@@ -125,14 +94,14 @@ UvVertMap *BKE_mesh_uv_vert_map_create(const blender::OffsetIndices<int> faces,
       v->next = newvlist;
       newvlist = v;
 
-      uv = mloopuv[faces[v->face_index].start() + v->loop_of_face_index];
+      uv = uv_map[faces[v->face_index].start() + v->loop_of_face_index];
       lastv = nullptr;
       iterv = vlist;
 
       while (iterv) {
         next = iterv->next;
 
-        uv2 = mloopuv[faces[iterv->face_index].start() + iterv->loop_of_face_index];
+        uv2 = uv_map[faces[iterv->face_index].start() + iterv->loop_of_face_index];
         sub_v2_v2v2(uvdiff, uv2, uv);
 
         if (fabsf(uv[0] - uv2[0]) < limit[0] && fabsf(uv[1] - uv2[1]) < limit[1] &&
@@ -157,15 +126,9 @@ UvVertMap *BKE_mesh_uv_vert_map_create(const blender::OffsetIndices<int> faces,
       newvlist->separate = true;
     }
 
-    vmap->vert[a] = newvlist;
+    vmap->vert[vert] = newvlist;
   }
 
-  if (use_winding) {
-    MEM_freeN(winding);
-  }
-
-  BLI_buffer_free(&tf_uv_buf);
-
   return vmap;
 }
 
@@ -924,12 +887,9 @@ void BKE_mesh_loop_islands_add(MeshIslandStore *island_store,
 }
 
 static bool mesh_calc_islands_loop_face_uv(const int totedge,
-                                           const bool *uv_seams,
+                                           const blender::Span<bool> uv_seams,
                                            const blender::OffsetIndices<int> faces,
-                                           const int *corner_verts,
-                                           const int *corner_edges,
-                                           const int corners_num,
-                                           const float (*luvs)[2],
+                                           const blender::Span<int> corner_edges,
                                            MeshIslandStore *r_island_store)
 {
   using namespace blender;
@@ -952,21 +912,16 @@ static bool mesh_calc_islands_loop_face_uv(const int totedge,
   int grp_idx;
 
   BKE_mesh_loop_islands_clear(r_island_store);
-  BKE_mesh_loop_islands_init(
-      r_island_store, MISLAND_TYPE_LOOP, corners_num, MISLAND_TYPE_POLY, MISLAND_TYPE_EDGE);
+  BKE_mesh_loop_islands_init(r_island_store,
+                             MISLAND_TYPE_LOOP,
+                             int(corner_edges.size()),
+                             MISLAND_TYPE_POLY,
+                             MISLAND_TYPE_EDGE);
 
   Array<int> edge_to_face_offsets;
   Array<int> edge_to_face_indices;
   const GroupedSpan<int> edge_to_face_map = bke::mesh::build_edge_to_face_map(
-      faces, {corner_edges, corners_num}, totedge, edge_to_face_offsets, edge_to_face_indices);
-
-  Array<int> edge_to_corner_offsets;
-  Array<int> edge_to_corner_indices;
-  GroupedSpan<int> edge_to_corner_map;
-  if (luvs) {
-    edge_to_corner_map = bke::mesh::build_edge_to_corner_map(
-        {corner_edges, corners_num}, totedge, edge_to_corner_offsets, edge_to_corner_indices);
-  }
+      faces, corner_edges, totedge, edge_to_face_offsets, edge_to_face_indices);
 
   /* TODO: I'm not sure edge seam flag is enough to define UV islands?
    *       Maybe we should also consider UV-maps values
@@ -975,48 +930,18 @@ static bool mesh_calc_islands_loop_face_uv(const int totedge,
    *       and each UVMap would then need its own mesh mapping, not sure we want that at all!
    */
   auto mesh_check_island_boundary_uv = [&](const int /*face_index*/,
-                                           const int corner,
+                                           const int /*corner*/,
                                            const int edge_index,
                                            const int /*edge_user_count*/,
                                            const Span<int> /*edge_face_map_elem*/) -> bool {
-    if (luvs) {
-      const Span<int> edge_to_corners = edge_to_corner_map[corner_edges[corner]];
-
-      BLI_assert(edge_to_corners.size() >= 2 && (edge_to_corners.size() % 2) == 0);
-
-      const int v1 = corner_verts[edge_to_corners[0]];
-      const int v2 = corner_verts[edge_to_corners[1]];
-      const float *uvco_v1 = luvs[edge_to_corners[0]];
-      const float *uvco_v2 = luvs[edge_to_corners[1]];
-      for (int i = 2; i < edge_to_corners.size(); i += 2) {
-        if (corner_verts[edge_to_corners[i]] == v1) {
-          if (!equals_v2v2(uvco_v1, luvs[edge_to_corners[i]]) ||
-              !equals_v2v2(uvco_v2, luvs[edge_to_corners[i + 1]]))
-          {
-            return true;
-          }
-        }
-        else {
-          BLI_assert(corner_verts[edge_to_corners[i]] == v2);
-          UNUSED_VARS_NDEBUG(v2);
-          if (!equals_v2v2(uvco_v2, luvs[edge_to_corners[i]]) ||
-              !equals_v2v2(uvco_v1, luvs[edge_to_corners[i + 1]]))
-          {
-            return true;
-          }
-        }
-      }
-      return false;
-    }
-
     /* Edge is UV boundary if tagged as seam. */
-    return uv_seams && uv_seams[edge_index];
+    return !uv_seams.is_empty() && uv_seams[edge_index];
   };
 
   face_edge_loop_islands_calc(totedge,
                               0,
                               faces,
-                              {corner_edges, corners_num},
+                              corner_edges,
                               {},
                               edge_to_face_map,
                               {},
@@ -1041,7 +966,7 @@ static bool mesh_calc_islands_loop_face_uv(const int totedge,
   }
 
   face_indices = MEM_malloc_arrayN<int>(size_t(faces.size()), __func__);
-  loop_indices = MEM_malloc_arrayN<int>(size_t(corners_num), __func__);
+  loop_indices = MEM_malloc_arrayN<int>(size_t(corner_edges.size()), __func__);
 
   /* NOTE: here we ignore '0' invalid group - this should *never* happen in this case anyway? */
   for (grp_idx = 1; grp_idx <= num_face_groups; grp_idx++) {
@@ -1094,38 +1019,17 @@ static bool mesh_calc_islands_loop_face_uv(const int totedge,
   return true;
 }
 
-bool BKE_mesh_calc_islands_loop_face_edgeseam(const float (*vert_positions)[3],
-                                              const int totvert,
-                                              const blender::int2 *edges,
-                                              const int totedge,
-                                              const bool *uv_seams,
+bool BKE_mesh_calc_islands_loop_face_edgeseam(const blender::Span<blender::float3> vert_positions,
+                                              const blender::Span<blender::int2> edges,
+                                              const blender::Span<bool> uv_seams,
                                               const blender::OffsetIndices<int> faces,
-                                              const int *corner_verts,
-                                              const int *corner_edges,
-                                              const int corners_num,
+                                              const blender::Span<int> /*corner_verts*/,
+                                              const blender::Span<int> corner_edges,
                                               MeshIslandStore *r_island_store)
 {
-  UNUSED_VARS(vert_positions, totvert, edges);
+  UNUSED_VARS(vert_positions);
   return mesh_calc_islands_loop_face_uv(
-      totedge, uv_seams, faces, corner_verts, corner_edges, corners_num, nullptr, r_island_store);
-}
-
-bool BKE_mesh_calc_islands_loop_face_uvmap(float (*vert_positions)[3],
-                                           const int totvert,
-                                           blender::int2 *edges,
-                                           const int totedge,
-                                           const bool *uv_seams,
-                                           const blender::OffsetIndices<int> faces,
-                                           const int *corner_verts,
-                                           const int *corner_edges,
-                                           const int corners_num,
-                                           const float (*luvs)[2],
-                                           MeshIslandStore *r_island_store)
-{
-  UNUSED_VARS(vert_positions, totvert, edges);
-  BLI_assert(luvs != nullptr);
-  return mesh_calc_islands_loop_face_uv(
-      totedge, uv_seams, faces, corner_verts, corner_edges, corners_num, luvs, r_island_store);
+      int(edges.size()), uv_seams, faces, corner_edges, r_island_store);
 }
 
 /** \} */
