@@ -15,12 +15,10 @@
 #include "BLI_math_geom.h"
 #include "BLI_math_vector.h"
 
-#include "DNA_meshdata_types.h"
-
-#include "bmesh.h"
-#include "bmesh_path_uv.h" /* own include */
-#include "intern/bmesh_query.h"
-#include "intern/bmesh_query_uv.h"
+#include "bmesh.hh"
+#include "bmesh_path_uv.hh" /* own include */
+#include "intern/bmesh_query.hh"
+#include "intern/bmesh_query_uv.hh"
 
 #define COST_INIT_MAX FLT_MAX
 
@@ -142,8 +140,8 @@ LinkNode *BM_mesh_calc_path_uv_vert(BMesh *bm,
 
   /* Allocate. */
   totloop = bm->totloop;
-  loops_prev = static_cast<BMLoop **>(MEM_callocN(sizeof(*loops_prev) * totloop, __func__));
-  cost = static_cast<float *>(MEM_mallocN(sizeof(*cost) * totloop, __func__));
+  loops_prev = MEM_calloc_arrayN<BMLoop *>(totloop, __func__);
+  cost = MEM_malloc_arrayN<float>(totloop, __func__);
 
   copy_vn_fl(cost, totloop, COST_INIT_MAX);
 
@@ -249,6 +247,9 @@ static void edgetag_add_adjacent_uv(HeapSimple *heap,
       BMEdge *e_b;
       BMIter eiter;
       BM_ITER_ELEM (e_b, &eiter, l_a_verts[i]->v, BM_EDGES_OF_VERT) {
+        if (e_b->l == nullptr) {
+          continue;
+        }
         BMLoop *l_first, *l_b;
         l_first = l_b = e_b->l;
         do {
@@ -343,8 +344,8 @@ LinkNode *BM_mesh_calc_path_uv_edge(BMesh *bm,
   bm->elem_index_dirty &= ~BM_LOOP;
 
   totloop = bm->totloop;
-  loops_prev = static_cast<BMLoop **>(MEM_callocN(sizeof(*loops_prev) * totloop, __func__));
-  cost = static_cast<float *>(MEM_mallocN(sizeof(*cost) * totloop, __func__));
+  loops_prev = MEM_calloc_arrayN<BMLoop *>(totloop, __func__);
+  cost = MEM_malloc_arrayN<float>(totloop, __func__);
 
   copy_vn_fl(cost, totloop, COST_INIT_MAX);
 
@@ -571,8 +572,8 @@ LinkNode *BM_mesh_calc_path_uv_face(BMesh *bm,
 
   /* Allocate. */
   totface = bm->totface;
-  faces_prev = static_cast<BMFace **>(MEM_callocN(sizeof(*faces_prev) * totface, __func__));
-  cost = static_cast<float *>(MEM_mallocN(sizeof(*cost) * totface, __func__));
+  faces_prev = MEM_calloc_arrayN<BMFace *>(totface, __func__);
+  cost = MEM_malloc_arrayN<float>(totface, __func__);
 
   copy_vn_fl(cost, totface, COST_INIT_MAX);
 
