@@ -16,6 +16,7 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "BLI_listbase.h"
 #include "BLI_span.hh"
 #include "BLI_utildefines.h"
 
@@ -59,7 +60,6 @@
 #include "BKE_curve.hh"
 #include "BKE_effect.h"
 #include "BKE_fcurve_driver.h"
-#include "BKE_gpencil_legacy.h"
 #include "BKE_gpencil_modifier_legacy.h"
 #include "BKE_grease_pencil.hh"
 #include "BKE_idprop.hh"
@@ -104,6 +104,7 @@
 
 #include "intern/builder/deg_builder.h"
 #include "intern/builder/deg_builder_key.h"
+#include "intern/builder/deg_builder_relations_drivers.h"
 #include "intern/builder/deg_builder_rna.h"
 #include "intern/depsgraph.hh"
 #include "intern/depsgraph_light_linking.hh"
@@ -136,12 +137,12 @@ DepsgraphNodeBuilder::DepsgraphNodeBuilder(Main *bmain,
 
 DepsgraphNodeBuilder::~DepsgraphNodeBuilder()
 {
-  for (IDInfo *id_info : id_info_hash_.values()) {
-    if (id_info->id_cow != nullptr) {
-      deg_free_eval_copy_datablock(id_info->id_cow);
-      MEM_freeN(id_info->id_cow);
+  /* Cannot be in an IDInfo destructor, as these COW IDs do not belong to the IDInfo data. */
+  for (IDInfo &id_info : id_info_hash_.values()) {
+    if (id_info.id_cow != nullptr) {
+      deg_free_eval_copy_datablock(id_info.id_cow);
+      MEM_freeN(id_info.id_cow);
     }
-    MEM_freeN(id_info);
   }
 }
 
@@ -155,7 +156,7 @@ IDNode *DepsgraphNodeBuilder::add_id_node(ID *id)
   IDComponentsMask previously_visible_components_mask = 0;
   uint32_t previous_eval_flags = 0;
   DEGCustomDataMeshMasks previous_customdata_masks;
-  IDInfo *id_info = id_info_hash_.lookup_default(id->session_uid, nullptr);
+  IDInfo *id_info = id_info_hash_.lookup_ptr(id->session_uid);
   if (id_info != nullptr) {
     id_cow = id_info->id_cow;
     previously_visible_components_mask = id_info->previously_visible_components_mask;
@@ -382,20 +383,21 @@ void DepsgraphNodeBuilder::begin_build()
      * for whether id_cow is expanded to access freed memory. In order to deal with this we
      * check whether an evaluated copy is needed based on a scalar value which does not lead to
      * access of possibly deleted memory. */
-    IDInfo *id_info = (IDInfo *)MEM_mallocN(sizeof(IDInfo), "depsgraph id info");
-    if (deg_eval_copy_is_needed(id_node->id_type) && deg_eval_copy_is_expanded(id_node->id_cow) &&
-        id_node->id_orig != id_node->id_cow)
-    {
-      id_info->id_cow = id_node->id_cow;
+    IDInfo id_info{};
+    if (deg_eval_copy_is_needed(id_node->id_type) && id_node->id_orig != id_node->id_cow) {
+      if (deg_eval_copy_is_expanded(id_node->id_cow)) {
+        id_info.id_cow = id_node->id_cow;
+      }
+      else {
+        /* This ID has not been expanded yet. Don't reuse it like already expanded IDs. */
+        MEM_SAFE_FREE(id_node->id_cow);
+      }
     }
-    else {
-      id_info->id_cow = nullptr;
-    }
-    id_info->previously_visible_components_mask = id_node->visible_components_mask;
-    id_info->previous_eval_flags = id_node->eval_flags;
-    id_info->previous_customdata_masks = id_node->customdata_masks;
+    id_info.previously_visible_components_mask = id_node->visible_components_mask;
+    id_info.previous_eval_flags = id_node->eval_flags;
+    id_info.previous_customdata_masks = id_node->customdata_masks;
     BLI_assert(!id_info_hash_.contains(id_node->id_orig_session_uid));
-    id_info_hash_.add_new(id_node->id_orig_session_uid, id_info);
+    id_info_hash_.add_new(id_node->id_orig_session_uid, std::move(id_info));
     id_node->id_cow = nullptr;
   }
 
@@ -491,14 +493,15 @@ void DepsgraphNodeBuilder::update_invalid_cow_pointers()
    * code), but cannot really be avoided currently. */
 
   for (const IDNode *id_node : graph_->id_nodes) {
-    if (id_node->previously_visible_components_mask == 0) {
-      /* Newly added node/ID, no need to check it. */
-      continue;
-    }
     if (ELEM(id_node->id_cow, id_node->id_orig, nullptr)) {
       /* Node/ID with no copy-on-eval data, no need to check it. */
       continue;
     }
+    if (!deg_eval_copy_is_expanded(id_node->id_cow)) {
+      /* Copy-on-eval data is not expanded yet, so this is a newly added node/ID that has not been
+       * evaluated yet. */
+      continue;
+    }
     if ((id_node->id_cow->recalc & ID_RECALC_SYNC_TO_EVAL) != 0) {
       /* Node/ID already tagged for copy-on-eval flush, no need to check it. */
       continue;
@@ -656,7 +659,6 @@ void DepsgraphNodeBuilder::build_id(ID *id, const bool force_be_visible)
       break;
 
     case ID_LI:
-    case ID_IP:
     case ID_SCR:
     case ID_VF:
     case ID_BR:
@@ -672,11 +674,12 @@ void DepsgraphNodeBuilder::build_id(ID *id, const bool force_be_visible)
 
 void DepsgraphNodeBuilder::build_generic_id(ID *id)
 {
-  if (built_map_.checkIsBuiltAndTag(id)) {
+  if (built_map_.check_is_built_and_tag(id)) {
     return;
   }
 
   build_idproperties(id->properties);
+  build_idproperties(id->system_properties);
   build_animdata(id);
   build_parameters(id);
 }
@@ -696,7 +699,7 @@ void DepsgraphNodeBuilder::build_collection(LayerCollection *from_layer_collecti
   const bool is_collection_restricted = (collection->flag & visibility_flag);
   const bool is_collection_visible = !is_collection_restricted && is_parent_collection_visible_;
   IDNode *id_node;
-  if (built_map_.checkIsBuiltAndTag(collection)) {
+  if (built_map_.check_is_built_and_tag(collection)) {
     id_node = find_id_node(&collection->id);
     if (is_collection_visible && id_node->is_visible_on_build == false &&
         id_node->is_collection_fully_expanded == true)
@@ -721,6 +724,7 @@ void DepsgraphNodeBuilder::build_collection(LayerCollection *from_layer_collecti
     add_operation_node(&collection->id, NodeType::HIERARCHY, OperationCode::HIERARCHY);
 
     build_idproperties(collection->id.properties);
+    build_idproperties(collection->id.system_properties);
     build_parameters(&collection->id);
     add_operation_node(&collection->id, NodeType::GEOMETRY, OperationCode::GEOMETRY_EVAL_DONE);
   }
@@ -751,7 +755,7 @@ void DepsgraphNodeBuilder::build_object(int base_index,
                                         eDepsNode_LinkedState_Type linked_state,
                                         bool is_visible)
 {
-  const bool has_object = built_map_.checkIsBuiltAndTag(object);
+  const bool has_object = built_map_.check_is_built_and_tag(object);
 
   /* When there is already object in the dependency graph accumulate visibility an linked state
    * flags. Only do it on the object itself (apart from very special cases) and leave dealing with
@@ -825,6 +829,7 @@ void DepsgraphNodeBuilder::build_object(int base_index,
    * from object's data. */
   build_parameters(&object->id);
   build_idproperties(object->id.properties);
+  build_idproperties(object->id.system_properties);
   /* Build animation data,
    *
    * Do it now because it's possible object data will affect
@@ -947,6 +952,12 @@ void DepsgraphNodeBuilder::build_object_modifiers(Object *object)
             Object *ob_eval = reinterpret_cast<Object *>(id_node->id_cow);
             ModifierData *md_eval = reinterpret_cast<ModifierData *>(
                 BLI_findlink(&ob_eval->modifiers, modifier_index));
+            if (!md_eval) {
+              /* The modifiers may not be available on the evaluated object if the object has an
+               * error that turned it into an Empty. Modifiers are not copied on this object type.
+               * Also see #142290. */
+              return;
+            }
             /* Set flag that the modifier can check when it is evaluated. */
             const bool is_user_modified = modifier_node->flag & DEPSOP_FLAG_USER_MODIFIED;
             SET_FLAG_FROM_TEST(md_eval->flag, is_user_modified, eModifierFlag_UserModified);
@@ -1013,7 +1024,7 @@ void DepsgraphNodeBuilder::build_object_data(Object *object)
       break;
     default: {
       ID *obdata = (ID *)object->data;
-      if (!built_map_.checkIsBuilt(obdata)) {
+      if (!built_map_.check_is_built(obdata)) {
         build_animdata(obdata);
       }
       break;
@@ -1260,11 +1271,7 @@ void DepsgraphNodeBuilder::build_animdata(ID *id)
     build_animdata_nlastrip_targets(&nlt->strips);
   }
   /* Drivers. */
-  int driver_index = 0;
-  LISTBASE_FOREACH (FCurve *, fcu, &adt->drivers) {
-    /* create driver */
-    build_driver(id, fcu, driver_index++);
-  }
+  build_animdata_drivers(id, adt);
 }
 
 void DepsgraphNodeBuilder::build_animdata_nlastrip_targets(ListBase *strips)
@@ -1304,13 +1311,40 @@ void DepsgraphNodeBuilder::build_animation_images(ID *id)
 
 void DepsgraphNodeBuilder::build_action(bAction *action)
 {
-  if (built_map_.checkIsBuiltAndTag(action)) {
+  if (built_map_.check_is_built_and_tag(action)) {
     return;
   }
+
+  /* To make it possible to use animation data as a variable for drivers: */
+  build_parameters(&action->id);
+
   build_idproperties(action->id.properties);
+  build_idproperties(action->id.system_properties);
   add_operation_node(&action->id, NodeType::ANIMATION, OperationCode::ANIMATION_EVAL);
 }
 
+void DepsgraphNodeBuilder::build_animdata_drivers(ID *id, AnimData *adt)
+{
+  bool needs_unshare = false;
+
+  /* Drivers. */
+  int driver_index;
+  LISTBASE_FOREACH_INDEX (FCurve *, fcu, &adt->drivers, driver_index) {
+    build_driver(id, fcu, driver_index);
+    needs_unshare = needs_unshare || data_path_maybe_shared(*id, fcu->rna_path);
+  }
+
+  if (!needs_unshare) {
+    return;
+  }
+
+  ID *id_cow = get_cow_id(id);
+  ensure_operation_node(
+      id, NodeType::PARAMETERS, OperationCode::DRIVER_UNSHARE, [id_cow](::Depsgraph *depsgraph) {
+        BKE_animsys_eval_driver_unshare(depsgraph, id_cow);
+      });
+}
+
 void DepsgraphNodeBuilder::build_driver(ID *id, FCurve *fcurve, int driver_index)
 {
   /* Create data node for this driver */
@@ -1457,7 +1491,7 @@ void DepsgraphNodeBuilder::build_dimensions(Object *object)
 
 void DepsgraphNodeBuilder::build_world(World *world)
 {
-  if (built_map_.checkIsBuiltAndTag(world)) {
+  if (built_map_.check_is_built_and_tag(world)) {
     return;
   }
   /* World itself. */
@@ -1470,6 +1504,7 @@ void DepsgraphNodeBuilder::build_world(World *world)
       OperationCode::WORLD_UPDATE,
       [world_cow](::Depsgraph *depsgraph) { BKE_world_eval(depsgraph, world_cow); });
   build_idproperties(world->id.properties);
+  build_idproperties(world->id.system_properties);
   /* Animation. */
   build_animdata(&world->id);
   build_parameters(&world->id);
@@ -1628,7 +1663,7 @@ void DepsgraphNodeBuilder::build_particle_systems(Object *object, bool is_object
 
 void DepsgraphNodeBuilder::build_particle_settings(ParticleSettings *particle_settings)
 {
-  if (built_map_.checkIsBuiltAndTag(particle_settings)) {
+  if (built_map_.check_is_built_and_tag(particle_settings)) {
     return;
   }
   /* Make sure we've got proper copied ID pointer. */
@@ -1662,10 +1697,11 @@ void DepsgraphNodeBuilder::build_particle_settings(ParticleSettings *particle_se
 
 void DepsgraphNodeBuilder::build_shapekeys(Key *key)
 {
-  if (built_map_.checkIsBuiltAndTag(key)) {
+  if (built_map_.check_is_built_and_tag(key)) {
     return;
   }
   build_idproperties(key->id.properties);
+  build_idproperties(key->id.system_properties);
   build_animdata(&key->id);
   build_parameters(&key->id);
   /* This is an exit operation for the entire key datablock, is what is used
@@ -1715,7 +1751,7 @@ void DepsgraphNodeBuilder::build_object_data_geometry(Object *object)
 
 void DepsgraphNodeBuilder::build_object_data_geometry_datablock(ID *obdata)
 {
-  if (built_map_.checkIsBuiltAndTag(obdata)) {
+  if (built_map_.check_is_built_and_tag(obdata)) {
     return;
   }
   OperationNode *op_node;
@@ -1723,6 +1759,7 @@ void DepsgraphNodeBuilder::build_object_data_geometry_datablock(ID *obdata)
   (void)add_id_node(obdata);
   ID *obdata_cow = get_cow_id(obdata);
   build_idproperties(obdata->properties);
+  build_idproperties(obdata->system_properties);
   /* Animation. */
   build_animdata(obdata);
   /* ShapeKeys */
@@ -1840,10 +1877,11 @@ void DepsgraphNodeBuilder::build_object_data_geometry_datablock(ID *obdata)
 
 void DepsgraphNodeBuilder::build_armature(bArmature *armature)
 {
-  if (built_map_.checkIsBuiltAndTag(armature)) {
+  if (built_map_.check_is_built_and_tag(armature)) {
     return;
   }
   build_idproperties(armature->id.properties);
+  build_idproperties(armature->id.system_properties);
   build_animdata(&armature->id);
   build_parameters(&armature->id);
   /* This operation is no longer necessary, as it was updating things with the bone layers (which
@@ -1862,6 +1900,7 @@ void DepsgraphNodeBuilder::build_armature_bones(ListBase *bones)
 {
   LISTBASE_FOREACH (Bone *, bone, bones) {
     build_idproperties(bone->prop);
+    build_idproperties(bone->system_properties);
     build_armature_bones(&bone->childbase);
   }
 }
@@ -1871,15 +1910,17 @@ void DepsgraphNodeBuilder::build_armature_bone_collections(
 {
   for (BoneCollection *bcoll : collections) {
     build_idproperties(bcoll->prop);
+    build_idproperties(bcoll->system_properties);
   }
 }
 
 void DepsgraphNodeBuilder::build_camera(Camera *camera)
 {
-  if (built_map_.checkIsBuiltAndTag(camera)) {
+  if (built_map_.check_is_built_and_tag(camera)) {
     return;
   }
   build_idproperties(camera->id.properties);
+  build_idproperties(camera->id.system_properties);
   build_animdata(&camera->id);
   build_parameters(&camera->id);
   if (camera->dof.focus_object != nullptr) {
@@ -1889,10 +1930,11 @@ void DepsgraphNodeBuilder::build_camera(Camera *camera)
 
 void DepsgraphNodeBuilder::build_light(Light *lamp)
 {
-  if (built_map_.checkIsBuiltAndTag(lamp)) {
+  if (built_map_.check_is_built_and_tag(lamp)) {
     return;
   }
   build_idproperties(lamp->id.properties);
+  build_idproperties(lamp->id.system_properties);
   build_animdata(&lamp->id);
   build_parameters(&lamp->id);
   /* light's nodetree */
@@ -1931,7 +1973,7 @@ void DepsgraphNodeBuilder::build_nodetree(bNodeTree *ntree)
   if (ntree == nullptr) {
     return;
   }
-  if (built_map_.checkIsBuiltAndTag(ntree)) {
+  if (built_map_.check_is_built_and_tag(ntree)) {
     return;
   }
   /* nodetree itself */
@@ -1939,6 +1981,7 @@ void DepsgraphNodeBuilder::build_nodetree(bNodeTree *ntree)
   /* General parameters. */
   build_parameters(&ntree->id);
   build_idproperties(ntree->id.properties);
+  build_idproperties(ntree->id.system_properties);
   /* Animation, */
   build_animdata(&ntree->id);
   /* Output update. */
@@ -2035,7 +2078,7 @@ void DepsgraphNodeBuilder::build_nodetree(bNodeTree *ntree)
 
 void DepsgraphNodeBuilder::build_material(Material *material)
 {
-  if (built_map_.checkIsBuiltAndTag(material)) {
+  if (built_map_.check_is_built_and_tag(material)) {
     return;
   }
   /* Material itself. */
@@ -2048,6 +2091,7 @@ void DepsgraphNodeBuilder::build_material(Material *material)
       OperationCode::MATERIAL_UPDATE,
       [material_cow](::Depsgraph *depsgraph) { BKE_material_eval(depsgraph, material_cow); });
   build_idproperties(material->id.properties);
+  build_idproperties(material->id.system_properties);
   /* Material animation. */
   build_animdata(&material->id);
   build_parameters(&material->id);
@@ -2067,12 +2111,14 @@ void DepsgraphNodeBuilder::build_materials(Material **materials, int num_materia
 
 void DepsgraphNodeBuilder::build_texture(Tex *texture)
 {
-  if (built_map_.checkIsBuiltAndTag(texture)) {
+  if (built_map_.check_is_built_and_tag(texture)) {
     return;
   }
   /* Texture itself. */
   add_id_node(&texture->id);
+  Tex *texture_cow = get_cow_datablock(texture);
   build_idproperties(texture->id.properties);
+  build_idproperties(texture->id.system_properties);
   build_animdata(&texture->id);
   build_parameters(&texture->id);
   /* Texture's nodetree. */
@@ -2083,30 +2129,36 @@ void DepsgraphNodeBuilder::build_texture(Tex *texture)
       build_image(texture->ima);
     }
   }
-  add_operation_node(
-      &texture->id, NodeType::GENERIC_DATABLOCK, OperationCode::GENERIC_DATABLOCK_UPDATE);
+  add_operation_node(&texture->id,
+                     NodeType::GENERIC_DATABLOCK,
+                     OperationCode::GENERIC_DATABLOCK_UPDATE,
+                     [texture_cow](::Depsgraph *depsgraph) {
+                       texture_cow->runtime.last_update = DEG_get_update_count(depsgraph);
+                     });
 }
 
 void DepsgraphNodeBuilder::build_image(Image *image)
 {
-  if (built_map_.checkIsBuiltAndTag(image)) {
+  if (built_map_.check_is_built_and_tag(image)) {
     return;
   }
   build_parameters(&image->id);
   build_idproperties(image->id.properties);
+  build_idproperties(image->id.system_properties);
   add_operation_node(
       &image->id, NodeType::GENERIC_DATABLOCK, OperationCode::GENERIC_DATABLOCK_UPDATE);
 }
 
 void DepsgraphNodeBuilder::build_cachefile(CacheFile *cache_file)
 {
-  if (built_map_.checkIsBuiltAndTag(cache_file)) {
+  if (built_map_.check_is_built_and_tag(cache_file)) {
     return;
   }
   ID *cache_file_id = &cache_file->id;
   add_id_node(cache_file_id);
   CacheFile *cache_file_cow = get_cow_datablock(cache_file);
   build_idproperties(cache_file_id->properties);
+  build_idproperties(cache_file_id->system_properties);
   /* Animation, */
   build_animdata(cache_file_id);
   build_parameters(cache_file_id);
@@ -2121,12 +2173,13 @@ void DepsgraphNodeBuilder::build_cachefile(CacheFile *cache_file)
 
 void DepsgraphNodeBuilder::build_mask(Mask *mask)
 {
-  if (built_map_.checkIsBuiltAndTag(mask)) {
+  if (built_map_.check_is_built_and_tag(mask)) {
     return;
   }
   ID *mask_id = &mask->id;
   Mask *mask_cow = (Mask *)ensure_cow_id(mask_id);
   build_idproperties(mask->id.properties);
+  build_idproperties(mask->id.system_properties);
   /* F-Curve based animation. */
   build_animdata(mask_id);
   build_parameters(mask_id);
@@ -2158,25 +2211,27 @@ void DepsgraphNodeBuilder::build_mask(Mask *mask)
 
 void DepsgraphNodeBuilder::build_freestyle_linestyle(FreestyleLineStyle *linestyle)
 {
-  if (built_map_.checkIsBuiltAndTag(linestyle)) {
+  if (built_map_.check_is_built_and_tag(linestyle)) {
     return;
   }
 
   ID *linestyle_id = &linestyle->id;
   build_parameters(linestyle_id);
   build_idproperties(linestyle->id.properties);
+  build_idproperties(linestyle->id.system_properties);
   build_animdata(linestyle_id);
   build_nodetree(linestyle->nodetree);
 }
 
 void DepsgraphNodeBuilder::build_movieclip(MovieClip *clip)
 {
-  if (built_map_.checkIsBuiltAndTag(clip)) {
+  if (built_map_.check_is_built_and_tag(clip)) {
     return;
   }
   ID *clip_id = &clip->id;
   MovieClip *clip_cow = (MovieClip *)ensure_cow_id(clip_id);
   build_idproperties(clip_id->properties);
+  build_idproperties(clip_id->system_properties);
   /* Animation. */
   build_animdata(clip_id);
   build_parameters(clip_id);
@@ -2191,24 +2246,26 @@ void DepsgraphNodeBuilder::build_movieclip(MovieClip *clip)
 
 void DepsgraphNodeBuilder::build_lightprobe(LightProbe *probe)
 {
-  if (built_map_.checkIsBuiltAndTag(probe)) {
+  if (built_map_.check_is_built_and_tag(probe)) {
     return;
   }
   /* Placeholder so we can add relations and tag ID node for update. */
   add_operation_node(&probe->id, NodeType::PARAMETERS, OperationCode::LIGHT_PROBE_EVAL);
   build_idproperties(probe->id.properties);
+  build_idproperties(probe->id.system_properties);
   build_animdata(&probe->id);
   build_parameters(&probe->id);
 }
 
 void DepsgraphNodeBuilder::build_speaker(Speaker *speaker)
 {
-  if (built_map_.checkIsBuiltAndTag(speaker)) {
+  if (built_map_.check_is_built_and_tag(speaker)) {
     return;
   }
   /* Placeholder so we can add relations and tag ID node for update. */
   add_operation_node(&speaker->id, NodeType::AUDIO, OperationCode::SPEAKER_EVAL);
   build_idproperties(speaker->id.properties);
+  build_idproperties(speaker->id.system_properties);
   build_animdata(&speaker->id);
   build_parameters(&speaker->id);
   if (speaker->sound != nullptr) {
@@ -2218,7 +2275,7 @@ void DepsgraphNodeBuilder::build_speaker(Speaker *speaker)
 
 void DepsgraphNodeBuilder::build_sound(bSound *sound)
 {
-  if (built_map_.checkIsBuiltAndTag(sound)) {
+  if (built_map_.check_is_built_and_tag(sound)) {
     return;
   }
   add_id_node(&sound->id);
@@ -2230,17 +2287,19 @@ void DepsgraphNodeBuilder::build_sound(bSound *sound)
                        BKE_sound_evaluate(depsgraph, bmain, sound_cow);
                      });
   build_idproperties(sound->id.properties);
+  build_idproperties(sound->id.system_properties);
   build_animdata(&sound->id);
   build_parameters(&sound->id);
 }
 
 void DepsgraphNodeBuilder::build_vfont(VFont *vfont)
 {
-  if (built_map_.checkIsBuiltAndTag(vfont)) {
+  if (built_map_.check_is_built_and_tag(vfont)) {
     return;
   }
   build_parameters(&vfont->id);
   build_idproperties(vfont->id.properties);
+  build_idproperties(vfont->id.system_properties);
   add_operation_node(
       &vfont->id, NodeType::GENERIC_DATABLOCK, OperationCode::GENERIC_DATABLOCK_UPDATE);
 }
@@ -2249,6 +2308,7 @@ static bool strip_node_build_cb(Strip *strip, void *user_data)
 {
   DepsgraphNodeBuilder *nb = (DepsgraphNodeBuilder *)user_data;
   nb->build_idproperties(strip->prop);
+  nb->build_idproperties(strip->system_properties);
   if (strip->sound != nullptr) {
     nb->build_sound(strip->sound);
   }
@@ -2262,6 +2322,18 @@ static bool strip_node_build_cb(Strip *strip, void *user_data)
     ViewLayer *sequence_view_layer = BKE_view_layer_default_render(strip->scene);
     nb->build_scene_speakers(strip->scene, sequence_view_layer);
   }
+  LISTBASE_FOREACH (StripModifierData *, modifier, &strip->modifiers) {
+    if (modifier->type != eSeqModifierType_Compositor) {
+      continue;
+    }
+
+    const SequencerCompositorModifierData *modifier_data =
+        reinterpret_cast<SequencerCompositorModifierData *>(modifier);
+    if (!modifier_data->node_group) {
+      continue;
+    }
+    nb->build_nodetree(modifier_data->node_group);
+  }
   /* TODO(sergey): Movie clip, scene, camera, mask. */
   return true;
 }
@@ -2271,7 +2343,7 @@ void DepsgraphNodeBuilder::build_scene_sequencer(Scene *scene)
   if (scene->ed == nullptr) {
     return;
   }
-  if (built_map_.checkIsBuiltAndTag(scene, BuilderMap::TAG_SCENE_SEQUENCER)) {
+  if (built_map_.check_is_built_and_tag(scene, BuilderMap::TAG_SCENE_SEQUENCER)) {
     return;
   }
   build_scene_audio(scene);
@@ -2280,15 +2352,15 @@ void DepsgraphNodeBuilder::build_scene_sequencer(Scene *scene)
                      NodeType::SEQUENCER,
                      OperationCode::SEQUENCES_EVAL,
                      [scene_cow](::Depsgraph *depsgraph) {
-                       SEQ_eval_sequences(depsgraph, scene_cow, &scene_cow->ed->seqbase);
+                       seq::eval_strips(depsgraph, scene_cow, &scene_cow->ed->seqbase);
                      });
   /* Make sure data for sequences is in the graph. */
-  SEQ_for_each_callback(&scene->ed->seqbase, strip_node_build_cb, this);
+  seq::foreach_strip(&scene->ed->seqbase, strip_node_build_cb, this);
 }
 
 void DepsgraphNodeBuilder::build_scene_audio(Scene *scene)
 {
-  if (built_map_.checkIsBuiltAndTag(scene, BuilderMap::TAG_SCENE_AUDIO)) {
+  if (built_map_.check_is_built_and_tag(scene, BuilderMap::TAG_SCENE_AUDIO)) {
     return;
   }
 
