@@ -8,6 +8,11 @@
  * LUT generation module.
  */
 
+#include "GPU_state.hh"
+
+#include "draw_pass.hh"
+
+#include "eevee_defines.hh"
 #include "eevee_precompute.hh"
 
 namespace blender::eevee {
@@ -18,9 +23,9 @@ Precompute::Precompute(draw::Manager &manager, PrecomputeType type, int3 table_e
 
   eGPUTextureUsage usage = GPU_TEXTURE_USAGE_SHADER_WRITE | GPU_TEXTURE_USAGE_HOST_READ;
   Texture table_tx = {"Precompute"};
-  table_tx.ensure_3d(GPU_RGBA32F, table_extent, usage);
+  table_tx.ensure_3d(gpu::TextureFormat::SFLOAT_32_32_32_32, table_extent, usage);
 
-  GPUShader *shader = GPU_shader_create_from_info_name("eevee_lut");
+  gpu::Shader *shader = GPU_shader_create_from_info_name("eevee_lut");
 
   PassSimple lut_ps = {"Precompute"};
   lut_ps.shader_set(shader);
