@@ -12,8 +12,10 @@
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
+#include "DNA_sequence_types.h"
 #include "DNA_space_types.h"
 #include "DNA_userdef_types.h"
+#include "DNA_workspace_types.h"
 
 #include "BLI_listbase.h"
 #include "BLI_math_rotation.h"
@@ -31,6 +33,7 @@
 #include "ED_anim_api.hh"
 #include "ED_keyframes_edit.hh"
 #include "ED_keyframes_keylist.hh"
+#include "ED_sequencer.hh"
 
 #include "RNA_access.hh"
 #include "RNA_path.hh"
@@ -41,6 +44,10 @@
 #include "GPU_immediate.hh"
 #include "GPU_state.hh"
 
+#include "SEQ_time.hh"
+
+#include <utility>
+
 /* *************************************************** */
 /* CURRENT FRAME DRAWING */
 
@@ -54,7 +61,7 @@ void ANIM_draw_cfra(const bContext *C, View2D *v2d, short flag)
   GPU_line_width((flag & DRAWCFRA_WIDE) ? 3.0 : 2.0);
 
   GPUVertFormat *format = immVertexFormat();
-  uint pos = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+  uint pos = GPU_vertformat_attr_add(format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
 
   immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
 
@@ -79,7 +86,7 @@ void ANIM_draw_previewrange(const Scene *scene, View2D *v2d, int end_frame_width
     GPU_blend(GPU_BLEND_ALPHA);
 
     GPUVertFormat *format = immVertexFormat();
-    uint pos = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+    uint pos = GPU_vertformat_attr_add(format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
 
     immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
     immUniformThemeColorShadeAlpha(TH_ANIM_PREVIEW_RANGE, -25, -30);
@@ -103,6 +110,61 @@ void ANIM_draw_previewrange(const Scene *scene, View2D *v2d, int end_frame_width
   }
 }
 
+void ANIM_draw_scene_strip_range(const bContext *C, View2D *v2d)
+{
+  using namespace blender;
+  SpaceAction *space_action = CTX_wm_space_action(C);
+  if (!space_action || (space_action->overlays.flag & ADS_OVERLAY_SHOW_OVERLAYS) == 0 ||
+      (space_action->overlays.flag & ADS_SHOW_SCENE_STRIP_FRAME_RANGE) == 0)
+  {
+    return;
+  }
+  WorkSpace *workspace = CTX_wm_workspace(C);
+  if (!workspace) {
+    return;
+  }
+  if ((workspace->flags & WORKSPACE_SYNC_SCENE_TIME) == 0) {
+    return;
+  }
+  const Scene *sequencer_scene = workspace->sequencer_scene;
+  if (!sequencer_scene) {
+    return;
+  }
+  const Strip *scene_strip = blender::ed::vse::get_scene_strip_for_time_sync(sequencer_scene);
+  if (!scene_strip || !scene_strip->scene) {
+    return;
+  }
+  GPU_blend(GPU_BLEND_ALPHA);
+
+  GPUVertFormat *format = immVertexFormat();
+  uint pos = GPU_vertformat_attr_add(format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
+
+  immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
+  immUniformThemeColorShadeAlpha(TH_ANIM_SCENE_STRIP_RANGE, -25, -30);
+
+  /* ..._handle are frames in "sequencer logic", meaning that on the right_handle point in time,
+   * the strip is not visible any more. The last visible frame of the strip is actually on
+   * (right_handle-1), hence the -1 when computing the end_frame. */
+  const float left_handle = seq::time_left_handle_frame_get(sequencer_scene, scene_strip);
+  const float right_handle = seq::time_right_handle_frame_get(sequencer_scene, scene_strip);
+  float start_frame = seq::give_frame_index(sequencer_scene, scene_strip, left_handle) +
+                      scene_strip->scene->r.sfra;
+  float end_frame = seq::give_frame_index(sequencer_scene, scene_strip, right_handle - 1) +
+                    scene_strip->scene->r.sfra;
+
+  /* This can happen when the strip time is reversed. */
+  if (start_frame > end_frame) {
+    std::swap(start_frame, end_frame);
+  }
+
+  immRectf(pos, v2d->cur.xmin, v2d->cur.ymin, start_frame, v2d->cur.ymax);
+  immRectf(pos, end_frame, v2d->cur.ymin, v2d->cur.xmax, v2d->cur.ymax);
+
+  immUnbindProgram();
+
+  GPU_blend(GPU_BLEND_NONE);
+}
+
 /* *************************************************** */
 /* SCENE FRAME RANGE */
 
@@ -112,7 +174,7 @@ void ANIM_draw_framerange(Scene *scene, View2D *v2d)
   GPU_blend(GPU_BLEND_ALPHA);
 
   GPUVertFormat *format = immVertexFormat();
-  uint pos = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+  uint pos = GPU_vertformat_attr_add(format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
 
   immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
   immUniformThemeColorShadeAlpha(TH_BACK, -25, -100);
@@ -164,7 +226,7 @@ void ANIM_draw_action_framerange(
   GPU_blend(GPU_BLEND_ALPHA);
 
   GPUVertFormat *format = immVertexFormat();
-  uint pos = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+  uint pos = GPU_vertformat_attr_add(format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
 
   immBindBuiltinProgram(GPU_SHADER_2D_DIAG_STRIPES);
 
@@ -569,30 +631,40 @@ float ANIM_unit_mapping_get_factor(Scene *scene, ID *id, FCurve *fcu, short flag
     *r_offset = 0.0f;
   }
 
-  /* sanity checks */
-  if (id && fcu && fcu->rna_path) {
-    PointerRNA ptr;
-    PropertyRNA *prop;
+  /* TODO: change the pointer parameters to references, as this function should not be called
+   * without an animated ID or a scene (to get the preferred units). */
 
-    /* get RNA property that F-Curve affects */
-    PointerRNA id_ptr = RNA_id_pointer_create(id);
-    if (RNA_path_resolve_property(&id_ptr, fcu->rna_path, &ptr, &prop)) {
-      /* rotations: radians <-> degrees? */
-      if (RNA_SUBTYPE_UNIT(RNA_property_subtype(prop)) == PROP_UNIT_ROTATION) {
-        /* if the radians flag is not set, default to using degrees which need conversions */
-        if ((scene) && (scene->unit.system_rotation == USER_UNIT_ROT_RADIANS) == 0) {
-          if (flag & ANIM_UNITCONV_RESTORE) {
-            return DEG2RADF(1.0f); /* degrees to radians */
-          }
-          return RAD2DEGF(1.0f); /* radians to degrees */
-        }
+  if (!id || !fcu || !fcu->rna_path || !scene) {
+    /* Not enough information to do the remapping, so just show the data as-is. */
+    return 1.0f;
+  }
+
+  PointerRNA ptr;
+  PropertyRNA *prop;
+  PointerRNA id_ptr = RNA_id_pointer_create(id);
+  if (!RNA_path_resolve_property(&id_ptr, fcu->rna_path, &ptr, &prop)) {
+    /* Without resolving the property, its type & subtype are unknown; remapping is impossible. */
+    return 1.0f;
+  }
+
+  const PropertyUnit prop_unit = PropertyUnit(RNA_SUBTYPE_UNIT(RNA_property_subtype(prop)));
+
+  switch (prop_unit) {
+    case PROP_UNIT_ROTATION:
+      if (scene->unit.system_rotation == USER_UNIT_ROT_RADIANS) {
+        return 1.0f;
+      }
+
+      if (flag & ANIM_UNITCONV_RESTORE) {
+        return DEG2RADF(1.0f);
       }
+      return RAD2DEGF(1.0f);
 
+    default:
       /* TODO: other rotation types here as necessary */
-    }
+      break;
   }
 
-  /* no mapping needs to occur... */
   return 1.0f;
 }
 
@@ -703,7 +775,7 @@ void ANIM_center_frame(bContext *C, int smooth_viewtx)
 
   switch (U.view_frame_type) {
     case ZOOM_FRAME_MODE_SECONDS: {
-      const float fps = FPS;
+      const float fps = scene->frames_per_second();
       newrct.xmax = scene->r.cfra + U.view_frame_seconds * fps + 1;
       newrct.xmin = scene->r.cfra - U.view_frame_seconds * fps - 1;
       newrct.ymax = region->v2d.cur.ymax;
