@@ -10,6 +10,7 @@
 #include "BKE_curveprofile.h"
 #include "BKE_library.hh"
 
+#include "BLI_math_base.h"
 #include "BLI_rect.h"
 #include "BLI_string_ref.hh"
 
@@ -21,7 +22,7 @@
 #include "RNA_access.hh"
 #include "RNA_prototypes.hh"
 
-#include "UI_interface.hh"
+#include "UI_interface_layout.hh"
 #include "interface_intern.hh"
 #include "interface_templates_intern.hh"
 
@@ -49,7 +50,7 @@ static uiBlock *curve_profile_presets_fn(bContext *C, ARegion *region, void *cb_
                                                    PROF_PRESET_STEPS)})
   {
     uiBut *but = uiDefIconTextBut(block,
-                                  UI_BTYPE_BUT_MENU,
+                                  ButType::ButMenu,
                                   1,
                                   ICON_BLANK1,
                                   item.first,
@@ -58,8 +59,6 @@ static uiBlock *curve_profile_presets_fn(bContext *C, ARegion *region, void *cb_
                                   0,
                                   UI_UNIT_Y,
                                   nullptr,
-                                  0.0,
-                                  0.0,
                                   "");
     const eCurveProfilePresets preset = item.second;
     UI_but_func_set(but, [profile, cb, preset](bContext &C) {
@@ -89,7 +88,7 @@ static uiBlock *curve_profile_tools_fn(bContext *C, ARegion *region, void *cb_v)
 
   {
     uiBut *but = uiDefIconTextBut(block,
-                                  UI_BTYPE_BUT_MENU,
+                                  ButType::ButMenu,
                                   1,
                                   ICON_BLANK1,
                                   IFACE_("Reset View"),
@@ -98,8 +97,6 @@ static uiBlock *curve_profile_tools_fn(bContext *C, ARegion *region, void *cb_v)
                                   0,
                                   UI_UNIT_Y,
                                   nullptr,
-                                  0.0,
-                                  0.0,
                                   "");
     UI_but_func_set(but, [profile](bContext &C) {
       BKE_curveprofile_reset_view(profile);
@@ -108,7 +105,7 @@ static uiBlock *curve_profile_tools_fn(bContext *C, ARegion *region, void *cb_v)
   }
   {
     uiBut *but = uiDefIconTextBut(block,
-                                  UI_BTYPE_BUT_MENU,
+                                  ButType::ButMenu,
                                   1,
                                   ICON_BLANK1,
                                   IFACE_("Reset Curve"),
@@ -117,8 +114,6 @@ static uiBlock *curve_profile_tools_fn(bContext *C, ARegion *region, void *cb_v)
                                   0,
                                   UI_UNIT_Y,
                                   nullptr,
-                                  0.0,
-                                  0.0,
                                   "");
     UI_but_func_set(but, [profile, cb](bContext &C) {
       BKE_curveprofile_reset(profile);
@@ -207,11 +202,11 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
   CurveProfile *profile = static_cast<CurveProfile *>(ptr->data);
   uiBut *bt;
 
-  uiBlock *block = uiLayoutGetBlock(layout);
+  uiBlock *block = layout->block();
 
   UI_block_emboss_set(block, blender::ui::EmbossType::Emboss);
 
-  uiLayoutSetPropSep(layout, false);
+  layout->use_property_split_set(false);
 
   /* Preset selector */
   /* There is probably potential to use simpler "uiLayout::prop" functions here, but automatic
@@ -241,7 +236,7 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
     /* Only for dynamic presets. */
     if (ELEM(profile->preset, PROF_PRESET_STEPS, PROF_PRESET_SUPPORTS)) {
       bt = uiDefIconTextBut(block,
-                            UI_BTYPE_BUT,
+                            ButType::But,
                             0,
                             ICON_NONE,
                             IFACE_("Apply Preset"),
@@ -250,8 +245,6 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
                             UI_UNIT_X,
                             UI_UNIT_X,
                             nullptr,
-                            0.0,
-                            0.0,
                             TIP_("Reapply and update the preset, removing changes"));
       UI_but_func_set(bt, [profile, cb](bContext &C) {
         BKE_curveprofile_reset(profile);
@@ -265,11 +258,11 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
 
   /* (Left aligned) */
   uiLayout *sub = &row->row(true);
-  uiLayoutSetAlignment(sub, UI_LAYOUT_ALIGN_LEFT);
+  sub->alignment_set(blender::ui::LayoutAlign::Left);
 
   /* Zoom in */
   bt = uiDefIconBut(block,
-                    UI_BTYPE_BUT,
+                    ButType::But,
                     0,
                     ICON_ZOOM_IN,
                     0,
@@ -287,7 +280,7 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
 
   /* Zoom out */
   bt = uiDefIconBut(block,
-                    UI_BTYPE_BUT,
+                    ButType::But,
                     0,
                     ICON_ZOOM_OUT,
                     0,
@@ -305,11 +298,11 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
 
   /* (Right aligned) */
   sub = &row->row(true);
-  uiLayoutSetAlignment(sub, UI_LAYOUT_ALIGN_RIGHT);
+  sub->alignment_set(blender::ui::LayoutAlign::Right);
 
   /* Flip path */
   bt = uiDefIconBut(block,
-                    UI_BTYPE_BUT,
+                    ButType::But,
                     0,
                     ICON_ARROW_LEFTRIGHT,
                     0,
@@ -329,7 +322,7 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
   /* Clipping toggle */
   const int icon = (profile->flag & PROF_USE_CLIP) ? ICON_CLIPUV_HLT : ICON_CLIPUV_DEHLT;
   bt = uiDefIconBut(block,
-                    UI_BTYPE_BUT,
+                    ButType::But,
                     0,
                     icon,
                     0,
@@ -375,12 +368,12 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
                      but_func_argN_copy<RNAUpdateCb>);
 
   /* The path itself */
-  int path_width = max_ii(uiLayoutGetWidth(layout), UI_UNIT_X);
+  int path_width = max_ii(layout->width(), UI_UNIT_X);
   path_width = min_ii(path_width, int(16.0f * UI_UNIT_X));
   const int path_height = path_width;
   layout->row(false);
   uiDefBut(block,
-           UI_BTYPE_CURVEPROFILE,
+           ButType::CurveProfile,
            0,
            "",
            0,
@@ -445,7 +438,7 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
 
     /* Position */
     bt = uiDefButF(block,
-                   UI_BTYPE_NUM,
+                   ButType::Num,
                    0,
                    "X:",
                    0,
@@ -466,7 +459,7 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
       UI_but_flag_enable(bt, UI_BUT_DISABLED);
     }
     bt = uiDefButF(block,
-                   UI_BTYPE_NUM,
+                   ButType::Num,
                    0,
                    "Y:",
                    0,
@@ -489,7 +482,7 @@ static void CurveProfile_buttons_layout(uiLayout *layout, PointerRNA *ptr, const
 
     /* Delete points */
     bt = uiDefIconBut(block,
-                      UI_BTYPE_BUT,
+                      ButType::But,
                       0,
                       ICON_X,
                       0,
@@ -520,7 +513,7 @@ void uiTemplateCurveProfile(uiLayout *layout, PointerRNA *ptr, const StringRefNu
 {
   PropertyRNA *prop = RNA_struct_find_property(ptr, propname.c_str());
 
-  uiBlock *block = uiLayoutGetBlock(layout);
+  uiBlock *block = layout->block();
 
   if (!prop) {
     RNA_warning("Curve Profile property not found: %s.%s",
