@@ -15,6 +15,7 @@
 #include "BKE_material.hh"
 #include "BKE_object_deform.h"
 #include "BKE_paint.hh"
+#include "BKE_paint_types.hh"
 #include "BKE_report.hh"
 #include "BKE_screen.hh"
 
@@ -39,6 +40,7 @@
 
 #include "DEG_depsgraph_query.hh"
 
+#include "GEO_curves_remove_and_split.hh"
 #include "GEO_join_geometries.hh"
 #include "GEO_smooth_curves.hh"
 
@@ -110,10 +112,10 @@ static std::unique_ptr<GreasePencilStrokeOperation> get_stroke_operation(bContex
       case GPAINT_BRUSH_TYPE_ERASE:
         return greasepencil::new_erase_operation();
       case GPAINT_BRUSH_TYPE_FILL:
-        /* Fill tool keymap uses the paint operator as alternative mode. */
-        return greasepencil::new_paint_operation(true);
+        /* Fill tool keymap uses the paint operator to draw fill guides. */
+        return greasepencil::new_paint_operation(/* do_fill_guides = */ true);
       case GPAINT_BRUSH_TYPE_TINT:
-        return greasepencil::new_tint_operation();
+        return greasepencil::new_tint_operation(stroke_mode == BRUSH_STROKE_ERASE);
     }
   }
   else if (mode == PaintMode::SculptGPencil) {
@@ -235,7 +237,9 @@ static bool grease_pencil_brush_stroke_poll(bContext *C)
   return true;
 }
 
-static int grease_pencil_brush_stroke_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus grease_pencil_brush_stroke_invoke(bContext *C,
+                                                          wmOperator *op,
+                                                          const wmEvent *event)
 {
   if (event->tablet.active == EVT_TABLET_ERASER) {
     RNA_enum_set(op->ptr, "mode", BRUSH_STROKE_ERASE);
@@ -265,10 +269,10 @@ static int grease_pencil_brush_stroke_invoke(bContext *C, wmOperator *op, const
     }
     return false;
   }();
-  int return_value = ed::greasepencil::grease_pencil_draw_operator_invoke(
+  wmOperatorStatus retval = ed::greasepencil::grease_pencil_draw_operator_invoke(
       C, op, use_duplicate_previous_key);
-  if (return_value != OPERATOR_RUNNING_MODAL) {
-    return return_value;
+  if (retval != OPERATOR_RUNNING_MODAL) {
+    return retval;
   }
 
   op->customdata = paint_stroke_new(C,
@@ -280,8 +284,10 @@ static int grease_pencil_brush_stroke_invoke(bContext *C, wmOperator *op, const
                                     stroke_done,
                                     event->type);
 
-  return_value = op->type->modal(C, op, event);
-  if (return_value == OPERATOR_FINISHED) {
+  retval = op->type->modal(C, op, event);
+  OPERATOR_RETVAL_CHECK(retval);
+
+  if (retval == OPERATOR_FINISHED) {
     return OPERATOR_FINISHED;
   }
 
@@ -289,7 +295,9 @@ static int grease_pencil_brush_stroke_invoke(bContext *C, wmOperator *op, const
   return OPERATOR_RUNNING_MODAL;
 }
 
-static int grease_pencil_brush_stroke_modal(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus grease_pencil_brush_stroke_modal(bContext *C,
+                                                         wmOperator *op,
+                                                         const wmEvent *event)
 {
   return paint_stroke_modal(C, op, event, reinterpret_cast<PaintStroke **>(&op->customdata));
 }
@@ -332,7 +340,9 @@ static bool grease_pencil_sculpt_paint_poll(bContext *C)
   return true;
 }
 
-static int grease_pencil_sculpt_paint_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus grease_pencil_sculpt_paint_invoke(bContext *C,
+                                                          wmOperator *op,
+                                                          const wmEvent *event)
 {
   const Scene *scene = CTX_data_scene(C);
   const Object *object = CTX_data_active_object(C);
@@ -341,8 +351,8 @@ static int grease_pencil_sculpt_paint_invoke(bContext *C, wmOperator *op, const
   }
 
   GreasePencil &grease_pencil = *static_cast<GreasePencil *>(object->data);
-  if (!grease_pencil.has_active_layer()) {
-    BKE_report(op->reports, RPT_ERROR, "No active Grease Pencil layer");
+  if (!ed::greasepencil::has_editable_layer(grease_pencil)) {
+    BKE_report(op->reports, RPT_ERROR, "No editable Grease Pencil layer");
     return OPERATOR_CANCELLED;
   }
 
@@ -352,12 +362,6 @@ static int grease_pencil_sculpt_paint_invoke(bContext *C, wmOperator *op, const
     return OPERATOR_CANCELLED;
   }
 
-  bke::greasepencil::Layer &active_layer = *grease_pencil.get_active_layer();
-  if (!active_layer.is_editable()) {
-    BKE_report(op->reports, RPT_ERROR, "Active layer is locked or hidden");
-    return OPERATOR_CANCELLED;
-  }
-
   /* Ensure a drawing at the current keyframe. */
   bool inserted_keyframe = false;
   /* For the sculpt tools, we don't want the auto-key to create an empty keyframe, so we duplicate
@@ -386,8 +390,10 @@ static int grease_pencil_sculpt_paint_invoke(bContext *C, wmOperator *op, const
                                     stroke_done,
                                     event->type);
 
-  const int return_value = op->type->modal(C, op, event);
-  if (return_value == OPERATOR_FINISHED) {
+  const wmOperatorStatus retval = op->type->modal(C, op, event);
+  OPERATOR_RETVAL_CHECK(retval);
+
+  if (retval == OPERATOR_FINISHED) {
     return OPERATOR_FINISHED;
   }
 
@@ -395,7 +401,9 @@ static int grease_pencil_sculpt_paint_invoke(bContext *C, wmOperator *op, const
   return OPERATOR_RUNNING_MODAL;
 }
 
-static int grease_pencil_sculpt_paint_modal(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus grease_pencil_sculpt_paint_modal(bContext *C,
+                                                         wmOperator *op,
+                                                         const wmEvent *event)
 {
   return paint_stroke_modal(C, op, event, reinterpret_cast<PaintStroke **>(&op->customdata));
 }
@@ -438,9 +446,9 @@ static bool grease_pencil_weight_brush_stroke_poll(bContext *C)
   return true;
 }
 
-static int grease_pencil_weight_brush_stroke_invoke(bContext *C,
-                                                    wmOperator *op,
-                                                    const wmEvent *event)
+static wmOperatorStatus grease_pencil_weight_brush_stroke_invoke(bContext *C,
+                                                                 wmOperator *op,
+                                                                 const wmEvent *event)
 {
   const Scene *scene = CTX_data_scene(C);
   const Object *object = CTX_data_active_object(C);
@@ -477,8 +485,10 @@ static int grease_pencil_weight_brush_stroke_invoke(bContext *C,
                                     stroke_done,
                                     event->type);
 
-  const int return_value = op->type->modal(C, op, event);
-  if (return_value == OPERATOR_FINISHED) {
+  const wmOperatorStatus retval = op->type->modal(C, op, event);
+  OPERATOR_RETVAL_CHECK(retval);
+
+  if (retval == OPERATOR_FINISHED) {
     return OPERATOR_FINISHED;
   }
 
@@ -486,9 +496,9 @@ static int grease_pencil_weight_brush_stroke_invoke(bContext *C,
   return OPERATOR_RUNNING_MODAL;
 }
 
-static int grease_pencil_weight_brush_stroke_modal(bContext *C,
-                                                   wmOperator *op,
-                                                   const wmEvent *event)
+static wmOperatorStatus grease_pencil_weight_brush_stroke_modal(bContext *C,
+                                                                wmOperator *op,
+                                                                const wmEvent *event)
 {
   return paint_stroke_modal(C, op, event, reinterpret_cast<PaintStroke **>(&op->customdata));
 }
@@ -531,9 +541,9 @@ static bool grease_pencil_vertex_brush_stroke_poll(bContext *C)
   return true;
 }
 
-static int grease_pencil_vertex_brush_stroke_invoke(bContext *C,
-                                                    wmOperator *op,
-                                                    const wmEvent *event)
+static wmOperatorStatus grease_pencil_vertex_brush_stroke_invoke(bContext *C,
+                                                                 wmOperator *op,
+                                                                 const wmEvent *event)
 {
   const Scene *scene = CTX_data_scene(C);
   const Object *object = CTX_data_active_object(C);
@@ -542,14 +552,8 @@ static int grease_pencil_vertex_brush_stroke_invoke(bContext *C,
   }
 
   GreasePencil &grease_pencil = *static_cast<GreasePencil *>(object->data);
-  if (!grease_pencil.has_active_layer()) {
-    BKE_report(op->reports, RPT_ERROR, "No active Grease Pencil layer");
-    return OPERATOR_CANCELLED;
-  }
-
-  bke::greasepencil::Layer &active_layer = *grease_pencil.get_active_layer();
-  if (!active_layer.is_editable()) {
-    BKE_report(op->reports, RPT_ERROR, "Active layer is locked or hidden");
+  if (!ed::greasepencil::has_editable_layer(grease_pencil)) {
+    BKE_report(op->reports, RPT_ERROR, "No editable Grease Pencil layer");
     return OPERATOR_CANCELLED;
   }
 
@@ -564,15 +568,19 @@ static int grease_pencil_vertex_brush_stroke_invoke(bContext *C,
   /* For the vertex paint tools, we don't want the auto-key to create an empty keyframe, so we
    * duplicate the previous key. */
   const bool use_duplicate_previous_key = true;
-  if (!ed::greasepencil::ensure_active_keyframe(
-          *scene, grease_pencil, active_layer, use_duplicate_previous_key, inserted_keyframe))
-  {
+  for (bke::greasepencil::Layer *layer : grease_pencil.layers_for_write()) {
+    if (layer->is_editable() &&
+        ed::greasepencil::ensure_active_keyframe(
+            *scene, grease_pencil, *layer, use_duplicate_previous_key, inserted_keyframe))
+    {
+      inserted_keyframe = true;
+    }
+  }
+  if (!inserted_keyframe) {
     BKE_report(op->reports, RPT_ERROR, "No Grease Pencil frame to draw on");
     return OPERATOR_CANCELLED;
   }
-  if (inserted_keyframe) {
-    WM_event_add_notifier(C, NC_GPENCIL | NA_EDITED, nullptr);
-  }
+  WM_event_add_notifier(C, NC_GPENCIL | NA_EDITED, nullptr);
 
   op->customdata = paint_stroke_new(C,
                                     op,
@@ -583,8 +591,10 @@ static int grease_pencil_vertex_brush_stroke_invoke(bContext *C,
                                     stroke_done,
                                     event->type);
 
-  const int return_value = op->type->modal(C, op, event);
-  if (return_value == OPERATOR_FINISHED) {
+  const wmOperatorStatus retval = op->type->modal(C, op, event);
+  OPERATOR_RETVAL_CHECK(retval);
+
+  if (retval == OPERATOR_FINISHED) {
     return OPERATOR_FINISHED;
   }
 
@@ -592,9 +602,9 @@ static int grease_pencil_vertex_brush_stroke_invoke(bContext *C,
   return OPERATOR_RUNNING_MODAL;
 }
 
-static int grease_pencil_vertex_brush_stroke_modal(bContext *C,
-                                                   wmOperator *op,
-                                                   const wmEvent *event)
+static wmOperatorStatus grease_pencil_vertex_brush_stroke_modal(bContext *C,
+                                                                wmOperator *op,
+                                                                const wmEvent *event)
 {
   return paint_stroke_modal(C, op, event, reinterpret_cast<PaintStroke **>(&op->customdata));
 }
@@ -1098,7 +1108,7 @@ static void grease_pencil_fill_overlay_cb(const bContext *C, ARegion * /*region*
 
     for (const ed::greasepencil::DrawingInfo &info : drawings) {
       const IndexMask curve_mask = info.drawing.strokes().curves_range();
-      const VArray<ColorGeometry4f> colors = VArray<ColorGeometry4f>::ForSingle(
+      const VArray<ColorGeometry4f> colors = VArray<ColorGeometry4f>::from_single(
           stroke_curves_color, info.drawing.strokes().points_num());
       const float4x4 layer_to_world = grease_pencil.layer(info.layer_index).to_world_space(object);
       const bool use_xray = false;
@@ -1124,7 +1134,7 @@ static void grease_pencil_fill_overlay_cb(const bContext *C, ARegion * /*region*
 
     const IndexRange lines_range = extensions.lines.starts.index_range();
     if (!lines_range.is_empty()) {
-      const VArray<ColorGeometry4f> line_colors = VArray<ColorGeometry4f>::ForSingle(
+      const VArray<ColorGeometry4f> line_colors = VArray<ColorGeometry4f>::from_single(
           extension_lines_color, lines_range.size());
 
       ed::greasepencil::image_render::draw_lines(world_to_view,
@@ -1136,14 +1146,14 @@ static void grease_pencil_fill_overlay_cb(const bContext *C, ARegion * /*region*
     }
     const IndexRange circles_range = extensions.circles.centers.index_range();
     if (!circles_range.is_empty()) {
-      const VArray<ColorGeometry4f> circle_colors = VArray<ColorGeometry4f>::ForSingle(
+      const VArray<ColorGeometry4f> circle_colors = VArray<ColorGeometry4f>::from_single(
           extension_circles_color, circles_range.size());
 
       ed::greasepencil::image_render::draw_circles(
           world_to_view,
           circles_range,
           extensions.circles.centers,
-          VArray<float>::ForSpan(extensions.circles.radii),
+          VArray<float>::from_span(extensions.circles.radii),
           circle_colors,
           float2(region.winx, region.winy),
           line_width,
@@ -1188,27 +1198,27 @@ static VArray<bool> get_fill_boundary_layers(const GreasePencil &grease_pencil,
 
   switch (fill_layer_mode) {
     case GP_FILL_GPLMODE_ACTIVE:
-      return VArray<bool>::ForFunc(all_layers.size(), [active_layer_index](const int index) {
+      return VArray<bool>::from_std_func(all_layers.size(), [active_layer_index](const int index) {
         return index != active_layer_index;
       });
     case GP_FILL_GPLMODE_ABOVE:
-      return VArray<bool>::ForFunc(all_layers.size(), [active_layer_index](const int index) {
+      return VArray<bool>::from_std_func(all_layers.size(), [active_layer_index](const int index) {
         return index != active_layer_index + 1;
       });
     case GP_FILL_GPLMODE_BELOW:
-      return VArray<bool>::ForFunc(all_layers.size(), [active_layer_index](const int index) {
+      return VArray<bool>::from_std_func(all_layers.size(), [active_layer_index](const int index) {
         return index != active_layer_index - 1;
       });
     case GP_FILL_GPLMODE_ALL_ABOVE:
-      return VArray<bool>::ForFunc(all_layers.size(), [active_layer_index](const int index) {
+      return VArray<bool>::from_std_func(all_layers.size(), [active_layer_index](const int index) {
         return index <= active_layer_index;
       });
     case GP_FILL_GPLMODE_ALL_BELOW:
-      return VArray<bool>::ForFunc(all_layers.size(), [active_layer_index](const int index) {
+      return VArray<bool>::from_std_func(all_layers.size(), [active_layer_index](const int index) {
         return index >= active_layer_index;
       });
     case GP_FILL_GPLMODE_VISIBLE:
-      return VArray<bool>::ForFunc(all_layers.size(), [grease_pencil](const int index) {
+      return VArray<bool>::from_std_func(all_layers.size(), [grease_pencil](const int index) {
         return !grease_pencil.layers()[index]->is_visible();
       });
   }
@@ -1303,7 +1313,7 @@ static void smooth_fill_strokes(bke::CurvesGeometry &curves, const IndexMask &st
   bke::MutableAttributeAccessor attributes = curves.attributes_for_write();
   const OffsetIndices points_by_curve = curves.points_by_curve();
   const VArray<bool> cyclic = curves.cyclic();
-  const VArray<bool> point_selection = VArray<bool>::ForSingle(true, curves.points_num());
+  const VArray<bool> point_selection = VArray<bool>::from_single(true, curves.points_num());
 
   bke::GSpanAttributeWriter positions = attributes.lookup_for_write_span("position");
   geometry::smooth_curve_attribute(stroke_mask,
@@ -1372,6 +1382,8 @@ static bool grease_pencil_apply_fill(bContext &C, wmOperator &op, const wmEvent
           std::nullopt :
           std::make_optional(brush.gpencil_settings->fill_threshold);
   const bool on_back = (ts.gpencil_flags & GP_TOOL_FLAG_PAINT_ONBACK);
+  const bool auto_remove_fill_guides = (brush.gpencil_settings->flag &
+                                        GP_BRUSH_FILL_AUTO_REMOVE_FILL_GUIDES) != 0;
 
   if (!grease_pencil.has_active_layer()) {
     return false;
@@ -1383,6 +1395,7 @@ static bool grease_pencil_apply_fill(bContext &C, wmOperator &op, const wmEvent
   const VArray<bool> boundary_layers = get_fill_boundary_layers(
       grease_pencil, eGP_FillLayerModes(brush.gpencil_settings->fill_layer_mode));
 
+  bool did_create_fill = false;
   for (const FillToolTargetInfo &info : target_drawings) {
     const Layer &layer = *grease_pencil.layers()[info.target.layer_index];
 
@@ -1402,6 +1415,9 @@ static bool grease_pencil_apply_fill(bContext &C, wmOperator &op, const wmEvent
                                                    fit_method,
                                                    op_data.material_index,
                                                    keep_images);
+    if (fill_curves.is_empty()) {
+      continue;
+    }
 
     smooth_fill_strokes(fill_curves, fill_curves.curves_range());
 
@@ -1410,6 +1426,11 @@ static bool grease_pencil_apply_fill(bContext &C, wmOperator &op, const wmEvent
     }
 
     bke::CurvesGeometry &dst_curves = info.target.drawing.strokes_for_write();
+    if (auto_remove_fill_guides) {
+      /* Remove strokes that were created using the fill tool as boundary strokes. */
+      ed::greasepencil::remove_fill_guides(dst_curves);
+    }
+
     /* If the `fill_strokes` function creates the "fill_opacity" attribute, make sure that we
      * initialize this to full opacity on the target geometry. */
     if (fill_curves.attributes().contains("fill_opacity") &&
@@ -1419,7 +1440,7 @@ static bool grease_pencil_apply_fill(bContext &C, wmOperator &op, const wmEvent
           dst_curves.attributes_for_write().lookup_or_add_for_write_span<float>(
               "fill_opacity",
               bke::AttrDomain::Curve,
-              bke::AttributeInitVArray(VArray<float>::ForSingle(1.0f, dst_curves.curves_num())));
+              bke::AttributeInitVArray(VArray<float>::from_single(1.0f, dst_curves.curves_num())));
       fill_opacities.finish();
     }
 
@@ -1446,6 +1467,12 @@ static bool grease_pencil_apply_fill(bContext &C, wmOperator &op, const wmEvent
       Array<float4x2> texture_matrices(num_new_curves, texture_space);
       info.target.drawing.set_texture_matrices(texture_matrices, new_curves_range);
     }
+
+    did_create_fill = true;
+  }
+
+  if (!did_create_fill) {
+    BKE_reportf(op.reports, RPT_ERROR, "Unable to fill unclosed areas");
   }
 
   WM_cursor_modal_restore(&win);
@@ -1487,9 +1514,9 @@ static bool grease_pencil_fill_init(bContext &C, wmOperator &op)
   BKE_curvemapping_init(brush.gpencil_settings->curve_rand_pressure);
   BKE_curvemapping_init(brush.gpencil_settings->curve_rand_strength);
   BKE_curvemapping_init(brush.gpencil_settings->curve_rand_uv);
-  BKE_curvemapping_init(brush.gpencil_settings->curve_rand_hue);
-  BKE_curvemapping_init(brush.gpencil_settings->curve_rand_saturation);
-  BKE_curvemapping_init(brush.gpencil_settings->curve_rand_value);
+  BKE_curvemapping_init(brush.curve_rand_hue);
+  BKE_curvemapping_init(brush.curve_rand_saturation);
+  BKE_curvemapping_init(brush.curve_rand_value);
 
   Material *material = BKE_grease_pencil_object_material_ensure_from_brush(&bmain, &ob, &brush);
   const int material_index = BKE_object_material_index_get(&ob, material);
@@ -1532,7 +1559,9 @@ static void grease_pencil_fill_exit(bContext &C, wmOperator &op)
   WM_event_add_notifier(&C, NC_GPENCIL | ND_DATA | NA_EDITED, nullptr);
 }
 
-static int grease_pencil_fill_invoke(bContext *C, wmOperator *op, const wmEvent * /*event*/)
+static wmOperatorStatus grease_pencil_fill_invoke(bContext *C,
+                                                  wmOperator *op,
+                                                  const wmEvent * /*event*/)
 {
   const ARegion &region = *CTX_wm_region(C);
   ToolSettings &ts = *CTX_data_tool_settings(C);
@@ -1582,7 +1611,9 @@ enum class FillToolModalKey : int8_t {
   Precision,
 };
 
-static int grease_pencil_fill_event_modal_map(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus grease_pencil_fill_event_modal_map(bContext *C,
+                                                           wmOperator *op,
+                                                           const wmEvent *event)
 {
   auto &op_data = *static_cast<GreasePencilFillOpData *>(op->customdata);
   /* Extension line length increment, for normal and precise mode respectively. */
@@ -1670,13 +1701,13 @@ static int grease_pencil_fill_event_modal_map(bContext *C, wmOperator *op, const
   return OPERATOR_RUNNING_MODAL;
 }
 
-static int grease_pencil_fill_modal(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus grease_pencil_fill_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   const RegionView3D &rv3d = *CTX_wm_region_view3d(C);
 
   auto &op_data = *static_cast<GreasePencilFillOpData *>(op->customdata);
 
-  int estate = OPERATOR_CANCELLED;
+  wmOperatorStatus estate = OPERATOR_CANCELLED;
   if (!op_data.show_extension) {
     /* Apply fill immediately if "Visual Aids" (aka. extension lines) is disabled. */
     op_data.fill_mouse_pos = float2(event->mval);
@@ -1799,8 +1830,8 @@ static bool remove_points_and_split_from_drawings(
     if (Drawing *drawing = get_current_drawing_or_duplicate_for_autokey(
             scene, grease_pencil, info.layer_index))
     {
-      drawing->strokes_for_write() = ed::greasepencil::remove_points_and_split(drawing->strokes(),
-                                                                               points_to_remove);
+      drawing->strokes_for_write() = geometry::remove_points_and_split(drawing->strokes(),
+                                                                       points_to_remove);
       drawing->tag_topology_changed();
       changed = true;
     }
@@ -1809,13 +1840,30 @@ static bool remove_points_and_split_from_drawings(
   return changed;
 }
 
+static inline bool is_point_inside_bounds(const Bounds<int2> bounds, const int2 point)
+{
+  if (point.x < bounds.min.x) {
+    return false;
+  }
+  if (point.x > bounds.max.x) {
+    return false;
+  }
+  if (point.y < bounds.min.y) {
+    return false;
+  }
+  if (point.y > bounds.max.y) {
+    return false;
+  }
+  return true;
+}
+
 static inline bool is_point_inside_lasso(const Array<int2> lasso, const int2 point)
 {
   return isect_point_poly_v2_int(
-      point, reinterpret_cast<const int(*)[2]>(lasso.data()), uint(lasso.size()));
+      point, reinterpret_cast<const int (*)[2]>(lasso.data()), uint(lasso.size()));
 }
 
-static int grease_pencil_erase_lasso_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_erase_lasso_exec(bContext *C, wmOperator *op)
 {
   using namespace bke::greasepencil;
   using namespace ed::greasepencil;
@@ -1823,7 +1871,7 @@ static int grease_pencil_erase_lasso_exec(bContext *C, wmOperator *op)
   const Depsgraph *depsgraph = CTX_data_depsgraph_pointer(C);
   const ARegion *region = CTX_wm_region(C);
   Object *object = CTX_data_active_object(C);
-  const Object *ob_eval = DEG_get_evaluated_object(depsgraph, object);
+  const Object *ob_eval = DEG_get_evaluated(depsgraph, object);
   GreasePencil &grease_pencil = *static_cast<GreasePencil *>(object->data);
 
   const Array<int2> lasso = WM_gesture_lasso_path_to_array(C, op);
@@ -1844,7 +1892,7 @@ static int grease_pencil_erase_lasso_exec(bContext *C, wmOperator *op)
       const bke::greasepencil::Layer &layer = grease_pencil.layer(info.layer_index);
       const bke::crazyspace::GeometryDeformation deformation =
           bke::crazyspace::get_evaluated_grease_pencil_drawing_deformation(
-              ob_eval, *object, info.layer_index, info.frame_number);
+              ob_eval, *object, info.drawing);
       const float4x4 layer_to_world = layer.to_world_space(*ob_eval);
 
       const bke::CurvesGeometry &curves = info.drawing.strokes();
@@ -1872,6 +1920,15 @@ static int grease_pencil_erase_lasso_exec(bContext *C, wmOperator *op)
       IndexMaskMemory &memory = memories[drawing_i];
       const IndexMask curve_selection = IndexMask::from_predicate(
           curves.curves_range(), GrainSize(512), memory, [&](const int64_t index) {
+            /* For a single point curve, its screen_space_curve_bounds Bounds will be empty (by
+             * definition), so intersecting will fail. Check if the single point is in the bounds
+             * instead. */
+            const IndexRange points = points_by_curve[index];
+            if (points.size() == 1) {
+              return is_point_inside_bounds(lasso_bounds_int,
+                                            int2(screen_space_positions[points.first()]));
+            }
+
             return bounds::intersect(lasso_bounds, screen_space_curve_bounds[index]).has_value();
           });
 
@@ -1917,24 +1974,7 @@ static void GREASE_PENCIL_OT_erase_lasso(wmOperatorType *ot)
   WM_operator_properties_gesture_lasso(ot);
 }
 
-static inline bool is_point_inside_bounds(const Bounds<int2> bounds, const int2 point)
-{
-  if (point.x < bounds.min.x) {
-    return false;
-  }
-  if (point.x > bounds.max.x) {
-    return false;
-  }
-  if (point.y < bounds.min.y) {
-    return false;
-  }
-  if (point.y > bounds.max.y) {
-    return false;
-  }
-  return true;
-}
-
-static int grease_pencil_erase_box_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_erase_box_exec(bContext *C, wmOperator *op)
 {
   using namespace bke::greasepencil;
   using namespace ed::greasepencil;
@@ -1942,7 +1982,7 @@ static int grease_pencil_erase_box_exec(bContext *C, wmOperator *op)
   const Depsgraph *depsgraph = CTX_data_depsgraph_pointer(C);
   const ARegion *region = CTX_wm_region(C);
   Object *object = CTX_data_active_object(C);
-  const Object *ob_eval = DEG_get_evaluated_object(depsgraph, object);
+  const Object *ob_eval = DEG_get_evaluated(depsgraph, object);
   GreasePencil &grease_pencil = *static_cast<GreasePencil *>(object->data);
 
   const Bounds<int2> box_bounds = WM_operator_properties_border_to_bounds(op);
@@ -1960,7 +2000,7 @@ static int grease_pencil_erase_box_exec(bContext *C, wmOperator *op)
       const bke::greasepencil::Layer &layer = grease_pencil.layer(info.layer_index);
       const bke::crazyspace::GeometryDeformation deformation =
           bke::crazyspace::get_evaluated_grease_pencil_drawing_deformation(
-              ob_eval, *object, info.layer_index, info.frame_number);
+              ob_eval, *object, info.drawing);
       const float4x4 layer_to_world = layer.to_world_space(*ob_eval);
 
       const bke::CurvesGeometry &curves = info.drawing.strokes();
