@@ -10,6 +10,7 @@
 
 #include "DNA_scene_types.h"
 
+#include "BLI_listbase.h"
 #include "BLI_math_base.h"
 #include "BLI_utildefines.h"
 
@@ -38,7 +39,7 @@
 /** \name Set Cursor
  *
  * The 'cursor' in the Graph Editor consists of two parts:
- * 1) Current Frame Indicator (as per ANIM_OT_change_frame)
+ * 1) Current Frame Indicator (as per #ANIM_OT_change_frame)
  * 2) Value Indicator (stored per Graph Editor instance)
  * \{ */
 
@@ -66,9 +67,9 @@ static void graphview_cursor_apply(bContext *C, wmOperator *op)
     sipo->cursorTime = frame;
   }
   else {
-    /* adjust the frame
-     * NOTE: sync this part of the code with ANIM_OT_change_frame
-     */
+    /* Adjust the frame.
+     * NOTE: sync this part of the code with #ANIM_OT_change_frame. */
+
     /* 1) frame is rounded to the nearest int, since frames are ints */
     scene->r.cfra = round_fl_to_int(frame);
 
@@ -98,7 +99,7 @@ static void graphview_cursor_apply(bContext *C, wmOperator *op)
 /* ... */
 
 /* Non-modal callback for running operator without user input */
-static int graphview_cursor_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus graphview_cursor_exec(bContext *C, wmOperator *op)
 {
   graphview_cursor_apply(C, op);
   return OPERATOR_FINISHED;
@@ -127,7 +128,7 @@ static void graphview_cursor_setprops(bContext *C, wmOperator *op, const wmEvent
 }
 
 /* Modal Operator init */
-static int graphview_cursor_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus graphview_cursor_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   bScreen *screen = CTX_wm_screen(C);
 
@@ -149,7 +150,7 @@ static int graphview_cursor_invoke(bContext *C, wmOperator *op, const wmEvent *e
 }
 
 /* Modal event handling of cursor changing */
-static int graphview_cursor_modal(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus graphview_cursor_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   bScreen *screen = CTX_wm_screen(C);
   Scene *scene = CTX_data_scene(C);
@@ -183,6 +184,9 @@ static int graphview_cursor_modal(bContext *C, wmOperator *op, const wmEvent *ev
         return OPERATOR_FINISHED;
       }
       break;
+    default: {
+      break;
+    }
   }
 
   return OPERATOR_RUNNING_MODAL;
@@ -195,7 +199,7 @@ static void GRAPH_OT_cursor_set(wmOperatorType *ot)
   ot->idname = "GRAPH_OT_cursor_set";
   ot->description = "Interactively set the current frame and value cursor";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = graphview_cursor_exec;
   ot->invoke = graphview_cursor_invoke;
   ot->modal = graphview_cursor_modal;
@@ -215,7 +219,7 @@ static void GRAPH_OT_cursor_set(wmOperatorType *ot)
 /** \name Hide/Reveal
  * \{ */
 
-static int graphview_curves_hide_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus graphview_curves_hide_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
   ListBase anim_data = {nullptr, nullptr};
@@ -316,7 +320,7 @@ static void GRAPH_OT_hide(wmOperatorType *ot)
   ot->idname = "GRAPH_OT_hide";
   ot->description = "Hide selected curves from Graph Editor view";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = graphview_curves_hide_exec;
   ot->poll = ED_operator_graphedit_active;
 
@@ -330,7 +334,7 @@ static void GRAPH_OT_hide(wmOperatorType *ot)
 
 /* ........ */
 
-static int graphview_curves_reveal_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus graphview_curves_reveal_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
   ListBase anim_data = {nullptr, nullptr};
@@ -400,7 +404,7 @@ static void GRAPH_OT_reveal(wmOperatorType *ot)
   ot->idname = "GRAPH_OT_reveal";
   ot->description = "Make previously hidden curves visible again in Graph Editor view";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = graphview_curves_reveal_exec;
   ot->poll = ED_operator_graphedit_active;
 
