@@ -20,7 +20,11 @@
 /* Own include. */
 #include "sequencer_intern.hh"
 
-static int sequencer_rename_channel_invoke(bContext *C, wmOperator * /*op*/, const wmEvent *event)
+namespace blender::ed::vse {
+
+static wmOperatorStatus sequencer_rename_channel_invoke(bContext *C,
+                                                        wmOperator * /*op*/,
+                                                        const wmEvent *event)
 {
   SeqChannelDrawContext context;
   SpaceSeq *sseq = CTX_wm_space_seq(C);
@@ -28,7 +32,7 @@ static int sequencer_rename_channel_invoke(bContext *C, wmOperator * /*op*/, con
   float mouse_y = UI_view2d_region_to_view_y(context.timeline_region_v2d, event->mval[1]);
 
   sseq->runtime->rename_channel_index = mouse_y;
-  WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, CTX_data_scene(C));
+  WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, CTX_data_sequencer_scene(C));
   return OPERATOR_FINISHED;
 }
 
@@ -38,10 +42,12 @@ void SEQUENCER_OT_rename_channel(wmOperatorType *ot)
   ot->name = "Rename Channel";
   ot->idname = "SEQUENCER_OT_rename_channel";
 
-  /* Api callbacks. */
+  /* API callbacks. */
   ot->invoke = sequencer_rename_channel_invoke;
   ot->poll = sequencer_edit_with_channel_region_poll;
 
   /* Flags. */
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_INTERNAL;
 }
+
+}  // namespace blender::ed::vse
