@@ -20,6 +20,7 @@
 
 #include "RNA_define.hh"
 #include "RNA_enum_types.hh"
+#include "RNA_prototypes.hh"
 
 #include "SEQ_modifier.hh"
 #include "SEQ_relations.hh"
@@ -27,6 +28,8 @@
 #include "SEQ_sequencer.hh"
 #include "SEQ_sound.hh"
 
+#include "UI_interface_c.hh"
+
 /* Own include. */
 #include "sequencer_intern.hh"
 
@@ -38,11 +41,12 @@ namespace blender::ed::vse {
 
 static wmOperatorStatus strip_modifier_add_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Strip *strip = seq::select_active_get(scene);
   int type = RNA_enum_get(op->ptr, "type");
 
-  seq::modifier_new(strip, nullptr, type);
+  StripModifierData *smd = seq::modifier_new(strip, nullptr, type);
+  seq::modifier_persistent_uid_init(*strip, *smd);
 
   seq::relations_invalidate_cache(scene, strip);
   WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, scene);
@@ -59,7 +63,7 @@ static const EnumPropertyItem *filter_modifiers_by_sequence_type_itemf(bContext
     return rna_enum_strip_modifier_type_items;
   }
 
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Strip *strip = seq::select_active_get(scene);
   if (strip) {
     if (ELEM(strip->type, STRIP_TYPE_SOUND_RAM)) {
@@ -99,7 +103,7 @@ void SEQUENCER_OT_strip_modifier_add(wmOperatorType *ot)
 
 static wmOperatorStatus strip_modifier_remove_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Strip *strip = seq::select_active_get(scene);
   char name[MAX_NAME];
   StripModifierData *smd;
@@ -159,7 +163,7 @@ enum {
 
 static wmOperatorStatus strip_modifier_move_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Strip *strip = seq::select_active_get(scene);
   char name[MAX_NAME];
   int direction;
@@ -240,53 +244,53 @@ enum {
 
 static wmOperatorStatus strip_modifier_copy_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
-  Editing *ed = scene->ed;
-  Strip *strip = seq::select_active_get(scene);
+  Scene *scene = CTX_data_sequencer_scene(C);
+  Strip *active_strip = seq::select_active_get(scene);
   const int type = RNA_enum_get(op->ptr, "type");
 
-  if (!strip || !strip->modifiers.first) {
+  if (!active_strip || !active_strip->modifiers.first) {
     return OPERATOR_CANCELLED;
   }
 
-  int isSound = ELEM(strip->type, STRIP_TYPE_SOUND_RAM);
+  int isSound = ELEM(active_strip->type, STRIP_TYPE_SOUND_RAM);
 
-  LISTBASE_FOREACH (Strip *, strip_iter, seq::active_seqbase_get(ed)) {
-    if (strip_iter->flag & SELECT) {
-      if (strip_iter == strip) {
-        continue;
-      }
-      int strip_iter_is_sound = ELEM(strip_iter->type, STRIP_TYPE_SOUND_RAM);
-      /* If original is sound, only copy to "sound" strips
-       * If original is not sound, only copy to "not sound" strips
-       */
-      if (isSound != strip_iter_is_sound) {
-        continue;
-      }
+  VectorSet<Strip *> selected = selected_strips_from_context(C);
+  selected.remove(active_strip);
 
-      if (type == SEQ_MODIFIER_COPY_REPLACE) {
-        if (strip_iter->modifiers.first) {
-          StripModifierData *smd_tmp,
-              *smd = static_cast<StripModifierData *>(strip_iter->modifiers.first);
-          while (smd) {
-            smd_tmp = smd->next;
-            BLI_remlink(&strip_iter->modifiers, smd);
-            seq::modifier_free(smd);
-            smd = smd_tmp;
-          }
-          BLI_listbase_clear(&strip_iter->modifiers);
+  for (Strip *strip_iter : selected) {
+    int strip_iter_is_sound = ELEM(strip_iter->type, STRIP_TYPE_SOUND_RAM);
+    /* If original is sound, only copy to "sound" strips
+     * If original is not sound, only copy to "not sound" strips
+     */
+    if (isSound != strip_iter_is_sound) {
+      continue;
+    }
+
+    if (type == SEQ_MODIFIER_COPY_REPLACE) {
+      if (strip_iter->modifiers.first) {
+        StripModifierData *smd_tmp,
+            *smd = static_cast<StripModifierData *>(strip_iter->modifiers.first);
+        while (smd) {
+          smd_tmp = smd->next;
+          BLI_remlink(&strip_iter->modifiers, smd);
+          seq::modifier_free(smd);
+          smd = smd_tmp;
         }
+        BLI_listbase_clear(&strip_iter->modifiers);
       }
+    }
 
-      seq::modifier_list_copy(strip_iter, strip);
+    LISTBASE_FOREACH (StripModifierData *, smd, &active_strip->modifiers) {
+      StripModifierData *smd_new = seq::modifier_copy(*strip_iter, smd);
+      seq::modifier_persistent_uid_init(*strip_iter, *smd_new);
     }
   }
 
-  if (ELEM(strip->type, STRIP_TYPE_SOUND_RAM)) {
+  if (ELEM(active_strip->type, STRIP_TYPE_SOUND_RAM)) {
     DEG_id_tag_update(&scene->id, ID_RECALC_SEQUENCER_STRIPS | ID_RECALC_AUDIO);
   }
   else {
-    seq::relations_invalidate_cache(scene, strip);
+    seq::relations_invalidate_cache(scene, active_strip);
   }
 
   WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, scene);
@@ -331,7 +335,7 @@ void SEQUENCER_OT_strip_modifier_copy(wmOperatorType *ot)
 
 static wmOperatorStatus strip_modifier_equalizer_redefine_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Strip *strip = seq::select_active_get(scene);
   StripModifierData *smd;
   char name[MAX_NAME];
@@ -385,4 +389,120 @@ void SEQUENCER_OT_strip_modifier_equalizer_redefine(wmOperatorType *ot)
 
 /** \} */
 
+/* ------------------------------------------------------------------- */
+/** \name Move to Index Modifier Operator
+ * \{ */
+
+static wmOperatorStatus modifier_move_to_index_exec(bContext *C, wmOperator *op)
+{
+  Scene *scene = CTX_data_sequencer_scene(C);
+  Strip *strip = seq::select_active_get(scene);
+
+  char name[MAX_NAME];
+  RNA_string_get(op->ptr, "modifier", name);
+  const int index = RNA_int_get(op->ptr, "index");
+
+  StripModifierData *smd = seq::modifier_find_by_name(strip, name);
+  if (!smd) {
+    return OPERATOR_CANCELLED;
+  }
+
+  if (!seq::modifier_move_to_index(strip, smd, index)) {
+    return OPERATOR_CANCELLED;
+  }
+
+  if (ELEM(strip->type, STRIP_TYPE_SOUND_RAM)) {
+    DEG_id_tag_update(&scene->id, ID_RECALC_SEQUENCER_STRIPS | ID_RECALC_AUDIO);
+  }
+  else {
+    seq::relations_invalidate_cache(scene, strip);
+  }
+
+  WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, scene);
+
+  return OPERATOR_FINISHED;
+}
+
+static wmOperatorStatus modifier_move_to_index_invoke(bContext *C,
+                                                      wmOperator *op,
+                                                      const wmEvent * /*event*/)
+{
+  BLI_assert(RNA_struct_property_is_set(op->ptr, "modifier"));
+  return modifier_move_to_index_exec(C, op);
+}
+
+void SEQUENCER_OT_strip_modifier_move_to_index(wmOperatorType *ot)
+{
+  PropertyRNA *prop;
+
+  ot->name = "Move Active Strip Modifier to Index";
+  ot->description =
+      "Change the strip modifier's index in the stack so it evaluates after the set number of "
+      "others";
+  ot->idname = "SEQUENCER_OT_strip_modifier_move_to_index";
+
+  ot->invoke = modifier_move_to_index_invoke;
+  ot->exec = modifier_move_to_index_exec;
+  ot->poll = sequencer_strip_editable_poll;
+
+  /* flags */
+  ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_INTERNAL;
+
+  prop = RNA_def_string(
+      ot->srna, "modifier", nullptr, MAX_NAME, "Modifier", "Name of the modifier to edit");
+  RNA_def_property_flag(prop, PROP_HIDDEN);
+  RNA_def_int(
+      ot->srna, "index", 0, 0, INT_MAX, "Index", "The index to move the modifier to", 0, INT_MAX);
+}
+
+/** \} */
+
+/* ------------------------------------------------------------------- */
+/** \name Set Active Modifier Operator
+ * \{ */
+
+static wmOperatorStatus modifier_set_active_exec(bContext *C, wmOperator *op)
+{
+  Scene *scene = CTX_data_sequencer_scene(C);
+  Strip *strip = seq::select_active_get(scene);
+
+  char name[MAX_NAME];
+  RNA_string_get(op->ptr, "modifier", name);
+
+  StripModifierData *smd = seq::modifier_find_by_name(strip, name);
+  /* If there is no modifier set for this operator, clear the active modifier field. */
+  blender::seq::modifier_set_active(strip, smd);
+
+  WM_main_add_notifier(NC_SCENE | ND_SEQUENCER, scene);
+
+  return OPERATOR_FINISHED;
+}
+
+static wmOperatorStatus modifier_set_active_invoke(bContext *C,
+                                                   wmOperator *op,
+                                                   const wmEvent * /*event*/)
+{
+  BLI_assert(RNA_struct_property_is_set(op->ptr, "modifier"));
+  return modifier_set_active_exec(C, op);
+}
+
+void SEQUENCER_OT_strip_modifier_set_active(wmOperatorType *ot)
+{
+  ot->name = "Set Active Strip Modifier";
+  ot->description = "Activate the strip modifier to use as the context";
+  ot->idname = "SEQUENCER_OT_strip_modifier_set_active";
+
+  ot->invoke = modifier_set_active_invoke;
+  ot->exec = modifier_set_active_exec;
+  ot->poll = sequencer_strip_editable_poll;
+
+  ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_INTERNAL;
+
+  ot->prop = RNA_def_string(
+      ot->srna, "modifier", nullptr, MAX_NAME, "Modifier", "Name of the strip modifier to edit");
+  RNA_def_property_flag(ot->prop, PROP_HIDDEN);
+}
+
+/** \} */
+
 }  // namespace blender::ed::vse
