@@ -10,6 +10,8 @@
 #include <cstdlib>
 #include <cstring>
 
+#include "BLI_lasso_2d.hh"
+#include "BLI_rect.h"
 #include "MEM_guardedalloc.h"
 
 #include "BLI_ghash.h"
@@ -17,6 +19,7 @@
 #include "BLI_math_geom.h"
 #include "BLI_math_vector.h"
 #include "BLI_math_vector_types.hh"
+#include "BLI_set.hh"
 #include "BLI_string.h"
 #include "BLI_utildefines.h"
 
@@ -117,7 +120,7 @@ Strip *strip_under_mouse_get(const Scene *scene, const View2D *v2d, const int mv
 
 blender::VectorSet<Strip *> all_strips_from_context(bContext *C)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
   ListBase *seqbase = seq::active_seqbase_get(ed);
   ListBase *channels = seq::channels_displayed_get(ed);
@@ -132,7 +135,7 @@ blender::VectorSet<Strip *> all_strips_from_context(bContext *C)
 
 blender::VectorSet<Strip *> selected_strips_from_context(bContext *C)
 {
-  const Scene *scene = CTX_data_scene(C);
+  const Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
   ListBase *seqbase = seq::active_seqbase_get(ed);
   ListBase *channels = seq::channels_displayed_get(ed);
@@ -288,24 +291,12 @@ void select_surround_from_last(Scene *scene)
 
 void select_strip_single(Scene *scene, Strip *strip, bool deselect_all)
 {
-  Editing *ed = seq::editing_get(scene);
-
   if (deselect_all) {
     deselect_all_strips(scene);
   }
 
   seq::select_active_set(scene, strip);
 
-  if (ELEM(strip->type, STRIP_TYPE_IMAGE, STRIP_TYPE_MOVIE)) {
-    if (strip->data) {
-      BLI_strncpy(ed->act_imagedir, strip->data->dirpath, FILE_MAXDIR);
-    }
-  }
-  else if (strip->type == STRIP_TYPE_SOUND_RAM) {
-    if (strip->data) {
-      BLI_strncpy(ed->act_sounddir, strip->data->dirpath, FILE_MAXDIR);
-    }
-  }
   strip->flag |= SELECT;
   recurs_sel_strip(strip);
 }
@@ -330,7 +321,7 @@ Strip *find_neighboring_strip(const Scene *scene, const Strip *test, const int l
   if (sel > 0) {
     sel = SELECT;
   }
-  LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+  LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
     if ((strip != test) && (test->channel == strip->channel) &&
         ((sel == -1) || (sel && (strip->flag & SELECT)) ||
          (sel == 0 && (strip->flag & SELECT) == 0)))
@@ -440,7 +431,7 @@ void sequencer_select_do_updates(const bContext *C, Scene *scene)
 static wmOperatorStatus sequencer_de_select_all_exec(bContext *C, wmOperator *op)
 {
   int action = RNA_enum_get(op->ptr, "action");
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
 
   if (sequencer_view_has_preview_poll(C) && !sequencer_view_preview_only_poll(C)) {
     return OPERATOR_CANCELLED;
@@ -516,7 +507,7 @@ void SEQUENCER_OT_select_all(wmOperatorType *ot)
 
 static wmOperatorStatus sequencer_select_inverse_exec(bContext *C, wmOperator * /*op*/)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
 
   if (sequencer_view_has_preview_poll(C) && !sequencer_view_preview_only_poll(C)) {
     return OPERATOR_CANCELLED;
@@ -563,20 +554,7 @@ void SEQUENCER_OT_select_inverse(wmOperatorType *ot)
 
 static void sequencer_select_set_active(Scene *scene, Strip *strip)
 {
-  Editing *ed = seq::editing_get(scene);
-
   seq::select_active_set(scene, strip);
-
-  if (ELEM(strip->type, STRIP_TYPE_IMAGE, STRIP_TYPE_MOVIE)) {
-    if (strip->data) {
-      BLI_strncpy(ed->act_imagedir, strip->data->dirpath, FILE_MAXDIR);
-    }
-  }
-  else if (strip->type == STRIP_TYPE_SOUND_RAM) {
-    if (strip->data) {
-      BLI_strncpy(ed->act_sounddir, strip->data->dirpath, FILE_MAXDIR);
-    }
-  }
   recurs_sel_strip(strip);
 }
 
@@ -622,7 +600,7 @@ static void sequencer_select_linked_handle(const bContext *C,
                                            Strip *strip,
                                            const eStripHandle handle_clicked)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
   if (!ELEM(handle_clicked, STRIP_HANDLE_LEFT, STRIP_HANDLE_RIGHT)) {
     /* First click selects the strip and its adjacent handles (if valid).
@@ -650,7 +628,7 @@ static void sequencer_select_linked_handle(const bContext *C,
           if ((strip->flag & SEQ_LEFTSEL) && (neighbor->flag & SEQ_RIGHTSEL)) {
             strip->flag |= SELECT;
             select_active_side(scene,
-                               ed->seqbasep,
+                               ed->current_strips(),
                                seq::SIDE_LEFT,
                                strip->channel,
                                seq::time_left_handle_frame_get(scene, strip));
@@ -667,7 +645,7 @@ static void sequencer_select_linked_handle(const bContext *C,
           if ((strip->flag & SEQ_RIGHTSEL) && (neighbor->flag & SEQ_LEFTSEL)) {
             strip->flag |= SELECT;
             select_active_side(scene,
-                               ed->seqbasep,
+                               ed->current_strips(),
                                seq::SIDE_RIGHT,
                                strip->channel,
                                seq::time_left_handle_frame_get(scene, strip));
@@ -685,7 +663,7 @@ static void sequencer_select_linked_handle(const bContext *C,
     else {
 
       select_active_side(scene,
-                         ed->seqbasep,
+                         ed->current_strips(),
                          sel_side,
                          strip->channel,
                          seq::time_left_handle_frame_get(scene, strip));
@@ -739,7 +717,7 @@ static int strip_sort_for_center_select(const void *a, const void *b)
 static Strip *strip_select_from_preview(
     const bContext *C, const int mval[2], const bool toggle, const bool extend, const bool center)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
   ListBase *seqbase = seq::active_seqbase_get(ed);
   ListBase *channels = seq::channels_displayed_get(ed);
@@ -1003,8 +981,8 @@ static void select_linked_time(const Scene *scene,
  * Similar to `strip_handle_draw_size_get()`, but returns a larger clickable area that is
  * the same for a given zoom level no matter whether "simplified tweaking" is turned off or on.
  * `strip_clickable_areas_get` will pad this past strip bounds by 1/3 of the inner handle size,
- * making the full handle size either 15 + 5 = 20px or 1/4 + 1/12 = 1/3 of the strip size.
- */
+ * making the full size 15 + 5 = 20px in frames for large strips, 1/4 + 1/12 = 1/3 of the strip
+ * size for small ones. */
 static float inner_clickable_handle_size_get(const Scene *scene,
                                              const Strip *strip,
                                              const View2D *v2d)
@@ -1027,14 +1005,10 @@ bool can_select_handle(const Scene *scene, const Strip *strip, const View2D *v2d
     return false;
   }
 
-  /* This ensures clickable handles are deactivated when the strip gets too small (25 or 15
-   * frames). Since the full handle size for a small strip is 1/3 of the strip size (see
-   * `inner_clickable_handle_size_get`), this means handles cannot be smaller than 25/3 = 8px for
-   * simple tweaking, 15/3 = 5px for legacy behavior. */
+  /* This ensures clickable handles are deactivated when the strip gets too small
+   * (25 pixels). Since the full handle size for a small strip is 1/3 of the strip size (see
+   * `inner_clickable_handle_size_get`), this means handles cannot be smaller than 25/3 = 8px. */
   int min_len = 25 * U.pixelsize;
-  if ((U.sequencer_editor_flag & USER_SEQ_ED_SIMPLE_TWEAKING) == 0) {
-    min_len = 15 * U.pixelsize;
-  }
 
   const float pixelx = 1 / UI_view2d_scale_get_x(v2d);
   const int strip_len = seq::time_right_handle_frame_get(scene, strip) -
@@ -1104,7 +1078,7 @@ static blender::Vector<Strip *> padded_strips_under_mouse_get(const Scene *scene
   }
 
   blender::Vector<Strip *> strips;
-  LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+  LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
     if (strip->channel != int(mouse_co[1])) {
       continue;
     }
@@ -1204,7 +1178,7 @@ StripSelection pick_strip_and_handle(const Scene *scene, const View2D *v2d, floa
   selection.strip1 = strips[0];
   selection.handle = strip_handle_under_cursor_get(scene, selection.strip1, v2d, mouse_co);
 
-  if (strips.size() == 2 && (U.sequencer_editor_flag & USER_SEQ_ED_SIMPLE_TWEAKING) != 0 &&
+  if (strips.size() == 2 &&
       is_mouse_over_both_handles_of_adjacent_strips(scene, strips, v2d, mouse_co))
   {
     selection.strip2 = strips[1];
@@ -1216,21 +1190,14 @@ StripSelection pick_strip_and_handle(const Scene *scene, const View2D *v2d, floa
 wmOperatorStatus sequencer_select_exec(bContext *C, wmOperator *op)
 {
   const View2D *v2d = UI_view2d_fromcontext(C);
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
   ARegion *region = CTX_wm_region(C);
-  ScrArea *area = CTX_wm_area(C);
 
   if (ed == nullptr) {
     return OPERATOR_CANCELLED;
   }
 
-  if (STREQ(area->runtime.tool->idname, "builtin.blade")) {
-    /* Blade tool overrides select operator everywhere except the padded part of handles. We
-     * should not be able to select these with this tool active, so return. */
-    return OPERATOR_CANCELLED;
-  }
-
   if (region->regiontype == RGN_TYPE_PREVIEW) {
     if (!sequencer_view_preview_only_poll(C)) {
       return OPERATOR_CANCELLED;
@@ -1353,8 +1320,7 @@ wmOperatorStatus sequencer_select_exec(bContext *C, wmOperator *op)
   VectorSet<Strip *> copy_to;
   /* True if the user selects either handle of a strip that is already selected, meaning that
    * handles should be propagated to all currently selected strips. */
-  bool copy_handles_to_sel = (U.sequencer_editor_flag & USER_SEQ_ED_SIMPLE_TWEAKING) &&
-                             (selection.handle != STRIP_HANDLE_NONE) &&
+  bool copy_handles_to_sel = (selection.handle != STRIP_HANDLE_NONE) &&
                              (selection.strip1->flag & SELECT);
 
   /* TODO(john): Dual handle propagation is not supported for now due to its complexity,
@@ -1364,7 +1330,7 @@ wmOperatorStatus sequencer_select_exec(bContext *C, wmOperator *op)
   if (copy_handles_to_sel) {
     copy_to = seq::query_selected_strips(seq::active_seqbase_get(scene->ed));
     copy_to.remove(selection.strip1);
-    copy_to.remove_if([](Strip *strip) { return (strip->type & STRIP_TYPE_EFFECT); });
+    copy_to.remove_if([](Strip *strip) { return strip->is_effect(); });
   }
 
   bool changed = false;
@@ -1506,21 +1472,17 @@ void SEQUENCER_OT_select(wmOperatorType *ot)
 /** \name Select Handle Operator
  * \{ */
 
+/** This operator is only used in the RCS keymap by default and is not exposed in any menus. */
 static wmOperatorStatus sequencer_select_handle_exec(bContext *C, wmOperator *op)
 {
-  /* This operator is only used in the RCS keymap by default and is not exposed in any menus. */
   const View2D *v2d = UI_view2d_fromcontext(C);
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
 
   if (ed == nullptr) {
     return OPERATOR_CANCELLED;
   }
 
-  if ((U.sequencer_editor_flag & USER_SEQ_ED_SIMPLE_TWEAKING) == 0) {
-    return OPERATOR_CANCELLED | OPERATOR_PASS_THROUGH;
-  }
-
   MouseCoords mouse_co(v2d, RNA_int_get(op->ptr, "mouse_x"), RNA_int_get(op->ptr, "mouse_y"));
 
   StripSelection selection = pick_strip_and_handle(scene, v2d, mouse_co.view);
@@ -1696,7 +1658,7 @@ static bool select_more_less_impl(Scene *scene, bool select_more)
 
 static wmOperatorStatus sequencer_select_more_exec(bContext *C, wmOperator * /*op*/)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
 
   if (!select_more_less_impl(scene, true)) {
     return OPERATOR_CANCELLED;
@@ -1732,7 +1694,7 @@ void SEQUENCER_OT_select_more(wmOperatorType *ot)
 
 static wmOperatorStatus sequencer_select_less_exec(bContext *C, wmOperator * /*op*/)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
 
   if (!select_more_less_impl(scene, false)) {
     return OPERATOR_CANCELLED;
@@ -1770,7 +1732,7 @@ static wmOperatorStatus sequencer_select_linked_pick_invoke(bContext *C,
                                                             wmOperator *op,
                                                             const wmEvent *event)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   const View2D *v2d = UI_view2d_fromcontext(C);
 
   bool extend = RNA_boolean_get(op->ptr, "extend");
@@ -1831,7 +1793,7 @@ void SEQUENCER_OT_select_linked_pick(wmOperatorType *ot)
 
 static wmOperatorStatus sequencer_select_linked_exec(bContext *C, wmOperator * /*op*/)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   bool selected;
 
   selected = true;
@@ -1888,10 +1850,10 @@ static const EnumPropertyItem prop_select_handles_side_types[] = {
 
 static wmOperatorStatus sequencer_select_handles_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
   int sel_side = RNA_enum_get(op->ptr, "side");
-  LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+  LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
     if (strip->flag & SELECT) {
       Strip *l_neighbor = find_neighboring_strip(scene, strip, seq::SIDE_LEFT, -1);
       Strip *r_neighbor = find_neighboring_strip(scene, strip, seq::SIDE_RIGHT, -1);
@@ -1937,8 +1899,8 @@ static wmOperatorStatus sequencer_select_handles_exec(bContext *C, wmOperator *o
       }
     }
   }
-  /*   Select strips */
-  LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+  /* Select strips. */
+  LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
     if ((strip->flag & SEQ_LEFTSEL) || (strip->flag & SEQ_RIGHTSEL)) {
       if (!(strip->flag & SELECT)) {
         strip->flag |= SELECT;
@@ -1985,7 +1947,7 @@ void SEQUENCER_OT_select_handles(wmOperatorType *ot)
 
 static wmOperatorStatus sequencer_select_side_of_frame_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
   const bool extend = RNA_boolean_get(op->ptr, "extend");
   const int side = RNA_enum_get(op->ptr, "side");
@@ -2060,7 +2022,7 @@ void SEQUENCER_OT_select_side_of_frame(wmOperatorType *ot)
 
 static wmOperatorStatus sequencer_select_side_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
 
   const int sel_side = RNA_enum_get(op->ptr, "side");
@@ -2070,7 +2032,7 @@ static wmOperatorStatus sequencer_select_side_exec(bContext *C, wmOperator *op)
 
   copy_vn_i(frame_ranges, ARRAY_SIZE(frame_ranges), frame_init);
 
-  LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+  LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
     if (UNLIKELY(strip->channel >= seq::MAX_CHANNELS)) {
       continue;
     }
@@ -2090,7 +2052,7 @@ static wmOperatorStatus sequencer_select_side_exec(bContext *C, wmOperator *op)
     return OPERATOR_CANCELLED;
   }
 
-  select_active_side_range(scene, ed->seqbasep, sel_side, frame_ranges, frame_init);
+  select_active_side_range(scene, ed->current_strips(), sel_side, frame_ranges, frame_init);
 
   ED_outliner_select_sync_from_sequence_tag(C);
 
@@ -2157,7 +2119,7 @@ static void seq_box_select_strip_from_preview(const bContext *C,
                                               const rctf *rect,
                                               const eSelectOp mode)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   Editing *ed = seq::editing_get(scene);
   ListBase *seqbase = seq::active_seqbase_get(ed);
   ListBase *channels = seq::channels_displayed_get(ed);
@@ -2182,7 +2144,7 @@ static void seq_box_select_strip_from_preview(const bContext *C,
 
 static wmOperatorStatus sequencer_box_select_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   View2D *v2d = UI_view2d_fromcontext(C);
   Editing *ed = seq::editing_get(scene);
 
@@ -2219,7 +2181,7 @@ static wmOperatorStatus sequencer_box_select_exec(bContext *C, wmOperator *op)
     return OPERATOR_FINISHED;
   }
 
-  LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+  LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
     rctf rq;
     strip_rectf(scene, strip, &rq);
     if (BLI_rctf_isect(&rq, &rectf, nullptr)) {
@@ -2289,7 +2251,7 @@ static wmOperatorStatus sequencer_box_select_invoke(bContext *C,
                                                     wmOperator *op,
                                                     const wmEvent *event)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   const View2D *v2d = UI_view2d_fromcontext(C);
   ARegion *region = CTX_wm_region(C);
 
@@ -2358,6 +2320,336 @@ void SEQUENCER_OT_select_box(wmOperatorType *ot)
                          "Select strips individually whether or not they are connected");
   RNA_def_property_flag(prop, PROP_SKIP_SAVE);
 }
+/** \} */
+
+/* -------------------------------------------------------------------- */
+/** \name Lasso Select Operator
+ * \{ */
+static bool do_lasso_select_is_origin_inside(const ARegion *region,
+                                             const rcti *clip_rect,
+                                             const Span<int2> mcoords,
+                                             const float co_test[2])
+{
+  int co_screen[2];
+  if (UI_view2d_view_to_region_clip(
+          &region->v2d, co_test[0], co_test[1], &co_screen[0], &co_screen[1]) &&
+      BLI_rcti_isect_pt_v(clip_rect, co_screen) &&
+      BLI_lasso_is_point_inside(mcoords, co_screen[0], co_screen[1], V2D_IS_CLIPPED))
+  {
+    return true;
+  }
+  return false;
+}
+
+static bool rcti_in_lasso(const rcti rect, const Span<int2> mcoords)
+{
+  rcti lasso_rect;
+  BLI_lasso_boundbox(&lasso_rect, mcoords);
+  /* Check if edge of strip is in the lasso. */
+  if (BLI_lasso_is_edge_inside(
+          mcoords, rect.xmin, rect.ymin, rect.xmax, rect.ymin, V2D_IS_CLIPPED) ||
+      BLI_lasso_is_edge_inside(
+          mcoords, rect.xmax, rect.ymin, rect.xmax, rect.ymax, V2D_IS_CLIPPED) ||
+      BLI_lasso_is_edge_inside(
+          mcoords, rect.xmax, rect.ymax, rect.xmin, rect.ymax, V2D_IS_CLIPPED) ||
+      BLI_lasso_is_edge_inside(
+          mcoords, rect.xmin, rect.ymax, rect.xmin, rect.ymin, V2D_IS_CLIPPED))
+  {
+    return true;
+  }
+
+  /* Check if lasso is in the strip rect. Used when the lasso is only inside one strip. */
+  if (BLI_rcti_inside_rcti(&rect, &lasso_rect)) {
+    return true;
+  }
+  return false;
+}
+
+static bool do_lasso_select_timeline(bContext *C,
+                                     const Span<int2> mcoords,
+                                     ARegion *region,
+                                     const eSelectOp sel_op)
+{
+  Scene *scene = CTX_data_scene(C);
+  Editing *ed = seq::editing_get(scene);
+
+  bool changed = false;
+  const bool select = (sel_op != SEL_OP_SUB);
+
+  LISTBASE_FOREACH (Strip *, strip, &ed->seqbase) {
+    rctf strip_rct;
+    rcti region_rct;
+    strip_rectf(scene, strip, &strip_rct);
+    UI_view2d_view_to_region_clip(
+        &region->v2d, strip_rct.xmin, strip_rct.ymin, &region_rct.xmin, &region_rct.ymin);
+    UI_view2d_view_to_region_clip(
+        &region->v2d, strip_rct.xmax, strip_rct.ymax, &region_rct.xmax, &region_rct.ymax);
+
+    if (rcti_in_lasso(region_rct, mcoords)) {
+      SET_FLAG_FROM_TEST(strip->flag, select, SELECT);
+      changed = true;
+    }
+  }
+  return changed;
+}
+
+static bool do_lasso_select_preview(bContext *C,
+                                    Editing *ed,
+                                    const Span<int2> mcoords,
+                                    const eSelectOp sel_op)
+{
+  Scene *scene = CTX_data_scene(C);
+  const ARegion *region = CTX_wm_region(C);
+
+  bool changed = false;
+  rcti rect;
+  BLI_lasso_boundbox(&rect, mcoords);
+
+  ListBase *seqbase = seq::active_seqbase_get(ed);
+  ListBase *channels = seq::channels_displayed_get(ed);
+  SpaceSeq *sseq = CTX_wm_space_seq(C);
+
+  blender::VectorSet strips = seq::query_rendered_strips(
+      scene, channels, seqbase, scene->r.cfra, sseq->chanshown);
+  for (Strip *strip : strips) {
+    float2 origin = seq::image_transform_origin_offset_pixelspace_get(scene, strip);
+    if (do_lasso_select_is_origin_inside(region, &rect, mcoords, origin)) {
+      changed = true;
+      if (ELEM(sel_op, SEL_OP_ADD, SEL_OP_SET)) {
+        strip->flag |= SELECT;
+      }
+      else {
+        BLI_assert(sel_op == SEL_OP_SUB);
+        strip->flag &= ~SELECT;
+      }
+    }
+  }
+
+  return changed;
+}
+
+static wmOperatorStatus vse_lasso_select_exec(bContext *C, wmOperator *op)
+{
+  Scene *scene = CTX_data_scene(C);
+  ARegion *region = CTX_wm_region(C);
+  Array<int2> mcoords = WM_gesture_lasso_path_to_array(C, op);
+  Editing *ed = seq::editing_get(scene);
+
+  if (ed == nullptr) {
+    return OPERATOR_CANCELLED;
+  }
+
+  if (mcoords.is_empty()) {
+    return OPERATOR_PASS_THROUGH;
+  }
+
+  const eSelectOp sel_op = eSelectOp(RNA_enum_get(op->ptr, "mode"));
+  const bool use_pre_deselect = SEL_OP_USE_PRE_DESELECT(sel_op);
+  bool changed = false;
+
+  if (use_pre_deselect) {
+    changed |= deselect_all_strips(scene);
+  }
+
+  if (region->regiontype == RGN_TYPE_PREVIEW) {
+    changed = do_lasso_select_preview(C, ed, mcoords, sel_op);
+  }
+  else {
+    changed = do_lasso_select_timeline(C, mcoords, region, sel_op);
+  }
+
+  if (changed) {
+    sequencer_select_do_updates(C, scene);
+    return OPERATOR_FINISHED;
+  }
+
+  return OPERATOR_CANCELLED;
+}
+
+void SEQUENCER_OT_select_lasso(wmOperatorType *ot)
+{
+  ot->name = "Lasso Select";
+  ot->description = "Select strips using lasso selection";
+  ot->idname = "SEQUENCER_OT_select_lasso";
+
+  ot->invoke = WM_gesture_lasso_invoke;
+  ot->modal = WM_gesture_lasso_modal;
+  ot->exec = vse_lasso_select_exec;
+  ot->poll = ED_operator_sequencer_active;
+  ot->cancel = WM_gesture_lasso_cancel;
+
+  /* flags */
+  ot->flag = OPTYPE_UNDO | OPTYPE_DEPENDS_ON_CURSOR;
+
+  /* properties */
+  WM_operator_properties_gesture_lasso(ot);
+  WM_operator_properties_select_operation_simple(ot);
+}
+/** \} */
+
+/* -------------------------------------------------------------------- */
+/** \name Circle Select Operator
+ * \{ */
+static bool strip_circle_select_radius_image_isect(const Scene *scene,
+                                                   const Strip *strip,
+                                                   const int *radius,
+                                                   const float2 mval)
+{
+  float2 origin = seq::image_transform_origin_offset_pixelspace_get(scene, strip);
+
+  float dx = origin.x - float(mval[0]);
+  float dy = origin.y - float(mval[1]);
+  float dist_sq = sqrt(dx * dx + dy * dy);
+
+  return dist_sq <= *radius;
+}
+
+static void seq_circle_select_strip_from_preview(bContext *C,
+                                                 int radius,
+                                                 const float2 mval,
+                                                 const eSelectOp mode)
+{
+  Scene *scene = CTX_data_scene(C);
+  Editing *ed = seq::editing_get(scene);
+  ListBase *seqbase = seq::active_seqbase_get(ed);
+  ListBase *channels = seq::channels_displayed_get(ed);
+  SpaceSeq *sseq = CTX_wm_space_seq(C);
+
+  VectorSet strips = seq::query_rendered_strips(
+      scene, channels, seqbase, scene->r.cfra, sseq->chanshown);
+  for (Strip *strip : strips) {
+    if (!strip_circle_select_radius_image_isect(scene, strip, &radius, mval)) {
+      continue;
+    }
+
+    if (ELEM(mode, SEL_OP_ADD, SEL_OP_SET)) {
+      strip->flag |= SELECT;
+    }
+    else {
+      BLI_assert(mode == SEL_OP_SUB);
+      strip->flag &= ~SELECT;
+    }
+  }
+}
+
+static bool check_circle_intersection_in_timeline(const rctf *rect,
+                                                  const float xy[2],
+                                                  const float x_radius,
+                                                  const float y_radius)
+{
+  float dx, dy;
+
+  if (xy[0] >= rect->xmin && xy[0] <= rect->xmax) {
+    dx = 0;
+  }
+  else {
+    dx = (xy[0] < rect->xmin) ? (rect->xmin - xy[0]) : (xy[0] - rect->xmax);
+  }
+
+  if (xy[1] >= rect->ymin && xy[1] <= rect->ymax) {
+    dy = 0;
+  }
+  else {
+    dy = (xy[1] < rect->ymin) ? (rect->ymin - xy[1]) : (xy[1] - rect->ymax);
+  }
+
+  return ((dx * dx) / (x_radius * x_radius) + (dy * dy) / (y_radius * y_radius) <= 1.0f);
+}
+static wmOperatorStatus vse_circle_select_exec(bContext *C, wmOperator *op)
+{
+  const int radius = RNA_int_get(op->ptr, "radius");
+  const int mval[2] = {RNA_int_get(op->ptr, "x"), RNA_int_get(op->ptr, "y")};
+  wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
+  const eSelectOp sel_op = eSelectOp(RNA_enum_get(op->ptr, "mode"));
+
+  Scene *scene = CTX_data_scene(C);
+  View2D *v2d = UI_view2d_fromcontext(C);
+  Editing *ed = seq::editing_get(scene);
+  ARegion *region = CTX_wm_region(C);
+
+  const bool use_pre_deselect = SEL_OP_USE_PRE_DESELECT(sel_op);
+
+  if (use_pre_deselect && WM_gesture_is_modal_first(gesture)) {
+    deselect_all_strips(scene);
+    sequencer_select_do_updates(C, scene);
+  }
+
+  if (ed == nullptr) {
+    return OPERATOR_CANCELLED;
+  }
+
+  float2 view_mval;
+  UI_view2d_region_to_view(v2d, mval[0], mval[1], &view_mval[0], &view_mval[1]);
+  float pixel_radius = radius / UI_view2d_scale_get_x(v2d);
+
+  if (region->regiontype == RGN_TYPE_PREVIEW) {
+    seq_circle_select_strip_from_preview(C, pixel_radius, view_mval, sel_op);
+    sequencer_select_do_updates(C, scene);
+    return OPERATOR_FINISHED;
+  }
+
+  float x_radius = radius / UI_view2d_scale_get_x(v2d);
+  float y_radius = radius / UI_view2d_scale_get_y(v2d);
+  bool changed = false;
+  LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
+    rctf rq;
+    strip_rectf(scene, strip, &rq);
+    /* Use custom function to check the distance because in timeline the circle is a ellipse. */
+    if (check_circle_intersection_in_timeline(&rq, view_mval, x_radius, y_radius)) {
+      if (ELEM(sel_op, SEL_OP_ADD, SEL_OP_SET)) {
+        strip->flag |= SELECT;
+      }
+      else {
+        BLI_assert(sel_op == SEL_OP_SUB);
+        strip->flag &= ~SELECT;
+      }
+      changed = true;
+
+      const bool ignore_connections = RNA_boolean_get(op->ptr, "ignore_connections");
+      if (!ignore_connections) {
+        /* Propagate selection to connected strips. */
+        StripSelection selection;
+        selection.strip1 = strip;
+        sequencer_select_connected_strips(selection);
+      }
+    }
+  }
+  if (changed) {
+    sequencer_select_do_updates(C, scene);
+  }
+  return OPERATOR_FINISHED;
+}
+
+void SEQUENCER_OT_select_circle(wmOperatorType *ot)
+{
+  PropertyRNA *prop;
+
+  ot->name = "Circle Select";
+  ot->description = "Select strips using circle selection";
+  ot->idname = "SEQUENCER_OT_select_circle";
+
+  ot->invoke = WM_gesture_circle_invoke;
+  ot->modal = WM_gesture_circle_modal;
+  ot->exec = vse_circle_select_exec;
+
+  ot->poll = ED_operator_sequencer_active;
+
+  ot->get_name = ED_select_circle_get_name;
+
+  /* flags */
+  ot->flag = OPTYPE_UNDO;
+
+  /* properties */
+  WM_operator_properties_gesture_circle(ot);
+  WM_operator_properties_select_operation_simple(ot);
+
+  prop = RNA_def_boolean(ot->srna,
+                         "ignore_connections",
+                         false,
+                         "Ignore Connections",
+                         "Select strips individually whether or not they are connected");
+  RNA_def_property_flag(prop, PROP_SKIP_SAVE);
+}
 
 /** \} */
 
@@ -2400,10 +2692,7 @@ static const EnumPropertyItem sequencer_prop_select_grouped_types[] = {
     {0, nullptr, 0, nullptr, nullptr},
 };
 
-#define STRIP_IS_SOUND(_strip) \
-  ((_strip->type & STRIP_TYPE_SOUND_RAM) && !(_strip->type & STRIP_TYPE_EFFECT))
-
-#define STRIP_IS_EFFECT(_strip) ((_strip->type & STRIP_TYPE_EFFECT) != 0)
+#define STRIP_IS_SOUND(_strip) (_strip->type == STRIP_TYPE_SOUND_RAM)
 
 #define STRIP_USE_DATA(_strip) \
   (ELEM(_strip->type, STRIP_TYPE_SCENE, STRIP_TYPE_MOVIECLIP, STRIP_TYPE_MASK) || \
@@ -2454,11 +2743,11 @@ static bool select_grouped_type_effect(blender::Span<Strip *> strips,
                                        const int channel)
 {
   bool changed = false;
-  const bool is_effect = STRIP_IS_EFFECT(act_strip);
+  const bool is_effect = act_strip->is_effect();
 
   for (Strip *strip : strips) {
     if (STRIP_CHANNEL_CHECK(strip, channel) &&
-        (is_effect ? STRIP_IS_EFFECT(strip) : !STRIP_IS_EFFECT(strip)))
+        (is_effect ? strip->is_effect() : !strip->is_effect()))
     {
       strip->flag |= SELECT;
       changed = true;
@@ -2533,22 +2822,18 @@ static bool select_grouped_effect(blender::Span<Strip *> strips,
                                   const int channel)
 {
   bool changed = false;
-  bool effects[STRIP_TYPE_MAX + 1];
+  blender::Set<StripType> effects;
 
-  for (int i = 0; i <= STRIP_TYPE_MAX; i++) {
-    effects[i] = false;
-  }
-
-  for (Strip *strip : strips) {
-    if (STRIP_CHANNEL_CHECK(strip, channel) && (strip->type & STRIP_TYPE_EFFECT) &&
+  for (const Strip *strip : strips) {
+    if (STRIP_CHANNEL_CHECK(strip, channel) && strip->is_effect() &&
         seq::relation_is_effect_of_strip(strip, act_strip))
     {
-      effects[strip->type] = true;
+      effects.add(StripType(strip->type));
     }
   }
 
   for (Strip *strip : strips) {
-    if (STRIP_CHANNEL_CHECK(strip, channel) && effects[strip->type]) {
+    if (STRIP_CHANNEL_CHECK(strip, channel) && effects.contains(StripType(strip->type))) {
       if (strip->input1) {
         strip->input1->flag |= SELECT;
       }
@@ -2632,12 +2917,11 @@ static bool select_grouped_effect_link(const Scene *scene,
 }
 
 #undef STRIP_IS_SOUND
-#undef STRIP_IS_EFFECT
 #undef STRIP_USE_DATA
 
 static wmOperatorStatus sequencer_select_grouped_exec(bContext *C, wmOperator *op)
 {
-  Scene *scene = CTX_data_scene(C);
+  Scene *scene = CTX_data_sequencer_scene(C);
   ListBase *seqbase = seq::active_seqbase_get(seq::editing_get(scene));
   Strip *act_strip = seq::select_active_get(scene);
 
