@@ -50,7 +50,6 @@ PyDoc_STRVAR(
     "\n"
     "   :arg integration_type: An integration method.\n"
     "   :type integration_type: :class:`IntegrationType`\n");
-
 static int UnaryFunction1DFloat___init__(BPy_UnaryFunction1DFloat *self,
                                          PyObject *args,
                                          PyObject *kwds)
@@ -119,8 +118,7 @@ PyDoc_STRVAR(
     integration_type_doc,
     "The integration method.\n"
     "\n"
-    ":type: :class:`IntegrationType`");
-
+    ":type: :class:`IntegrationType`\n");
 static PyObject *integration_type_get(BPy_UnaryFunction1DFloat *self, void * /*closure*/)
 {
   return BPy_IntegrationType_from_IntegrationType(self->uf1D_float->getIntegrationType());
