@@ -13,9 +13,7 @@
 
 #include "../winged_edge/WEdge.h"
 
-#ifdef WITH_CXX_GUARDEDALLOC
-#  include "MEM_guardedalloc.h"
-#endif
+#include "MEM_guardedalloc.h"
 
 namespace Freestyle {
 
@@ -51,9 +49,7 @@ class OccluderSource {
 
   void buildCachedPolygon();
 
-#ifdef WITH_CXX_GUARDEDALLOC
   MEM_CXX_CLASS_ALLOC_FUNCS("Freestyle:OccluderSource")
-#endif
 };
 
 } /* namespace Freestyle */
