@@ -2,6 +2,11 @@
  *
  * SPDX-License-Identifier: GPL-2.0-or-later */
 
+#include <cmath>
+
+#include "BLI_math_numbers.hh"
+
+#include "BKE_attribute.hh"
 #include "BKE_mesh.hh"
 
 #include "GEO_mesh_primitive_cylinder_cone.hh"
@@ -252,7 +257,7 @@ int ConeConfig::calculate_total_corners()
 static void calculate_cone_verts(const ConeConfig &config, MutableSpan<float3> positions)
 {
   Array<float2> circle(config.circle_segments);
-  const float angle_delta = 2.0f * (M_PI / float(config.circle_segments));
+  const float angle_delta = 2.0f * (math::numbers::pi / float(config.circle_segments));
   float angle = 0.0f;
   for (const int i : IndexRange(config.circle_segments)) {
     circle[i].x = std::cos(angle);
@@ -472,7 +477,7 @@ static void calculate_selection_outputs(const ConeConfig &config,
   if (attribute_outputs.top_id) {
     const bool face = !config.top_is_point && config.fill_type != ConeFillType::None;
     bke::SpanAttributeWriter<bool> selection = attributes.lookup_or_add_for_write_span<bool>(
-        attribute_outputs.top_id.get(), face ? ATTR_DOMAIN_FACE : ATTR_DOMAIN_POINT);
+        *attribute_outputs.top_id, face ? bke::AttrDomain::Face : bke::AttrDomain::Point);
 
     if (config.top_is_point) {
       selection.span[config.first_vert] = true;
@@ -487,7 +492,7 @@ static void calculate_selection_outputs(const ConeConfig &config,
   if (attribute_outputs.bottom_id) {
     const bool face = !config.bottom_is_point && config.fill_type != ConeFillType::None;
     bke::SpanAttributeWriter<bool> selection = attributes.lookup_or_add_for_write_span<bool>(
-        attribute_outputs.bottom_id.get(), face ? ATTR_DOMAIN_FACE : ATTR_DOMAIN_POINT);
+        *attribute_outputs.bottom_id, face ? bke::AttrDomain::Face : bke::AttrDomain::Point);
 
     if (config.bottom_is_point) {
       selection.span[config.last_vert] = true;
@@ -504,7 +509,7 @@ static void calculate_selection_outputs(const ConeConfig &config,
   /* Populate "Side" selection output. */
   if (attribute_outputs.side_id) {
     bke::SpanAttributeWriter<bool> selection = attributes.lookup_or_add_for_write_span<bool>(
-        attribute_outputs.side_id.get(), ATTR_DOMAIN_FACE);
+        *attribute_outputs.side_id, bke::AttrDomain::Face);
 
     selection.span.slice(config.side_faces_start, config.side_faces_len).fill(true);
     selection.finish();
@@ -519,26 +524,24 @@ static void calculate_selection_outputs(const ConeConfig &config,
  * If the mesh is a truncated cone or a cylinder, the side faces are unwrapped into
  * a rectangle that fills the top half of the UV (or the entire UV, if there are no fills).
  */
-static void calculate_cone_uvs(const ConeConfig &config,
-                               Mesh *mesh,
-                               const bke::AttributeIDRef &uv_map_id)
+static void calculate_cone_uvs(const ConeConfig &config, Mesh *mesh, const StringRef uv_map_id)
 {
   bke::MutableAttributeAccessor attributes = mesh->attributes_for_write();
 
   bke::SpanAttributeWriter<float2> uv_attribute =
-      attributes.lookup_or_add_for_write_only_span<float2>(uv_map_id, ATTR_DOMAIN_CORNER);
+      attributes.lookup_or_add_for_write_only_span<float2>(uv_map_id, bke::AttrDomain::Corner);
   MutableSpan<float2> uvs = uv_attribute.span;
 
   Array<float2> circle(config.circle_segments);
   float angle = 0.0f;
-  const float angle_delta = 2.0f * M_PI / float(config.circle_segments);
+  const float angle_delta = 2.0f * math::numbers::pi / float(config.circle_segments);
   for (const int i : IndexRange(config.circle_segments)) {
     circle[i].x = std::cos(angle) * 0.225f;
     circle[i].y = std::sin(angle) * 0.225f;
     angle += angle_delta;
   }
 
-  int loop_index = 0;
+  int corner = 0;
 
   /* Left circle of the UV representing the top fill or top cone tip. */
   if (config.top_is_point || config.fill_type != ConeFillType::None) {
@@ -551,16 +554,16 @@ static void calculate_cone_uvs(const ConeConfig &config,
     if (config.top_has_center_vert) {
       /* Cone tip itself or triangle fan center of the fill. */
       for (const int i : IndexRange(config.circle_segments)) {
-        uvs[loop_index++] = radius_factor_delta * circle[i] + center_left;
-        uvs[loop_index++] = radius_factor_delta * circle[(i + 1) % config.circle_segments] +
-                            center_left;
-        uvs[loop_index++] = center_left;
+        uvs[corner++] = radius_factor_delta * circle[i] + center_left;
+        uvs[corner++] = radius_factor_delta * circle[(i + 1) % config.circle_segments] +
+                        center_left;
+        uvs[corner++] = center_left;
       }
     }
     else if (!config.top_is_point && config.fill_type == ConeFillType::NGon) {
       /* N-gon at the center of the fill. */
       for (const int i : IndexRange(config.circle_segments)) {
-        uvs[loop_index++] = radius_factor_delta * circle[i] + center_left;
+        uvs[corner++] = radius_factor_delta * circle[i] + center_left;
       }
     }
     /* The rest of the top fill is made out of quad rings. */
@@ -568,12 +571,12 @@ static void calculate_cone_uvs(const ConeConfig &config,
       const float inner_radius_factor = i * radius_factor_delta;
       const float outer_radius_factor = (i + 1) * radius_factor_delta;
       for (const int j : IndexRange(config.circle_segments)) {
-        uvs[loop_index++] = inner_radius_factor * circle[j] + center_left;
-        uvs[loop_index++] = outer_radius_factor * circle[j] + center_left;
-        uvs[loop_index++] = outer_radius_factor * circle[(j + 1) % config.circle_segments] +
-                            center_left;
-        uvs[loop_index++] = inner_radius_factor * circle[(j + 1) % config.circle_segments] +
-                            center_left;
+        uvs[corner++] = inner_radius_factor * circle[j] + center_left;
+        uvs[corner++] = outer_radius_factor * circle[j] + center_left;
+        uvs[corner++] = outer_radius_factor * circle[(j + 1) % config.circle_segments] +
+                        center_left;
+        uvs[corner++] = inner_radius_factor * circle[(j + 1) % config.circle_segments] +
+                        center_left;
       }
     }
   }
@@ -586,10 +589,10 @@ static void calculate_cone_uvs(const ConeConfig &config,
 
     for (const int i : IndexRange(config.side_segments)) {
       for (const int j : IndexRange(config.circle_segments)) {
-        uvs[loop_index++] = float2(j * x_delta, i * y_delta + bottom);
-        uvs[loop_index++] = float2(j * x_delta, (i + 1) * y_delta + bottom);
-        uvs[loop_index++] = float2((j + 1) * x_delta, (i + 1) * y_delta + bottom);
-        uvs[loop_index++] = float2((j + 1) * x_delta, i * y_delta + bottom);
+        uvs[corner++] = float2(j * x_delta, i * y_delta + bottom);
+        uvs[corner++] = float2(j * x_delta, (i + 1) * y_delta + bottom);
+        uvs[corner++] = float2((j + 1) * x_delta, (i + 1) * y_delta + bottom);
+        uvs[corner++] = float2((j + 1) * x_delta, i * y_delta + bottom);
       }
     }
   }
@@ -608,30 +611,30 @@ static void calculate_cone_uvs(const ConeConfig &config,
       const float outer_radius_factor = 1.0f - i * radius_factor_delta;
       const float inner_radius_factor = 1.0f - (i + 1) * radius_factor_delta;
       for (const int j : IndexRange(config.circle_segments)) {
-        uvs[loop_index++] = outer_radius_factor * circle[j] + center_right;
-        uvs[loop_index++] = inner_radius_factor * circle[j] + center_right;
-        uvs[loop_index++] = inner_radius_factor * circle[(j + 1) % config.circle_segments] +
-                            center_right;
-        uvs[loop_index++] = outer_radius_factor * circle[(j + 1) % config.circle_segments] +
-                            center_right;
+        uvs[corner++] = outer_radius_factor * circle[j] + center_right;
+        uvs[corner++] = inner_radius_factor * circle[j] + center_right;
+        uvs[corner++] = inner_radius_factor * circle[(j + 1) % config.circle_segments] +
+                        center_right;
+        uvs[corner++] = outer_radius_factor * circle[(j + 1) % config.circle_segments] +
+                        center_right;
       }
     }
 
     if (config.bottom_has_center_vert) {
       /* Cone tip itself or triangle fan center of the fill. */
       for (const int i : IndexRange(config.circle_segments)) {
-        uvs[loop_index++] = radius_factor_delta * circle[i] + center_right;
-        uvs[loop_index++] = center_right;
-        uvs[loop_index++] = radius_factor_delta * circle[(i + 1) % config.circle_segments] +
-                            center_right;
+        uvs[corner++] = radius_factor_delta * circle[i] + center_right;
+        uvs[corner++] = center_right;
+        uvs[corner++] = radius_factor_delta * circle[(i + 1) % config.circle_segments] +
+                        center_right;
       }
     }
     else if (!config.bottom_is_point && config.fill_type == ConeFillType::NGon) {
       /* N-gon at the center of the fill. */
       for (const int i : IndexRange(config.circle_segments)) {
         /* Go backwards because of reversed face normal. */
-        uvs[loop_index++] = radius_factor_delta * circle[config.circle_segments - 1 - i] +
-                            center_right;
+        uvs[corner++] = radius_factor_delta * circle[config.circle_segments - 1 - i] +
+                        center_right;
       }
     }
   }
@@ -685,19 +688,20 @@ Mesh *create_cylinder_or_cone_mesh(const float radius_top,
   MutableSpan<int> face_offsets = mesh->face_offsets_for_write();
   MutableSpan<int> corner_verts = mesh->corner_verts_for_write();
   MutableSpan<int> corner_edges = mesh->corner_edges_for_write();
-  BKE_mesh_smooth_flag_set(mesh, false);
+  bke::mesh_smooth_set(*mesh, false);
 
   calculate_cone_verts(config, positions);
   calculate_cone_edges(config, edges);
   calculate_cone_faces(config, corner_verts, corner_edges, face_offsets.drop_back(1));
   offset_indices::accumulate_counts_to_offsets(face_offsets);
   if (attribute_outputs.uv_map_id) {
-    calculate_cone_uvs(config, mesh, attribute_outputs.uv_map_id.get());
+    calculate_cone_uvs(config, mesh, *attribute_outputs.uv_map_id);
   }
   calculate_selection_outputs(config, attribute_outputs, mesh->attributes_for_write());
 
   mesh->tag_loose_verts_none();
   mesh->tag_loose_edges_none();
+  mesh->tag_overlapping_none();
   mesh->bounds_set_eager(calculate_bounds_cylinder(config));
 
   return mesh;
