@@ -9,7 +9,7 @@
 #pragma once
 
 /** NOTE: Keep in sync with eUserPref_GPUBackendType. */
-enum eGPUBackendType {
+enum GPUBackendType {
   GPU_BACKEND_NONE = 0,
   GPU_BACKEND_OPENGL = 1 << 0,
   GPU_BACKEND_METAL = 1 << 1,
