@@ -54,7 +54,7 @@ Texture::~Texture()
 #endif
 }
 
-bool Texture::init_1D(int w, int layers, int mip_len, eGPUTextureFormat format)
+bool Texture::init_1D(int w, int layers, int mip_len, TextureFormat format)
 {
   w_ = w;
   h_ = layers;
@@ -70,7 +70,7 @@ bool Texture::init_1D(int w, int layers, int mip_len, eGPUTextureFormat format)
   return this->init_internal();
 }
 
-bool Texture::init_2D(int w, int h, int layers, int mip_len, eGPUTextureFormat format)
+bool Texture::init_2D(int w, int h, int layers, int mip_len, TextureFormat format)
 {
   w_ = w;
   h_ = h;
@@ -86,7 +86,7 @@ bool Texture::init_2D(int w, int h, int layers, int mip_len, eGPUTextureFormat f
   return this->init_internal();
 }
 
-bool Texture::init_3D(int w, int h, int d, int mip_len, eGPUTextureFormat format)
+bool Texture::init_3D(int w, int h, int d, int mip_len, TextureFormat format)
 {
   w_ = w;
   h_ = h;
@@ -102,7 +102,7 @@ bool Texture::init_3D(int w, int h, int d, int mip_len, eGPUTextureFormat format
   return this->init_internal();
 }
 
-bool Texture::init_cubemap(int w, int layers, int mip_len, eGPUTextureFormat format)
+bool Texture::init_cubemap(int w, int layers, int mip_len, TextureFormat format)
 {
   w_ = w;
   h_ = w;
@@ -118,12 +118,9 @@ bool Texture::init_cubemap(int w, int layers, int mip_len, eGPUTextureFormat for
   return this->init_internal();
 }
 
-bool Texture::init_buffer(VertBuf *vbo, eGPUTextureFormat format)
+bool Texture::init_buffer(VertBuf *vbo, TextureFormat format)
 {
   /* See to_texture_format(). */
-  if (format == GPU_DEPTH_COMPONENT24) {
-    return false;
-  }
   w_ = GPU_vertbuf_get_vertex_len(vbo);
   h_ = 0;
   d_ = 0;
@@ -133,9 +130,9 @@ bool Texture::init_buffer(VertBuf *vbo, eGPUTextureFormat format)
   return this->init_internal(vbo);
 }
 
-bool Texture::init_view(GPUTexture *src_,
-                        eGPUTextureFormat format,
-                        eGPUTextureType type,
+bool Texture::init_view(Texture *src,
+                        TextureFormat format,
+                        GPUTextureType type,
                         int mip_start,
                         int mip_len,
                         int layer_start,
@@ -143,7 +140,6 @@ bool Texture::init_view(GPUTexture *src_,
                         bool cube_as_array,
                         bool use_stencil)
 {
-  const Texture *src = unwrap(src_);
   w_ = src->w_;
   h_ = src->h_;
   d_ = src->d_;
@@ -174,7 +170,7 @@ bool Texture::init_view(GPUTexture *src_,
     type_ = (type_ & ~GPU_TEXTURE_CUBE) | GPU_TEXTURE_2D_ARRAY;
   }
   sampler_state = src->sampler_state;
-  return this->init_internal(src_, mip_start, layer_start, use_stencil);
+  return this->init_internal(src, mip_start, layer_start, use_stencil);
 }
 
 void Texture::usage_set(eGPUTextureUsage usage_flags)
@@ -253,16 +249,16 @@ uint GPU_texture_memory_usage_get()
 
 /* ------ Creation ------ */
 
-static inline GPUTexture *gpu_texture_create(const char *name,
-                                             const int w,
-                                             const int h,
-                                             const int d,
-                                             const eGPUTextureType type,
-                                             int mip_len,
-                                             eGPUTextureFormat tex_format,
-                                             eGPUTextureUsage usage,
-                                             const void *pixels,
-                                             eGPUDataFormat data_format = GPU_DATA_FLOAT)
+static inline gpu::Texture *gpu_texture_create(const char *name,
+                                               const int w,
+                                               const int h,
+                                               const int d,
+                                               const GPUTextureType type,
+                                               int mip_len,
+                                               TextureFormat tex_format,
+                                               eGPUTextureUsage usage,
+                                               const void *pixels,
+                                               eGPUDataFormat data_format = GPU_DATA_FLOAT)
 {
   BLI_assert(mip_len > 0);
   Texture *tex = GPUBackend::get()->texture_alloc(name);
@@ -296,97 +292,97 @@ static inline GPUTexture *gpu_texture_create(const char *name,
   if (pixels) {
     tex->update(data_format, pixels);
   }
-  return reinterpret_cast<GPUTexture *>(tex);
+  return reinterpret_cast<gpu::Texture *>(tex);
 }
 
-GPUTexture *GPU_texture_create_1d(const char *name,
-                                  int width,
-                                  int mip_len,
-                                  eGPUTextureFormat format,
-                                  eGPUTextureUsage usage,
-                                  const float *data)
+gpu::Texture *GPU_texture_create_1d(const char *name,
+                                    int width,
+                                    int mip_len,
+                                    TextureFormat format,
+                                    eGPUTextureUsage usage,
+                                    const float *data)
 {
   return gpu_texture_create(name, width, 0, 0, GPU_TEXTURE_1D, mip_len, format, usage, data);
 }
 
-GPUTexture *GPU_texture_create_1d_array(const char *name,
-                                        int width,
-                                        int layer_len,
-                                        int mip_len,
-                                        eGPUTextureFormat format,
-                                        eGPUTextureUsage usage,
-                                        const float *data)
+gpu::Texture *GPU_texture_create_1d_array(const char *name,
+                                          int width,
+                                          int layer_len,
+                                          int mip_len,
+                                          TextureFormat format,
+                                          eGPUTextureUsage usage,
+                                          const float *data)
 {
   return gpu_texture_create(
       name, width, layer_len, 0, GPU_TEXTURE_1D_ARRAY, mip_len, format, usage, data);
 }
 
-GPUTexture *GPU_texture_create_2d(const char *name,
-                                  int width,
-                                  int height,
-                                  int mip_len,
-                                  eGPUTextureFormat format,
-                                  eGPUTextureUsage usage,
-                                  const float *data)
+gpu::Texture *GPU_texture_create_2d(const char *name,
+                                    int width,
+                                    int height,
+                                    int mip_len,
+                                    TextureFormat format,
+                                    eGPUTextureUsage usage,
+                                    const float *data)
 {
   return gpu_texture_create(name, width, height, 0, GPU_TEXTURE_2D, mip_len, format, usage, data);
 }
 
-GPUTexture *GPU_texture_create_2d_array(const char *name,
-                                        int width,
-                                        int height,
-                                        int layer_len,
-                                        int mip_len,
-                                        eGPUTextureFormat format,
-                                        eGPUTextureUsage usage,
-                                        const float *data)
+gpu::Texture *GPU_texture_create_2d_array(const char *name,
+                                          int width,
+                                          int height,
+                                          int layer_len,
+                                          int mip_len,
+                                          TextureFormat format,
+                                          eGPUTextureUsage usage,
+                                          const float *data)
 {
   return gpu_texture_create(
       name, width, height, layer_len, GPU_TEXTURE_2D_ARRAY, mip_len, format, usage, data);
 }
 
-GPUTexture *GPU_texture_create_3d(const char *name,
-                                  int width,
-                                  int height,
-                                  int depth,
-                                  int mip_len,
-                                  eGPUTextureFormat texture_format,
-                                  eGPUTextureUsage usage,
-                                  const void *data)
+gpu::Texture *GPU_texture_create_3d(const char *name,
+                                    int width,
+                                    int height,
+                                    int depth,
+                                    int mip_len,
+                                    TextureFormat texture_format,
+                                    eGPUTextureUsage usage,
+                                    const void *data)
 {
   return gpu_texture_create(
       name, width, height, depth, GPU_TEXTURE_3D, mip_len, texture_format, usage, data);
 }
 
-GPUTexture *GPU_texture_create_cube(const char *name,
-                                    int width,
-                                    int mip_len,
-                                    eGPUTextureFormat format,
-                                    eGPUTextureUsage usage,
-                                    const float *data)
+gpu::Texture *GPU_texture_create_cube(const char *name,
+                                      int width,
+                                      int mip_len,
+                                      TextureFormat format,
+                                      eGPUTextureUsage usage,
+                                      const float *data)
 {
   return gpu_texture_create(name, width, width, 0, GPU_TEXTURE_CUBE, mip_len, format, usage, data);
 }
 
-GPUTexture *GPU_texture_create_cube_array(const char *name,
-                                          int width,
-                                          int layer_len,
-                                          int mip_len,
-                                          eGPUTextureFormat format,
-                                          eGPUTextureUsage usage,
-                                          const float *data)
+gpu::Texture *GPU_texture_create_cube_array(const char *name,
+                                            int width,
+                                            int layer_len,
+                                            int mip_len,
+                                            TextureFormat format,
+                                            eGPUTextureUsage usage,
+                                            const float *data)
 {
   return gpu_texture_create(
       name, width, width, layer_len, GPU_TEXTURE_CUBE_ARRAY, mip_len, format, usage, data);
 }
 
-GPUTexture *GPU_texture_create_compressed_2d(const char *name,
-                                             int width,
-                                             int height,
-                                             int mip_len,
-                                             eGPUTextureFormat tex_format,
-                                             eGPUTextureUsage usage,
-                                             const void *data)
+gpu::Texture *GPU_texture_create_compressed_2d(const char *name,
+                                               int width,
+                                               int height,
+                                               int mip_len,
+                                               TextureFormat tex_format,
+                                               eGPUTextureUsage usage,
+                                               const void *data)
 {
   Texture *tex = GPUBackend::get()->texture_alloc(name);
   tex->usage_set(usage);
@@ -403,15 +399,16 @@ GPUTexture *GPU_texture_create_compressed_2d(const char *name,
       tex->mip_size_get(mip, extent);
 
       size_t size = ((extent[0] + 3) / 4) * ((extent[1] + 3) / 4) * to_block_size(tex_format);
-      tex->update_sub(mip, offset, extent, to_data_format(tex_format), (uchar *)data + ofs);
+      tex->update_sub(
+          mip, offset, extent, to_texture_data_format(tex_format), (uchar *)data + ofs);
 
       ofs += size;
     }
   }
-  return reinterpret_cast<GPUTexture *>(tex);
+  return reinterpret_cast<gpu::Texture *>(tex);
 }
 
-GPUTexture *GPU_texture_create_from_vertbuf(const char *name, blender::gpu::VertBuf *vert)
+gpu::Texture *GPU_texture_create_from_vertbuf(const char *name, gpu::VertBuf *vert)
 {
 #ifndef NDEBUG
   /* Vertex buffers used for texture buffers must be flagged with:
@@ -420,7 +417,7 @@ GPUTexture *GPU_texture_create_from_vertbuf(const char *name, blender::gpu::Vert
                  "Vertex Buffers used for textures should have usage flag "
                  "GPU_USAGE_FLAG_BUFFER_TEXTURE_ONLY.");
 #endif
-  eGPUTextureFormat tex_format = to_texture_format(GPU_vertbuf_get_format(vert));
+  TextureFormat tex_format = to_texture_format(GPU_vertbuf_get_format(vert));
   Texture *tex = GPUBackend::get()->texture_alloc(name);
 
   bool success = tex->init_buffer(vert, tex_format);
@@ -428,33 +425,40 @@ GPUTexture *GPU_texture_create_from_vertbuf(const char *name, blender::gpu::Vert
     delete tex;
     return nullptr;
   }
-  return reinterpret_cast<GPUTexture *>(tex);
+  return reinterpret_cast<gpu::Texture *>(tex);
 }
 
-GPUTexture *GPU_texture_create_error(int dimension, bool is_array)
+gpu::Texture *GPU_texture_create_error(int dimension, bool is_array)
 {
   const float pixel[4] = {1.0f, 0.0f, 1.0f, 1.0f};
   int w = 1;
   int h = (dimension < 2 && !is_array) ? 0 : 1;
   int d = (dimension < 3 && !is_array) ? 0 : 1;
 
-  eGPUTextureType type = GPU_TEXTURE_3D;
+  GPUTextureType type = GPU_TEXTURE_3D;
   type = (dimension == 2) ? (is_array ? GPU_TEXTURE_2D_ARRAY : GPU_TEXTURE_2D) : type;
   type = (dimension == 1) ? (is_array ? GPU_TEXTURE_1D_ARRAY : GPU_TEXTURE_1D) : type;
 
-  return gpu_texture_create(
-      "invalid_tex", w, h, d, type, 1, GPU_RGBA8, GPU_TEXTURE_USAGE_GENERAL, pixel);
+  return gpu_texture_create("invalid_tex",
+                            w,
+                            h,
+                            d,
+                            type,
+                            1,
+                            TextureFormat::UNORM_8_8_8_8,
+                            GPU_TEXTURE_USAGE_GENERAL,
+                            pixel);
 }
 
-GPUTexture *GPU_texture_create_view(const char *name,
-                                    GPUTexture *source_texture,
-                                    eGPUTextureFormat view_format,
-                                    int mip_start,
-                                    int mip_len,
-                                    int layer_start,
-                                    int layer_len,
-                                    bool cube_as_array,
-                                    bool use_stencil)
+gpu::Texture *GPU_texture_create_view(const char *name,
+                                      gpu::Texture *source_texture,
+                                      TextureFormat view_format,
+                                      int mip_start,
+                                      int mip_len,
+                                      int layer_start,
+                                      int layer_len,
+                                      bool cube_as_array,
+                                      bool use_stencil)
 {
   BLI_assert(mip_len > 0);
   BLI_assert(layer_len > 0);
@@ -469,18 +473,18 @@ GPUTexture *GPU_texture_create_view(const char *name,
   Texture *view = GPUBackend::get()->texture_alloc(name);
   view->init_view(source_texture,
                   view_format,
-                  unwrap(source_texture)->type_get(),
+                  source_texture->type_get(),
                   mip_start,
                   mip_len,
                   layer_start,
                   layer_len,
                   cube_as_array,
                   use_stencil);
-  return wrap(view);
+  return view;
 }
 
 /* ------ Usage ------ */
-eGPUTextureUsage GPU_texture_usage(const GPUTexture *texture_)
+eGPUTextureUsage GPU_texture_usage(const gpu::Texture *texture_)
 {
   const Texture *tex = reinterpret_cast<const Texture *>(texture_);
   return tex->usage_get();
@@ -488,17 +492,17 @@ eGPUTextureUsage GPU_texture_usage(const GPUTexture *texture_)
 
 /* ------ Update ------ */
 
-void GPU_texture_update_mipmap(GPUTexture *texture,
+void GPU_texture_update_mipmap(gpu::Texture *texture,
                                int mip_level,
                                eGPUDataFormat data_format,
                                const void *pixels)
 {
   int extent[3] = {1, 1, 1}, offset[3] = {0, 0, 0};
-  unwrap(texture)->mip_size_get(mip_level, extent);
-  unwrap(texture)->update_sub(mip_level, offset, extent, data_format, pixels);
+  texture->mip_size_get(mip_level, extent);
+  texture->update_sub(mip_level, offset, extent, data_format, pixels);
 }
 
-void GPU_texture_update_sub(GPUTexture *tex,
+void GPU_texture_update_sub(gpu::Texture *tex,
                             eGPUDataFormat data_format,
                             const void *pixels,
                             int offset_x,
@@ -510,10 +514,10 @@ void GPU_texture_update_sub(GPUTexture *tex,
 {
   int offset[3] = {offset_x, offset_y, offset_z};
   int extent[3] = {width, height, depth};
-  unwrap(tex)->update_sub(0, offset, extent, data_format, pixels);
+  tex->update_sub(0, offset, extent, data_format, pixels);
 }
 
-void GPU_texture_update_sub_from_pixel_buffer(GPUTexture *texture,
+void GPU_texture_update_sub_from_pixel_buffer(gpu::Texture *texture,
                                               eGPUDataFormat data_format,
                                               GPUPixelBuffer *pixel_buf,
                                               int offset_x,
@@ -525,27 +529,27 @@ void GPU_texture_update_sub_from_pixel_buffer(GPUTexture *texture,
 {
   int offset[3] = {offset_x, offset_y, offset_z};
   int extent[3] = {width, height, depth};
-  unwrap(texture)->update_sub(offset, extent, data_format, pixel_buf);
+  texture->update_sub(offset, extent, data_format, pixel_buf);
 }
 
-void *GPU_texture_read(GPUTexture *texture, eGPUDataFormat data_format, int mip_level)
+void *GPU_texture_read(gpu::Texture *texture, eGPUDataFormat data_format, int mip_level)
 {
   BLI_assert_msg(
       GPU_texture_usage(texture) & GPU_TEXTURE_USAGE_HOST_READ,
       "The host-read usage flag must be specified up-front. Only textures which require data "
       "reads should be flagged, allowing the backend to make certain optimizations.");
-  return unwrap(texture)->read(mip_level, data_format);
+  return texture->read(mip_level, data_format);
 }
 
-void GPU_texture_clear(GPUTexture *tex, eGPUDataFormat data_format, const void *data)
+void GPU_texture_clear(gpu::Texture *tex, eGPUDataFormat data_format, const void *data)
 {
   BLI_assert(data != nullptr); /* Do not accept nullptr as parameter. */
-  unwrap(tex)->clear(data_format, data);
+  tex->clear(data_format, data);
 }
 
-void GPU_texture_update(GPUTexture *tex, eGPUDataFormat data_format, const void *data)
+void GPU_texture_update(gpu::Texture *tex, eGPUDataFormat data_format, const void *data)
 {
-  unwrap(tex)->update(data_format, data);
+  tex->update(data_format, data);
 }
 
 void GPU_unpack_row_length_set(uint len)
@@ -555,22 +559,22 @@ void GPU_unpack_row_length_set(uint len)
 
 /* ------ Binding ------ */
 
-void GPU_texture_bind_ex(GPUTexture *texture, GPUSamplerState state, int unit)
+void GPU_texture_bind_ex(gpu::Texture *texture, GPUSamplerState state, int unit)
 {
-  Texture *tex = unwrap(texture);
+  Texture *tex = texture;
   state = (state.type == GPU_SAMPLER_STATE_TYPE_INTERNAL) ? tex->sampler_state : state;
   Context::get()->state_manager->texture_bind(tex, state, unit);
 }
 
-void GPU_texture_bind(GPUTexture *texture, int unit)
+void GPU_texture_bind(gpu::Texture *texture, int unit)
 {
-  Texture *tex = unwrap(texture);
+  Texture *tex = texture;
   Context::get()->state_manager->texture_bind(tex, tex->sampler_state, unit);
 }
 
-void GPU_texture_unbind(GPUTexture *texture)
+void GPU_texture_unbind(gpu::Texture *texture)
 {
-  Texture *tex = unwrap(texture);
+  Texture *tex = texture;
   Context::get()->state_manager->texture_unbind(tex);
 }
 
@@ -579,14 +583,14 @@ void GPU_texture_unbind_all()
   Context::get()->state_manager->texture_unbind_all();
 }
 
-void GPU_texture_image_bind(GPUTexture *tex, int unit)
+void GPU_texture_image_bind(gpu::Texture *tex, int unit)
 {
-  Context::get()->state_manager->image_bind(unwrap(tex), unit);
+  Context::get()->state_manager->image_bind(tex, unit);
 }
 
-void GPU_texture_image_unbind(GPUTexture *tex)
+void GPU_texture_image_unbind(gpu::Texture *tex)
 {
-  Context::get()->state_manager->image_unbind(unwrap(tex));
+  Context::get()->state_manager->image_unbind(tex);
 }
 
 void GPU_texture_image_unbind_all()
@@ -594,21 +598,21 @@ void GPU_texture_image_unbind_all()
   Context::get()->state_manager->image_unbind_all();
 }
 
-void GPU_texture_update_mipmap_chain(GPUTexture *tex)
+void GPU_texture_update_mipmap_chain(gpu::Texture *tex)
 {
-  unwrap(tex)->generate_mipmap();
+  tex->generate_mipmap();
 }
 
-void GPU_texture_copy(GPUTexture *dst_, GPUTexture *src_)
+void GPU_texture_copy(gpu::Texture *dst_, gpu::Texture *src_)
 {
-  Texture *src = unwrap(src_);
-  Texture *dst = unwrap(dst_);
+  Texture *src = src_;
+  Texture *dst = dst_;
   src->copy_to(dst);
 }
 
-void GPU_texture_compare_mode(GPUTexture *texture, bool use_compare)
+void GPU_texture_compare_mode(gpu::Texture *texture, bool use_compare)
 {
-  Texture *tex = unwrap(texture);
+  Texture *tex = texture;
   /* Only depth formats does support compare mode. */
   BLI_assert(!(use_compare) || (tex->format_flag_get() & GPU_FORMAT_DEPTH));
 
@@ -617,18 +621,18 @@ void GPU_texture_compare_mode(GPUTexture *texture, bool use_compare)
   tex->sampler_state.custom_type = GPU_SAMPLER_CUSTOM_COMPARE;
 }
 
-void GPU_texture_filter_mode(GPUTexture *texture, bool use_filter)
+void GPU_texture_filter_mode(gpu::Texture *texture, bool use_filter)
 {
-  Texture *tex = unwrap(texture);
+  Texture *tex = texture;
   /* Stencil and integer format does not support filtering. */
   BLI_assert(!(use_filter) ||
              !(tex->format_flag_get() & (GPU_FORMAT_STENCIL | GPU_FORMAT_INTEGER)));
   tex->sampler_state.set_filtering_flag_from_test(GPU_SAMPLER_FILTERING_LINEAR, use_filter);
 }
 
-void GPU_texture_mipmap_mode(GPUTexture *texture, bool use_mipmap, bool use_filter)
+void GPU_texture_mipmap_mode(gpu::Texture *texture, bool use_mipmap, bool use_filter)
 {
-  Texture *tex = unwrap(texture);
+  Texture *tex = texture;
   /* Stencil and integer format does not support filtering. */
   BLI_assert(!(use_filter || use_mipmap) ||
              !(tex->format_flag_get() & (GPU_FORMAT_STENCIL | GPU_FORMAT_INTEGER)));
@@ -636,43 +640,43 @@ void GPU_texture_mipmap_mode(GPUTexture *texture, bool use_mipmap, bool use_filt
   tex->sampler_state.set_filtering_flag_from_test(GPU_SAMPLER_FILTERING_LINEAR, use_filter);
 }
 
-void GPU_texture_anisotropic_filter(GPUTexture *texture, bool use_aniso)
+void GPU_texture_anisotropic_filter(gpu::Texture *texture, bool use_aniso)
 {
-  Texture *tex = unwrap(texture);
+  Texture *tex = texture;
   /* Stencil and integer format does not support filtering. */
   BLI_assert(!(use_aniso) ||
              !(tex->format_flag_get() & (GPU_FORMAT_STENCIL | GPU_FORMAT_INTEGER)));
   tex->sampler_state.set_filtering_flag_from_test(GPU_SAMPLER_FILTERING_ANISOTROPIC, use_aniso);
 }
 
-void GPU_texture_extend_mode_x(GPUTexture *texture, GPUSamplerExtendMode extend_mode)
+void GPU_texture_extend_mode_x(gpu::Texture *texture, GPUSamplerExtendMode extend_mode)
 {
-  unwrap(texture)->sampler_state.extend_x = extend_mode;
+  texture->sampler_state.extend_x = extend_mode;
 }
 
-void GPU_texture_extend_mode_y(GPUTexture *texture, GPUSamplerExtendMode extend_mode)
+void GPU_texture_extend_mode_y(gpu::Texture *texture, GPUSamplerExtendMode extend_mode)
 {
-  unwrap(texture)->sampler_state.extend_yz = extend_mode;
+  texture->sampler_state.extend_yz = extend_mode;
 }
 
-void GPU_texture_extend_mode(GPUTexture *texture, GPUSamplerExtendMode extend_mode)
+void GPU_texture_extend_mode(gpu::Texture *texture, GPUSamplerExtendMode extend_mode)
 {
-  unwrap(texture)->sampler_state.extend_x = extend_mode;
-  unwrap(texture)->sampler_state.extend_yz = extend_mode;
+  texture->sampler_state.extend_x = extend_mode;
+  texture->sampler_state.extend_yz = extend_mode;
 }
 
-void GPU_texture_swizzle_set(GPUTexture *texture, const char swizzle[4])
+void GPU_texture_swizzle_set(gpu::Texture *texture, const char swizzle[4])
 {
-  unwrap(texture)->swizzle_set(swizzle);
+  texture->swizzle_set(swizzle);
 }
 
-void GPU_texture_free(GPUTexture *texture)
+void GPU_texture_free(gpu::Texture *texture)
 {
-  Texture *tex = unwrap(texture);
+  Texture *tex = texture;
   tex->refcount--;
 
   if (tex->refcount < 0) {
-    fprintf(stderr, "GPUTexture: negative refcount\n");
+    fprintf(stderr, "gpu::Texture: negative refcount\n");
   }
 
   if (tex->refcount == 0) {
@@ -680,14 +684,14 @@ void GPU_texture_free(GPUTexture *texture)
   }
 }
 
-void GPU_texture_ref(GPUTexture *texture)
+void GPU_texture_ref(gpu::Texture *texture)
 {
-  unwrap(texture)->refcount++;
+  texture->refcount++;
 }
 
-int GPU_texture_dimensions(const GPUTexture *texture)
+int GPU_texture_dimensions(const gpu::Texture *texture)
 {
-  eGPUTextureType type = unwrap(texture)->type_get();
+  GPUTextureType type = texture->type_get();
   if (type & GPU_TEXTURE_1D) {
     return 1;
   }
@@ -704,257 +708,251 @@ int GPU_texture_dimensions(const GPUTexture *texture)
   return 1;
 }
 
-int GPU_texture_width(const GPUTexture *texture)
+int GPU_texture_width(const gpu::Texture *texture)
 {
-  return unwrap(texture)->width_get();
+  return texture->width_get();
 }
 
-int GPU_texture_height(const GPUTexture *texture)
+int GPU_texture_height(const gpu::Texture *texture)
 {
-  return unwrap(texture)->height_get();
+  return texture->height_get();
 }
 
-int GPU_texture_depth(const GPUTexture *texture)
+int GPU_texture_depth(const gpu::Texture *texture)
 {
-  return unwrap(texture)->depth_get();
+  return texture->depth_get();
 }
 
-int GPU_texture_layer_count(const GPUTexture *texture)
+int GPU_texture_layer_count(const gpu::Texture *texture)
 {
-  return unwrap(texture)->layer_count();
+  return texture->layer_count();
 }
 
-int GPU_texture_mip_count(const GPUTexture *texture)
+int GPU_texture_mip_count(const gpu::Texture *texture)
 {
-  return unwrap(texture)->mip_count();
+  return texture->mip_count();
 }
 
-int GPU_texture_original_width(const GPUTexture *texture)
+int GPU_texture_original_width(const gpu::Texture *texture)
 {
-  return unwrap(texture)->src_w;
+  return texture->src_w;
 }
 
-int GPU_texture_original_height(const GPUTexture *texture)
+int GPU_texture_original_height(const gpu::Texture *texture)
 {
-  return unwrap(texture)->src_h;
+  return texture->src_h;
 }
 
-void GPU_texture_original_size_set(GPUTexture *texture, int w, int h)
+void GPU_texture_original_size_set(gpu::Texture *texture, int w, int h)
 {
-  unwrap(texture)->src_w = w;
-  unwrap(texture)->src_h = h;
+  texture->src_w = w;
+  texture->src_h = h;
 }
 
-eGPUTextureFormat GPU_texture_format(const GPUTexture *texture)
+TextureFormat GPU_texture_format(const gpu::Texture *texture)
 {
-  return unwrap(texture)->format_get();
+  return texture->format_get();
 }
 
-const char *GPU_texture_format_name(eGPUTextureFormat texture_format)
+const char *GPU_texture_format_name(TextureFormat texture_format)
 {
   switch (texture_format) {
     /* Formats texture & render-buffer */
-    case GPU_RGBA8UI:
+    case TextureFormat::UINT_8_8_8_8:
       return "RGBA8UI";
-    case GPU_RGBA8I:
+    case TextureFormat::SINT_8_8_8_8:
       return "RGBA8I";
-    case GPU_RGBA8:
+    case TextureFormat::UNORM_8_8_8_8:
       return "RGBA8";
-    case GPU_RGBA32UI:
+    case TextureFormat::UINT_32_32_32_32:
       return "RGBA32UI";
-    case GPU_RGBA32I:
+    case TextureFormat::SINT_32_32_32_32:
       return "RGBA32I";
-    case GPU_RGBA32F:
+    case TextureFormat::SFLOAT_32_32_32_32:
       return "RGBA32F";
-    case GPU_RGBA16UI:
+    case TextureFormat::UINT_16_16_16_16:
       return "RGBA16UI";
-    case GPU_RGBA16I:
+    case TextureFormat::SINT_16_16_16_16:
       return "RGBA16I";
-    case GPU_RGBA16F:
+    case TextureFormat::SFLOAT_16_16_16_16:
       return "RGBA16F";
-    case GPU_RGBA16:
+    case TextureFormat::UNORM_16_16_16_16:
       return "RGBA16";
-    case GPU_RG8UI:
+    case TextureFormat::UINT_8_8:
       return "RG8UI";
-    case GPU_RG8I:
+    case TextureFormat::SINT_8_8:
       return "RG8I";
-    case GPU_RG8:
+    case TextureFormat::UNORM_8_8:
       return "RG8";
-    case GPU_RG32UI:
+    case TextureFormat::UINT_32_32:
       return "RG32UI";
-    case GPU_RG32I:
+    case TextureFormat::SINT_32_32:
       return "RG32I";
-    case GPU_RG32F:
+    case TextureFormat::SFLOAT_32_32:
       return "RG32F";
-    case GPU_RG16UI:
+    case TextureFormat::UINT_16_16:
       return "RG16UI";
-    case GPU_RG16I:
+    case TextureFormat::SINT_16_16:
       return "RG16I";
-    case GPU_RG16F:
+    case TextureFormat::SFLOAT_16_16:
       return "RG16F";
-    case GPU_RG16:
+    case TextureFormat::UNORM_16_16:
       return "RG16";
-    case GPU_R8UI:
+    case TextureFormat::UINT_8:
       return "R8UI";
-    case GPU_R8I:
+    case TextureFormat::SINT_8:
       return "R8I";
-    case GPU_R8:
+    case TextureFormat::UNORM_8:
       return "R8";
-    case GPU_R32UI:
+    case TextureFormat::UINT_32:
       return "R32UI";
-    case GPU_R32I:
+    case TextureFormat::SINT_32:
       return "R32I";
-    case GPU_R32F:
+    case TextureFormat::SFLOAT_32:
       return "R32F";
-    case GPU_R16UI:
+    case TextureFormat::UINT_16:
       return "R16UI";
-    case GPU_R16I:
+    case TextureFormat::SINT_16:
       return "R16I";
-    case GPU_R16F:
+    case TextureFormat::SFLOAT_16:
       return "R16F";
-    case GPU_R16:
+    case TextureFormat::UNORM_16:
       return "R16";
     /* Special formats texture & render-buffer */
-    case GPU_RGB10_A2:
+    case TextureFormat::UNORM_10_10_10_2:
       return "RGB10_A2";
-    case GPU_RGB10_A2UI:
+    case TextureFormat::UINT_10_10_10_2:
       return "RGB10_A2UI";
-    case GPU_R11F_G11F_B10F:
+    case TextureFormat::UFLOAT_11_11_10:
       return "R11F_G11F_B10F";
-    case GPU_DEPTH32F_STENCIL8:
+    case TextureFormat::SFLOAT_32_DEPTH_UINT_8:
       return "DEPTH32F_STENCIL8";
-    case GPU_DEPTH24_STENCIL8:
-      return "DEPTH24_STENCIL8";
-    case GPU_SRGB8_A8:
+    case TextureFormat::SRGBA_8_8_8_8:
       return "SRGB8_A8";
     /* Texture only formats. */
-    case GPU_RGB16F:
+    case TextureFormat::SFLOAT_16_16_16:
       return "RGB16F";
-    case GPU_RGB16_SNORM:
+    case TextureFormat::SNORM_16_16_16:
       return "RGB16_SNORM";
-    case GPU_RGB16I:
+    case TextureFormat::SINT_16_16_16:
       return "RGB16I";
-    case GPU_RGB16UI:
+    case TextureFormat::UINT_16_16_16:
       return "RGB16UI";
-    case GPU_RGB16:
+    case TextureFormat::UNORM_16_16_16:
       return "RGB16";
-    case GPU_RGBA16_SNORM:
+    case TextureFormat::SNORM_16_16_16_16:
       return "RGBA16_SNORM";
-    case GPU_RGBA8_SNORM:
+    case TextureFormat::SNORM_8_8_8_8:
       return "RGBA8_SNORM";
-    case GPU_RGB32F:
+    case TextureFormat::SFLOAT_32_32_32:
       return "RGB32F";
-    case GPU_RGB32I:
+    case TextureFormat::SINT_32_32_32:
       return "RGB32I";
-    case GPU_RGB32UI:
+    case TextureFormat::UINT_32_32_32:
       return "RGB32UI";
-    case GPU_RGB8_SNORM:
+    case TextureFormat::SNORM_8_8_8:
       return "RGB8_SNORM";
-    case GPU_RGB8:
+    case TextureFormat::UNORM_8_8_8:
       return "RGB8";
-    case GPU_RGB8I:
+    case TextureFormat::SINT_8_8_8:
       return "RGB8I";
-    case GPU_RGB8UI:
+    case TextureFormat::UINT_8_8_8:
       return "RGB8UI";
-    case GPU_RG16_SNORM:
+    case TextureFormat::SNORM_16_16:
       return "RG16_SNORM";
-    case GPU_RG8_SNORM:
+    case TextureFormat::SNORM_8_8:
       return "RG8_SNORM";
-    case GPU_R16_SNORM:
+    case TextureFormat::SNORM_16:
       return "R16_SNORM";
-    case GPU_R8_SNORM:
+    case TextureFormat::SNORM_8:
       return "R8_SNORM";
     /* Special formats, texture only. */
-    case GPU_SRGB8_A8_DXT1:
+    case TextureFormat::SRGB_DXT1:
       return "SRGB8_A8_DXT1";
-    case GPU_SRGB8_A8_DXT3:
+    case TextureFormat::SRGB_DXT3:
       return "SRGB8_A8_DXT3";
-    case GPU_SRGB8_A8_DXT5:
+    case TextureFormat::SRGB_DXT5:
       return "SRGB8_A8_DXT5";
-    case GPU_RGBA8_DXT1:
+    case TextureFormat::SNORM_DXT1:
       return "RGBA8_DXT1";
-    case GPU_RGBA8_DXT3:
+    case TextureFormat::SNORM_DXT3:
       return "RGBA8_DXT3";
-    case GPU_RGBA8_DXT5:
+    case TextureFormat::SNORM_DXT5:
       return "RGBA8_DXT5";
-    case GPU_SRGB8:
+    case TextureFormat::SRGBA_8_8_8:
       return "SRGB8";
-    case GPU_RGB9_E5:
+    case TextureFormat::UFLOAT_9_9_9_EXP_5:
       return "RGB9_E5";
     /* Depth Formats. */
-    case GPU_DEPTH_COMPONENT32F:
+    case TextureFormat::SFLOAT_32_DEPTH:
       return "DEPTH_COMPONENT32F";
-    case GPU_DEPTH_COMPONENT24:
-      return "DEPTH_COMPONENT24";
-    case GPU_DEPTH_COMPONENT16:
+    case TextureFormat::UNORM_16_DEPTH:
       return "DEPTH_COMPONENT16";
+
+    case TextureFormat::Invalid:
+      BLI_assert_unreachable();
+      return "Invalid";
   }
   BLI_assert_unreachable();
   return "";
 }
 
-bool GPU_texture_has_depth_format(const GPUTexture *texture)
+bool GPU_texture_has_depth_format(const gpu::Texture *texture)
 {
-  return (unwrap(texture)->format_flag_get() & GPU_FORMAT_DEPTH) != 0;
+  return (texture->format_flag_get() & GPU_FORMAT_DEPTH) != 0;
 }
 
-bool GPU_texture_has_stencil_format(const GPUTexture *texture)
+bool GPU_texture_has_stencil_format(const gpu::Texture *texture)
 {
-  return (unwrap(texture)->format_flag_get() & GPU_FORMAT_STENCIL) != 0;
+  return (texture->format_flag_get() & GPU_FORMAT_STENCIL) != 0;
 }
 
-bool GPU_texture_has_integer_format(const GPUTexture *texture)
+bool GPU_texture_has_integer_format(const gpu::Texture *texture)
 {
-  return (unwrap(texture)->format_flag_get() & GPU_FORMAT_INTEGER) != 0;
+  return (texture->format_flag_get() & GPU_FORMAT_INTEGER) != 0;
 }
 
-bool GPU_texture_has_float_format(const GPUTexture *texture)
+bool GPU_texture_has_float_format(const gpu::Texture *texture)
 {
-  return (unwrap(texture)->format_flag_get() & GPU_FORMAT_FLOAT) != 0;
+  return (texture->format_flag_get() & GPU_FORMAT_FLOAT) != 0;
 }
 
-bool GPU_texture_has_normalized_format(const GPUTexture *texture)
+bool GPU_texture_has_normalized_format(const gpu::Texture *texture)
 {
-  return (unwrap(texture)->format_flag_get() & GPU_FORMAT_NORMALIZED_INTEGER) != 0;
+  return (texture->format_flag_get() & GPU_FORMAT_NORMALIZED_INTEGER) != 0;
 }
 
-bool GPU_texture_has_signed_format(const GPUTexture *texture)
+bool GPU_texture_has_signed_format(const gpu::Texture *texture)
 {
-  return (unwrap(texture)->format_flag_get() & GPU_FORMAT_SIGNED) != 0;
+  return (texture->format_flag_get() & GPU_FORMAT_SIGNED) != 0;
 }
 
-bool GPU_texture_is_cube(const GPUTexture *texture)
+bool GPU_texture_is_cube(const gpu::Texture *texture)
 {
-  return (unwrap(texture)->type_get() & GPU_TEXTURE_CUBE) != 0;
+  return (texture->type_get() & GPU_TEXTURE_CUBE) != 0;
 }
 
-bool GPU_texture_is_array(const GPUTexture *texture)
+bool GPU_texture_is_array(const gpu::Texture *texture)
 {
-  return (unwrap(texture)->type_get() & GPU_TEXTURE_ARRAY) != 0;
+  return (texture->type_get() & GPU_TEXTURE_ARRAY) != 0;
 }
 
 #ifndef GPU_NO_USE_PY_REFERENCES
-void **GPU_texture_py_reference_get(GPUTexture *texture)
+void **GPU_texture_py_reference_get(gpu::Texture *texture)
 {
-  return unwrap(texture)->py_ref;
+  return texture->py_ref;
 }
 
-void GPU_texture_py_reference_set(GPUTexture *texture, void **py_ref)
+void GPU_texture_py_reference_set(gpu::Texture *texture, void **py_ref)
 {
-  BLI_assert(py_ref == nullptr || unwrap(texture)->py_ref == nullptr);
-  unwrap(texture)->py_ref = py_ref;
+  BLI_assert(py_ref == nullptr || texture->py_ref == nullptr);
+  texture->py_ref = py_ref;
 }
 #endif
 
-/* TODO: remove. */
-int GPU_texture_opengl_bindcode(const GPUTexture *texture)
+void GPU_texture_get_mipmap_size(gpu::Texture *texture, int mip_level, int *r_size)
 {
-  return unwrap(texture)->gl_bindcode_get();
-}
-
-void GPU_texture_get_mipmap_size(GPUTexture *texture, int mip_level, int *r_size)
-{
-  unwrap(texture)->mip_size_get(mip_level, r_size);
+  texture->mip_size_get(mip_level, r_size);
 }
 
 /** \} */
@@ -1028,7 +1026,7 @@ void GPU_samplers_update()
 /** \name GPU texture utilities
  * \{ */
 
-size_t GPU_texture_component_len(eGPUTextureFormat tex_format)
+size_t GPU_texture_component_len(TextureFormat tex_format)
 {
   return to_component_len(tex_format);
 }
