@@ -8,20 +8,18 @@
 
 #include "oiio/openimageio_support.hh"
 
-#include "IMB_filetype.h"
-#include "IMB_imbuf_types.h"
+#include "IMB_filetype.hh"
+#include "IMB_imbuf_types.hh"
 
 OIIO_NAMESPACE_USING
 using namespace blender::imbuf;
 
-extern "C" {
-
 bool imb_is_a_hdr(const uchar *mem, size_t size)
 {
   return imb_oiio_check(mem, size, "hdr");
 }
 
-ImBuf *imb_load_hdr(const uchar *mem, size_t size, int flags, char colorspace[IM_MAX_SPACE])
+ImBuf *imb_load_hdr(const uchar *mem, size_t size, int flags, ImFileColorSpace &r_colorspace)
 {
   ImageSpec config, spec;
 
@@ -30,13 +28,13 @@ ImBuf *imb_load_hdr(const uchar *mem, size_t size, int flags, char colorspace[IM
   /* Always create ImBufs with a 4th alpha channel despite the format only supporting 3. */
   ctx.use_all_planes = true;
 
-  ImBuf *ibuf = imb_oiio_read(ctx, config, colorspace, spec);
+  ImBuf *ibuf = imb_oiio_read(ctx, config, r_colorspace, spec);
   if (ibuf) {
     if (flags & IB_alphamode_detect) {
       ibuf->flags |= IB_alphamode_premul;
     }
-    if (flags & IB_rect) {
-      IMB_rect_from_float(ibuf);
+    if (flags & IB_byte_data) {
+      IMB_byte_from_float(ibuf);
     }
   }
 
@@ -53,4 +51,3 @@ bool imb_save_hdr(ImBuf *ibuf, const char *filepath, int flags)
 
   return imb_oiio_write(ctx, filepath, file_spec);
 }
-}
