@@ -8,42 +8,37 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "BLI_utildefines.h"
-
+#include "BLI_color.hh"
 #include "BLI_listbase.h"
 #include "BLI_math_matrix.h"
 #include "BLI_math_rotation.h"
 #include "BLI_math_vector.h"
 #include "BLI_rand.h"
-#include "BLI_string.h"
+#include "BLI_utildefines.h"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
 #include "DNA_defaults.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_screen_types.h"
 
-#include "BKE_context.h"
-#include "BKE_effect.h"
-#include "BKE_lattice.h"
-#include "BKE_lib_query.h"
+#include "BKE_customdata.hh"
+#include "BKE_lib_query.hh"
 #include "BKE_mesh.hh"
-#include "BKE_modifier.h"
+#include "BKE_modifier.hh"
 #include "BKE_particle.h"
 #include "BKE_pointcache.h"
-#include "BKE_screen.hh"
 
-#include "UI_interface.hh"
+#include "UI_interface_layout.hh"
 #include "UI_resources.hh"
 
 #include "RNA_access.hh"
-#include "RNA_prototypes.h"
+#include "RNA_prototypes.hh"
 
 #include "DEG_depsgraph_build.hh"
 #include "DEG_depsgraph_query.hh"
 
-#include "MOD_modifiertypes.hh"
 #include "MOD_ui_common.hh"
 
 static void init_data(ModifierData *md)
@@ -183,7 +178,7 @@ static bool particle_skip(ParticleInstanceModifierData *pimd, ParticleSystem *ps
   return true;
 }
 
-static void store_float_in_vcol(MLoopCol *vcol, float float_value)
+static void store_float_in_vcol(blender::ColorGeometry4b *vcol, float float_value)
 {
   const uchar value = unit_float_to_uchar_clamp(float_value);
   vcol->r = vcol->g = vcol->b = value;
@@ -192,6 +187,7 @@ static void store_float_in_vcol(MLoopCol *vcol, float float_value)
 
 static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh *mesh)
 {
+  using namespace blender;
   Mesh *result;
   ParticleInstanceModifierData *pimd = (ParticleInstanceModifierData *)md;
   Scene *scene = DEG_get_evaluated_scene(ctx->depsgraph);
@@ -201,7 +197,8 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
   int totvert, faces_num, totloop, totedge;
   int maxvert, maxpoly, maxloop, maxedge, part_end = 0, part_start;
   int k, p, p_skip;
-  short track = ctx->object->trackflag % 3, trackneg, axis = pimd->axis;
+  const uint track = uint(ctx->object->trackflag) % 3;
+  short trackneg, axis = pimd->axis;
   float max_co = 0.0, min_co = 0.0, temp_co[3];
   float *size = nullptr;
   float spacemat[4][4];
@@ -249,7 +246,7 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
 
   if (pimd->flag & eParticleInstanceFlag_UseSize) {
     float *si;
-    si = size = static_cast<float *>(MEM_calloc_arrayN(part_end, sizeof(float), __func__));
+    si = size = MEM_calloc_arrayN<float>(part_end, __func__);
 
     if (pimd->flag & eParticleInstanceFlag_Parents) {
       for (p = 0, pa = psys->particles; p < psys->totpart; p++, pa++, si++) {
@@ -273,7 +270,7 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
       break;
     case eParticleInstanceSpace_Local:
       /* get particle states in the particle object's local space */
-      invert_m4_m4(spacemat, pimd->ob->object_to_world);
+      invert_m4_m4(spacemat, pimd->ob->object_to_world().ptr());
       break;
     default:
       /* should not happen */
@@ -281,10 +278,10 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
       break;
   }
 
-  totvert = mesh->totvert;
+  totvert = mesh->verts_num;
   faces_num = mesh->faces_num;
-  totloop = mesh->totloop;
-  totedge = mesh->totedge;
+  totloop = mesh->corners_num;
+  totedge = mesh->edges_num;
 
   /* count particles */
   maxvert = 0;
@@ -322,18 +319,20 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
   blender::MutableSpan<int> face_offsets = result->face_offsets_for_write();
   blender::MutableSpan<int> corner_verts = result->corner_verts_for_write();
   blender::MutableSpan<int> corner_edges = result->corner_edges_for_write();
-
-  MLoopCol *mloopcols_index = static_cast<MLoopCol *>(CustomData_get_layer_named_for_write(
-      &result->loop_data, CD_PROP_BYTE_COLOR, pimd->index_layer_name, result->totloop));
-  MLoopCol *mloopcols_value = static_cast<MLoopCol *>(CustomData_get_layer_named_for_write(
-      &result->loop_data, CD_PROP_BYTE_COLOR, pimd->value_layer_name, result->totloop));
+  blender::bke::MutableAttributeAccessor attributes = result->attributes_for_write();
+  bke::SpanAttributeWriter mloopcols_index =
+      attributes.lookup_or_add_for_write_span<ColorGeometry4b>(pimd->index_layer_name,
+                                                               bke::AttrDomain::Corner);
+  bke::SpanAttributeWriter mloopcols_value =
+      attributes.lookup_or_add_for_write_span<ColorGeometry4b>(pimd->value_layer_name,
+                                                               bke::AttrDomain::Corner);
   int *vert_part_index = nullptr;
   float *vert_part_value = nullptr;
-  if (mloopcols_index != nullptr) {
-    vert_part_index = MEM_cnew_array<int>(maxvert, "vertex part index array");
+  if (mloopcols_index) {
+    vert_part_index = MEM_calloc_arrayN<int>(maxvert, "vertex part index array");
   }
   if (mloopcols_value) {
-    vert_part_value = MEM_cnew_array<float>(maxvert, "vertex part value array");
+    vert_part_value = MEM_calloc_arrayN<float>(maxvert, "vertex part value array");
   }
 
   for (p = part_start, p_skip = 0; p < part_end; p++) {
@@ -451,7 +450,7 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
           cross_v3_v3v3(cross, temp, state.vel);
 
           /* state.vel[axis] is the only component surviving from a dot product with the axis */
-          axis_angle_to_quat(state.rot, cross, saacos(state.vel[axis]));
+          axis_angle_to_quat(state.rot, cross, safe_acosf(state.vel[axis]));
         }
 #endif
       }
@@ -491,19 +490,19 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
         int j = in_face.size();
 
         CustomData_copy_data(
-            &mesh->loop_data, &result->loop_data, in_face.start(), dst_face_start, j);
+            &mesh->corner_data, &result->corner_data, in_face.start(), dst_face_start, j);
         for (; j; j--, orig_corner_i++, dst_corner_i++) {
           corner_verts[dst_corner_i] = orig_corner_verts[orig_corner_i] + (p_skip * totvert);
           corner_edges[dst_corner_i] = orig_corner_edges[orig_corner_i] + (p_skip * totedge);
           const int vert = corner_verts[dst_corner_i];
-          if (mloopcols_index != nullptr) {
+          if (mloopcols_index) {
             const int part_index = vert_part_index[vert];
-            store_float_in_vcol(&mloopcols_index[dst_corner_i],
+            store_float_in_vcol(&mloopcols_index.span[dst_corner_i],
                                 float(part_index) / float(psys->totpart - 1));
           }
-          if (mloopcols_value != nullptr) {
+          if (mloopcols_value) {
             const float part_value = vert_part_value[vert];
-            store_float_in_vcol(&mloopcols_value[dst_corner_i], part_value);
+            store_float_in_vcol(&mloopcols_value.span[dst_corner_i], part_value);
           }
         }
       }
@@ -520,6 +519,9 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
   MEM_SAFE_FREE(vert_part_index);
   MEM_SAFE_FREE(vert_part_value);
 
+  mloopcols_index.finish();
+  mloopcols_value.finish();
+
   return result;
 }
 
@@ -534,45 +536,47 @@ static void panel_draw(const bContext * /*C*/, Panel *panel)
 
   PointerRNA particle_obj_ptr = RNA_pointer_get(ptr, "object");
 
-  uiLayoutSetPropSep(layout, true);
+  layout->use_property_split_set(true);
 
-  uiItemR(layout, ptr, "object", UI_ITEM_NONE, nullptr, ICON_NONE);
+  layout->prop(ptr, "object", UI_ITEM_NONE, std::nullopt, ICON_NONE);
   if (!RNA_pointer_is_null(&particle_obj_ptr)) {
-    uiItemPointerR(layout,
-                   ptr,
-                   "particle_system",
-                   &particle_obj_ptr,
-                   "particle_systems",
-                   IFACE_("Particle System"),
-                   ICON_NONE);
+    layout->prop_search(ptr,
+                        "particle_system",
+                        &particle_obj_ptr,
+                        "particle_systems",
+                        IFACE_("Particle System"),
+                        ICON_NONE);
   }
   else {
-    uiItemR(
-        layout, ptr, "particle_system_index", UI_ITEM_NONE, IFACE_("Particle System"), ICON_NONE);
+    layout->prop(ptr, "particle_system_index", UI_ITEM_NONE, IFACE_("Particle System"), ICON_NONE);
   }
 
-  uiItemS(layout);
+  layout->separator();
 
-  row = uiLayoutRowWithHeading(layout, true, IFACE_("Create Instances"));
-  uiItemR(row, ptr, "use_normal", toggles_flag, nullptr, ICON_NONE);
-  uiItemR(row, ptr, "use_children", toggles_flag, nullptr, ICON_NONE);
-  uiItemR(row, ptr, "use_size", toggles_flag, nullptr, ICON_NONE);
+  row = &layout->row(true, IFACE_("Create Instances"));
+  row->prop(ptr, "use_normal", toggles_flag, std::nullopt, ICON_NONE);
+  row->prop(ptr, "use_children", toggles_flag, std::nullopt, ICON_NONE);
+  row->prop(ptr, "use_size", toggles_flag, std::nullopt, ICON_NONE);
 
-  row = uiLayoutRowWithHeading(layout, true, IFACE_("Show"));
-  uiItemR(row, ptr, "show_alive", toggles_flag, nullptr, ICON_NONE);
-  uiItemR(row, ptr, "show_dead", toggles_flag, nullptr, ICON_NONE);
-  uiItemR(row, ptr, "show_unborn", toggles_flag, nullptr, ICON_NONE);
+  row = &layout->row(true, IFACE_("Show"));
+  row->prop(ptr, "show_alive", toggles_flag, std::nullopt, ICON_NONE);
+  row->prop(ptr, "show_dead", toggles_flag, std::nullopt, ICON_NONE);
+  row->prop(ptr, "show_unborn", toggles_flag, std::nullopt, ICON_NONE);
 
-  uiItemR(layout, ptr, "particle_amount", UI_ITEM_NONE, IFACE_("Amount"), ICON_NONE);
-  uiItemR(layout, ptr, "particle_offset", UI_ITEM_NONE, IFACE_("Offset"), ICON_NONE);
+  layout->prop(ptr,
+               "particle_amount",
+               UI_ITEM_NONE,
+               CTX_IFACE_(BLT_I18NCONTEXT_COUNTABLE, "Amount"),
+               ICON_NONE);
+  layout->prop(ptr, "particle_offset", UI_ITEM_NONE, IFACE_("Offset"), ICON_NONE);
 
-  uiItemS(layout);
+  layout->separator();
 
-  uiItemR(layout, ptr, "space", UI_ITEM_NONE, IFACE_("Coordinate Space"), ICON_NONE);
-  row = uiLayoutRow(layout, true);
-  uiItemR(row, ptr, "axis", UI_ITEM_R_EXPAND, nullptr, ICON_NONE);
+  layout->prop(ptr, "space", UI_ITEM_NONE, IFACE_("Coordinate Space"), ICON_NONE);
+  row = &layout->row(true);
+  row->prop(ptr, "axis", UI_ITEM_R_EXPAND, std::nullopt, ICON_NONE);
 
-  modifier_panel_end(layout, ptr);
+  modifier_error_message_draw(layout, ptr);
 }
 
 static void path_panel_draw_header(const bContext * /*C*/, Panel *panel)
@@ -581,7 +585,7 @@ static void path_panel_draw_header(const bContext * /*C*/, Panel *panel)
 
   PointerRNA *ptr = modifier_panel_get_property_pointers(panel, nullptr);
 
-  uiItemR(layout, ptr, "use_path", UI_ITEM_NONE, IFACE_("Create Along Paths"), ICON_NONE);
+  layout->prop(ptr, "use_path", UI_ITEM_NONE, IFACE_("Create Along Paths"), ICON_NONE);
 }
 
 static void path_panel_draw(const bContext * /*C*/, Panel *panel)
@@ -592,18 +596,18 @@ static void path_panel_draw(const bContext * /*C*/, Panel *panel)
   PointerRNA ob_ptr;
   PointerRNA *ptr = modifier_panel_get_property_pointers(panel, &ob_ptr);
 
-  uiLayoutSetPropSep(layout, true);
+  layout->use_property_split_set(true);
 
-  uiLayoutSetActive(layout, RNA_boolean_get(ptr, "use_path"));
+  layout->active_set(RNA_boolean_get(ptr, "use_path"));
 
-  col = uiLayoutColumn(layout, true);
-  uiItemR(col, ptr, "position", UI_ITEM_R_SLIDER, nullptr, ICON_NONE);
-  uiItemR(col, ptr, "random_position", UI_ITEM_R_SLIDER, IFACE_("Random"), ICON_NONE);
-  col = uiLayoutColumn(layout, true);
-  uiItemR(col, ptr, "rotation", UI_ITEM_R_SLIDER, nullptr, ICON_NONE);
-  uiItemR(col, ptr, "random_rotation", UI_ITEM_R_SLIDER, IFACE_("Random"), ICON_NONE);
+  col = &layout->column(true);
+  col->prop(ptr, "position", UI_ITEM_R_SLIDER, std::nullopt, ICON_NONE);
+  col->prop(ptr, "random_position", UI_ITEM_R_SLIDER, IFACE_("Random"), ICON_NONE);
+  col = &layout->column(true);
+  col->prop(ptr, "rotation", UI_ITEM_R_SLIDER, std::nullopt, ICON_NONE);
+  col->prop(ptr, "random_rotation", UI_ITEM_R_SLIDER, IFACE_("Random"), ICON_NONE);
 
-  uiItemR(layout, ptr, "use_preserve_shape", UI_ITEM_NONE, nullptr, ICON_NONE);
+  layout->prop(ptr, "use_preserve_shape", UI_ITEM_NONE, std::nullopt, ICON_NONE);
 }
 
 static void layers_panel_draw(const bContext * /*C*/, Panel *panel)
@@ -616,13 +620,13 @@ static void layers_panel_draw(const bContext * /*C*/, Panel *panel)
 
   PointerRNA obj_data_ptr = RNA_pointer_get(&ob_ptr, "data");
 
-  uiLayoutSetPropSep(layout, true);
+  layout->use_property_split_set(true);
 
-  col = uiLayoutColumn(layout, false);
-  uiItemPointerR(
-      col, ptr, "index_layer_name", &obj_data_ptr, "vertex_colors", IFACE_("Index"), ICON_NONE);
-  uiItemPointerR(
-      col, ptr, "value_layer_name", &obj_data_ptr, "vertex_colors", IFACE_("Value"), ICON_NONE);
+  col = &layout->column(false);
+  col->prop_search(
+      ptr, "index_layer_name", &obj_data_ptr, "vertex_colors", IFACE_("Index"), ICON_NONE);
+  col->prop_search(
+      ptr, "value_layer_name", &obj_data_ptr, "vertex_colors", IFACE_("Value"), ICON_NONE);
 }
 
 static void panel_register(ARegionType *region_type)
@@ -641,7 +645,7 @@ ModifierTypeInfo modifierType_ParticleInstance = {
     /*struct_name*/ "ParticleInstanceModifierData",
     /*struct_size*/ sizeof(ParticleInstanceModifierData),
     /*srna*/ &RNA_ParticleInstanceModifier,
-    /*type*/ eModifierTypeType_Constructive,
+    /*type*/ ModifierTypeType::Constructive,
     /*flags*/ eModifierTypeFlag_AcceptsMesh | eModifierTypeFlag_SupportsMapping |
         eModifierTypeFlag_SupportsEditmode | eModifierTypeFlag_EnableInEditmode,
     /*icon*/ ICON_MOD_PARTICLE_INSTANCE,
@@ -668,4 +672,6 @@ ModifierTypeInfo modifierType_ParticleInstance = {
     /*panel_register*/ panel_register,
     /*blend_write*/ nullptr,
     /*blend_read*/ nullptr,
+    /*foreach_cache*/ nullptr,
+    /*foreach_working_space_color*/ nullptr,
 };
