@@ -10,35 +10,84 @@
 #include "BLI_math_vector.hh"
 #include "BLI_math_vector_types.hh"
 
-#include "UI_interface.hh"
-#include "UI_resources.hh"
+#include "RNA_types.hh"
 
 #include "COM_node_operation.hh"
 #include "COM_utilities.hh"
 
 #include "node_composite_util.hh"
 
-/* **************** FILTER  ******************** */
-
 namespace blender::nodes::node_composite_filter_cc {
 
+static const EnumPropertyItem type_items[] = {
+    {CMP_NODE_FILTER_SOFT, "SOFTEN", 0, N_("Soften"), ""},
+    {CMP_NODE_FILTER_SHARP_BOX,
+     "SHARPEN",
+     0,
+     N_("Box Sharpen"),
+     N_("An aggressive sharpening filter")},
+    {CMP_NODE_FILTER_SHARP_DIAMOND,
+     "SHARPEN_DIAMOND",
+     0,
+     N_("Diamond Sharpen"),
+     N_("A moderate sharpening filter")},
+    {CMP_NODE_FILTER_LAPLACE, "LAPLACE", 0, N_("Laplace"), ""},
+    {CMP_NODE_FILTER_SOBEL, "SOBEL", 0, N_("Sobel"), ""},
+    {CMP_NODE_FILTER_PREWITT, "PREWITT", 0, N_("Prewitt"), ""},
+    {CMP_NODE_FILTER_KIRSCH, "KIRSCH", 0, N_("Kirsch"), ""},
+    {CMP_NODE_FILTER_SHADOW, "SHADOW", 0, N_("Shadow"), ""},
+    {0, nullptr, 0, nullptr, nullptr},
+};
+
 static void cmp_node_filter_declare(NodeDeclarationBuilder &b)
 {
-  b.add_input<decl::Float>("Fac")
+  b.use_custom_socket_order();
+  b.allow_any_socket_order();
+  b.add_input<decl::Color>("Image")
+      .default_value({1.0f, 1.0f, 1.0f, 1.0f})
+      .hide_value()
+      .structure_type(StructureType::Dynamic);
+  b.add_output<decl::Color>("Image").structure_type(StructureType::Dynamic).align_with_previous();
+
+  b.add_input<decl::Float>("Factor", "Fac")
       .default_value(1.0f)
       .min(0.0f)
       .max(1.0f)
       .subtype(PROP_FACTOR)
-      .compositor_domain_priority(1);
-  b.add_input<decl::Color>("Image")
-      .default_value({1.0f, 1.0f, 1.0f, 1.0f})
-      .compositor_domain_priority(0);
-  b.add_output<decl::Color>("Image");
+      .structure_type(StructureType::Dynamic);
+  b.add_input<decl::Menu>("Type")
+      .default_value(CMP_NODE_FILTER_SOFT)
+      .static_items(type_items)
+      .optional_label();
 }
 
-static void node_composit_buts_filter(uiLayout *layout, bContext * /*C*/, PointerRNA *ptr)
+class SocketSearchOp {
+ public:
+  CMPNodeFilterMethod filter_type = CMP_NODE_FILTER_SOFT;
+  void operator()(LinkSearchOpParams &params)
+  {
+    bNode &node = params.add_node("CompositorNodeFilter");
+    bNodeSocket &type_socket = *blender::bke::node_find_socket(node, SOCK_IN, "Type");
+    type_socket.default_value_typed<bNodeSocketValueMenu>()->value = this->filter_type;
+    params.update_and_connect_available_socket(node, "Image");
+  }
+};
+
+static void gather_link_searches(GatherLinkSearchOpParams &params)
 {
-  uiItemR(layout, ptr, "filter_type", UI_ITEM_R_SPLIT_EMPTY_NAME, "", ICON_NONE);
+  const eNodeSocketDatatype from_socket_type = eNodeSocketDatatype(params.other_socket().type);
+  if (!params.node_tree().typeinfo->validate_link(from_socket_type, SOCK_RGBA)) {
+    return;
+  }
+
+  params.add_item(IFACE_("Soften"), SocketSearchOp{CMP_NODE_FILTER_SOFT});
+  params.add_item(IFACE_("Box Sharpen"), SocketSearchOp{CMP_NODE_FILTER_SHARP_BOX});
+  params.add_item(IFACE_("Laplace"), SocketSearchOp{CMP_NODE_FILTER_LAPLACE});
+  params.add_item(IFACE_("Sobel"), SocketSearchOp{CMP_NODE_FILTER_SOBEL});
+  params.add_item(IFACE_("Prewitt"), SocketSearchOp{CMP_NODE_FILTER_PREWITT});
+  params.add_item(IFACE_("Kirsch"), SocketSearchOp{CMP_NODE_FILTER_KIRSCH});
+  params.add_item(IFACE_("Shadow"), SocketSearchOp{CMP_NODE_FILTER_SHADOW});
+  params.add_item(IFACE_("Diamond Sharpen"), SocketSearchOp{CMP_NODE_FILTER_SHARP_DIAMOND});
 }
 
 using namespace blender::compositor;
@@ -49,10 +98,10 @@ class FilterOperation : public NodeOperation {
 
   void execute() override
   {
-    Result &input_image = get_input("Image");
+    const Result &input_image = this->get_input("Image");
     if (input_image.is_single_value()) {
-      Result &output_image = get_result("Image");
-      input_image.pass_through(output_image);
+      Result &output_image = this->get_result("Image");
+      output_image.share_data(input_image);
       return;
     }
 
@@ -66,7 +115,7 @@ class FilterOperation : public NodeOperation {
 
   void execute_gpu()
   {
-    GPUShader *shader = context().get_shader(get_shader_name());
+    gpu::Shader *shader = context().get_shader(get_shader_name());
     GPU_shader_bind(shader);
 
     GPU_shader_uniform_mat3_as_mat4(shader, "ukernel", get_filter_kernel().ptr());
@@ -161,7 +210,7 @@ class FilterOperation : public NodeOperation {
 
   bool is_edge_filter()
   {
-    switch (this->get_filter_method()) {
+    switch (this->get_type()) {
       case CMP_NODE_FILTER_LAPLACE:
       case CMP_NODE_FILTER_SOBEL:
       case CMP_NODE_FILTER_PREWITT:
@@ -181,7 +230,7 @@ class FilterOperation : public NodeOperation {
     /* Initialize the kernels as arrays of rows with the top row first. Edge detection kernels
      * return the kernel in the X direction, while the kernel in the Y direction will be computed
      * inside the shader by transposing the kernel in the X direction. */
-    switch (get_filter_method()) {
+    switch (this->get_type()) {
       case CMP_NODE_FILTER_SOFT: {
         const float kernel[3][3] = {{1.0f / 16.0f, 2.0f / 16.0f, 1.0f / 16.0f},
                                     {2.0f / 16.0f, 4.0f / 16.0f, 2.0f / 16.0f},
@@ -221,16 +270,18 @@ class FilterOperation : public NodeOperation {
             {0.0f, -1.0f, 0.0f}, {-1.0f, 5.0f, -1.0f}, {0.0f, -1.0f, 0.0f}};
         return float3x3(kernel);
       }
-      default: {
-        const float kernel[3][3] = {{0.0f, 0.0f, 0.0f}, {0.0f, 1.0f, 0.0f}, {0.0f, 0.0f, 0.0f}};
-        return float3x3(kernel);
-      }
     }
+
+    const float kernel[3][3] = {{0.0f, 0.0f, 0.0f}, {0.0f, 1.0f, 0.0f}, {0.0f, 0.0f, 0.0f}};
+    return float3x3(kernel);
   }
 
-  CMPNodeFilterMethod get_filter_method()
+  CMPNodeFilterMethod get_type()
   {
-    return static_cast<CMPNodeFilterMethod>(bnode().custom1);
+    const Result &input = this->get_input("Type");
+    const MenuValue default_menu_value = MenuValue(CMP_NODE_FILTER_SOFT);
+    const MenuValue menu_value = input.get_single_value_default(default_menu_value);
+    return static_cast<CMPNodeFilterMethod>(menu_value.value);
   }
 };
 
@@ -241,7 +292,7 @@ static NodeOperation *get_compositor_operation(Context &context, DNode node)
 
 }  // namespace blender::nodes::node_composite_filter_cc
 
-void register_node_type_cmp_filter()
+static void register_node_type_cmp_filter()
 {
   namespace file_ns = blender::nodes::node_composite_filter_cc;
 
@@ -253,10 +304,10 @@ void register_node_type_cmp_filter()
   ntype.enum_name_legacy = "FILTER";
   ntype.nclass = NODE_CLASS_OP_FILTER;
   ntype.declare = file_ns::cmp_node_filter_declare;
-  ntype.draw_buttons = file_ns::node_composit_buts_filter;
-  ntype.labelfunc = node_filter_label;
   ntype.flag |= NODE_PREVIEW;
   ntype.get_compositor_operation = file_ns::get_compositor_operation;
+  ntype.gather_link_search_ops = file_ns::gather_link_searches;
 
-  blender::bke::node_register_type(&ntype);
+  blender::bke::node_register_type(ntype);
 }
+NOD_REGISTER_NODE(register_node_type_cmp_filter)
