@@ -26,9 +26,9 @@ class InstanceRotationFieldInput final : public bke::InstancesFieldInput {
                                  const IndexMask & /*mask*/) const final
   {
     const Span<float4x4> transforms = instances.transforms();
-    return VArray<math::Quaternion>::ForFunc(instances.instances_num(), [transforms](const int i) {
-      return math::to_quaternion(math::normalize(transforms[i]));
-    });
+    return VArray<math::Quaternion>::from_func(
+        instances.instances_num(),
+        [transforms](const int i) { return math::to_quaternion(math::normalize(transforms[i])); });
   }
 
   uint64_t hash() const override
