@@ -10,7 +10,7 @@
 #include "NOD_rna_define.hh"
 #include "NOD_socket_search_link.hh"
 
-#include "UI_interface.hh"
+#include "UI_interface_layout.hh"
 #include "UI_resources.hh"
 
 #include "RNA_enum_types.hh"
@@ -25,26 +25,51 @@ using namespace blender::bke::mesh_surface_sample;
 
 NODE_STORAGE_FUNCS(NodeGeometryRaycast)
 
+static EnumPropertyItem interpolation_items[] = {
+    {GEO_NODE_RAYCAST_INTERPOLATED,
+     "INTERPOLATED",
+     0,
+     N_("Interpolated"),
+     N_("Interpolate the attribute from the corners of the hit face")},
+    {GEO_NODE_RAYCAST_NEAREST,
+     "NEAREST",
+     0,
+     N_("Nearest"),
+     N_("Use the attribute value of the closest mesh element")},
+    {0, nullptr, 0, nullptr, nullptr},
+};
+
 static void node_declare(NodeDeclarationBuilder &b)
 {
   const bNode *node = b.node_or_null();
 
   b.add_input<decl::Geometry>("Target Geometry")
       .only_realized_data()
-      .supported_type(GeometryComponent::Type::Mesh);
+      .supported_type(GeometryComponent::Type::Mesh)
+      .description("Geometry to cast rays onto");
   if (node != nullptr) {
     const eCustomDataType data_type = eCustomDataType(node_storage(*node).data_type);
     /* TODO: Field interfacing depends on the offset of the next declarations! */
     b.add_input(data_type, "Attribute").hide_value().field_on_all();
   }
+  b.add_input<decl::Menu>("Interpolation")
+      .static_items(interpolation_items)
+      .optional_label()
+      .description("Mapping from the target geometry to hit points");
 
-  b.add_input<decl::Vector>("Source Position").implicit_field(implicit_field_inputs::position);
-  b.add_input<decl::Vector>("Ray Direction").default_value({0.0f, 0.0f, -1.0f}).supports_field();
+  b.add_input<decl::Vector>("Source Position")
+      .implicit_field(NODE_DEFAULT_INPUT_POSITION_FIELD)
+      .structure_type(StructureType::Dynamic);
+  b.add_input<decl::Vector>("Ray Direction")
+      .default_value({0.0f, 0.0f, -1.0f})
+      .supports_field()
+      .structure_type(StructureType::Dynamic);
   b.add_input<decl::Float>("Ray Length")
       .default_value(100.0f)
       .min(0.0f)
       .subtype(PROP_DISTANCE)
-      .supports_field();
+      .supports_field()
+      .structure_type(StructureType::Dynamic);
 
   b.add_output<decl::Bool>("Is Hit").dependent_field({2, 3, 4});
   b.add_output<decl::Vector>("Hit Position").dependent_field({2, 3, 4});
@@ -59,14 +84,12 @@ static void node_declare(NodeDeclarationBuilder &b)
 
 static void node_layout(uiLayout *layout, bContext * /*C*/, PointerRNA *ptr)
 {
-  uiItemR(layout, ptr, "data_type", UI_ITEM_NONE, "", ICON_NONE);
-  uiItemR(layout, ptr, "mapping", UI_ITEM_NONE, "", ICON_NONE);
+  layout->prop(ptr, "data_type", UI_ITEM_NONE, "", ICON_NONE);
 }
 
 static void node_init(bNodeTree * /*tree*/, bNode *node)
 {
-  NodeGeometryRaycast *data = MEM_cnew<NodeGeometryRaycast>(__func__);
-  data->mapping = GEO_NODE_RAYCAST_INTERPOLATED;
+  NodeGeometryRaycast *data = MEM_callocN<NodeGeometryRaycast>(__func__);
   data->data_type = CD_PROP_FLOAT;
   node->storage = data;
 }
@@ -203,8 +226,7 @@ class RaycastFunction : public mf::MultiFunction {
 static void node_geo_exec(GeoNodeExecParams params)
 {
   GeometrySet target = params.extract_input<GeometrySet>("Target Geometry");
-  const NodeGeometryRaycast &storage = node_storage(params.node());
-  const GeometryNodeRaycastMapMode mapping = GeometryNodeRaycastMapMode(storage.mapping);
+  const auto mapping = params.get_input<GeometryNodeRaycastMapMode>("Interpolation");
 
   if (target.is_empty()) {
     params.set_default_remaining_outputs();
@@ -222,73 +244,109 @@ static void node_geo_exec(GeoNodeExecParams params)
     return;
   }
 
-  static auto normalize_fn = mf::build::SI1_SO<float3, float3>(
-      "Normalize",
-      [](const float3 &v) { return math::normalize(v); },
-      mf::build::exec_presets::AllSpanOrSingle());
-  auto direction_op = FieldOperation::Create(
-      normalize_fn, {params.extract_input<Field<float3>>("Ray Direction")});
+  std::string error_message;
 
-  auto op = FieldOperation::Create(std::make_unique<RaycastFunction>(target),
-                                   {params.extract_input<Field<float3>>("Source Position"),
-                                    Field<float3>(direction_op),
-                                    params.extract_input<Field<float>>("Ray Length")});
+  bke::SocketValueVariant normalized_direction;
+  {
+    auto ray_direction = params.extract_input<bke::SocketValueVariant>("Ray Direction");
 
-  Field<float3> hit_position(op, 1);
-  params.set_output("Is Hit", Field<bool>(op, 0));
+    static auto normalize_fn = mf::build::SI1_SO<float3, float3>(
+        "Normalize",
+        [](const float3 &v) { return math::normalize(v); },
+        mf::build::exec_presets::AllSpanOrSingle());
+
+    if (!execute_multi_function_on_value_variant(normalize_fn,
+                                                 {&ray_direction},
+                                                 {&normalized_direction},
+                                                 params.user_data(),
+                                                 error_message))
+    {
+      params.set_default_remaining_outputs();
+      params.error_message_add(NodeWarningType::Error, std::move(error_message));
+      return;
+    }
+  }
+
+  auto position = params.extract_input<bke::SocketValueVariant>("Source Position");
+  auto ray_length = params.extract_input<bke::SocketValueVariant>("Ray Length");
+
+  bke::SocketValueVariant is_hit;
+  bke::SocketValueVariant hit_position;
+  bke::SocketValueVariant hit_normal;
+  bke::SocketValueVariant hit_distance;
+  bke::SocketValueVariant triangle_index;
+  if (!execute_multi_function_on_value_variant(
+          std::make_unique<RaycastFunction>(target),
+          {&position, &normalized_direction, &ray_length},
+          {&is_hit, &hit_position, &hit_normal, &hit_distance, &triangle_index},
+          params.user_data(),
+          error_message))
+  {
+    params.set_default_remaining_outputs();
+    params.error_message_add(NodeWarningType::Error, std::move(error_message));
+    return;
+  }
+
+  params.set_output("Is Hit", std::move(is_hit));
   params.set_output("Hit Position", hit_position);
-  params.set_output("Hit Normal", Field<float3>(op, 2));
-  params.set_output("Hit Distance", Field<float>(op, 3));
+  params.set_output("Hit Normal", std::move(hit_normal));
+  params.set_output("Hit Distance", std::move(hit_distance));
 
   if (!params.output_is_required("Attribute")) {
     return;
   }
 
   GField field = params.extract_input<GField>("Attribute");
-  Field<int> triangle_index(op, 4);
-  Field<float3> bary_weights;
+  bke::SocketValueVariant bary_weights;
+  bke::SocketValueVariant triangle_index_copy = triangle_index;
   switch (mapping) {
     case GEO_NODE_RAYCAST_INTERPOLATED:
-      bary_weights = Field<float3>(FieldOperation::Create(
-          std::make_shared<bke::mesh_surface_sample::BaryWeightFromPositionFn>(target),
-          {hit_position, triangle_index}));
+      if (!execute_multi_function_on_value_variant(
+              std::make_shared<bke::mesh_surface_sample::BaryWeightFromPositionFn>(target),
+              {&hit_position, &triangle_index_copy},
+              {&bary_weights},
+              params.user_data(),
+              error_message))
+      {
+        params.set_default_remaining_outputs();
+        params.error_message_add(NodeWarningType::Error, std::move(error_message));
+        return;
+      }
       break;
     case GEO_NODE_RAYCAST_NEAREST:
-      bary_weights = Field<float3>(FieldOperation::Create(
-          std::make_shared<bke::mesh_surface_sample::CornerBaryWeightFromPositionFn>(target),
-          {hit_position, triangle_index}));
+      if (!execute_multi_function_on_value_variant(
+              std::make_shared<bke::mesh_surface_sample::CornerBaryWeightFromPositionFn>(target),
+              {&hit_position, &triangle_index_copy},
+              {&bary_weights},
+              params.user_data(),
+              error_message))
+      {
+        params.set_default_remaining_outputs();
+        params.error_message_add(NodeWarningType::Error, std::move(error_message));
+        return;
+      }
       break;
   }
-  auto sample_op = FieldOperation::Create(
-      std::make_shared<bke::mesh_surface_sample::BaryWeightSampleFn>(std::move(target),
-                                                                     std::move(field)),
-      {triangle_index, bary_weights});
-  params.set_output("Attribute", GField(sample_op));
+
+  bke::SocketValueVariant sampled_atribute;
+  if (!execute_multi_function_on_value_variant(
+          std::make_shared<bke::mesh_surface_sample::BaryWeightSampleFn>(std::move(target),
+                                                                         std::move(field)),
+          {&triangle_index, &bary_weights},
+          {&sampled_atribute},
+          params.user_data(),
+          error_message))
+  {
+    params.set_default_remaining_outputs();
+    params.error_message_add(NodeWarningType::Error, std::move(error_message));
+    return;
+  }
+
+  params.set_output("Attribute", std::move(sampled_atribute));
 }
 
 static void node_rna(StructRNA *srna)
 {
-  static EnumPropertyItem mapping_items[] = {
-      {GEO_NODE_RAYCAST_INTERPOLATED,
-       "INTERPOLATED",
-       0,
-       "Interpolated",
-       "Interpolate the attribute from the corners of the hit face"},
-      {GEO_NODE_RAYCAST_NEAREST,
-       "NEAREST",
-       0,
-       "Nearest",
-       "Use the attribute value of the closest mesh element"},
-      {0, nullptr, 0, nullptr, nullptr},
-  };
-
-  RNA_def_node_enum(srna,
-                    "mapping",
-                    "Mapping",
-                    "Mapping from the target geometry to hit points",
-                    mapping_items,
-                    NOD_storage_enum_accessors(mapping),
-                    GEO_NODE_RAYCAST_INTERPOLATED);
   RNA_def_node_enum(srna,
                     "data_type",
                     "Data Type",
@@ -310,15 +368,15 @@ static void node_register()
       "each hit point";
   ntype.enum_name_legacy = "RAYCAST";
   ntype.nclass = NODE_CLASS_GEOMETRY;
-  bke::node_type_size_preset(&ntype, bke::eNodeSizePreset::Middle);
+  bke::node_type_size_preset(ntype, bke::eNodeSizePreset::Middle);
   ntype.initfunc = node_init;
   blender::bke::node_type_storage(
-      &ntype, "NodeGeometryRaycast", node_free_standard_storage, node_copy_standard_storage);
+      ntype, "NodeGeometryRaycast", node_free_standard_storage, node_copy_standard_storage);
   ntype.declare = node_declare;
   ntype.geometry_node_execute = node_geo_exec;
   ntype.draw_buttons = node_layout;
   ntype.gather_link_search_ops = node_gather_link_searches;
-  blender::bke::node_register_type(&ntype);
+  blender::bke::node_register_type(ntype);
 
   node_rna(ntype.rna_ext.srna);
 }
