@@ -5,7 +5,10 @@
 #include "NOD_node_declaration.hh"
 #include "NOD_socket_declarations.hh"
 #include "NOD_socket_declarations_geometry.hh"
+#include "NOD_socket_usage_inference.hh"
 
+#include "BLI_assert.h"
+#include "BLI_listbase.h"
 #include "BLI_utildefines.h"
 
 #include "BKE_geometry_fields.hh"
@@ -37,19 +40,19 @@ void build_node_declaration(const bke::bNodeType &typeinfo,
 void NodeDeclarationBuilder::build_remaining_anonymous_attribute_relations()
 {
   auto is_data_socket_decl = [](const SocketDeclaration *socket_decl) {
-    return dynamic_cast<const decl::Geometry *>(socket_decl);
+    return ELEM(socket_decl->socket_type, SOCK_GEOMETRY, SOCK_BUNDLE, SOCK_CLOSURE);
   };
 
-  Vector<int> geometry_inputs;
+  Vector<int> data_inputs;
   for (const int i : declaration_.inputs.index_range()) {
     if (is_data_socket_decl(declaration_.inputs[i])) {
-      geometry_inputs.append(i);
+      data_inputs.append(i);
     }
   }
-  Vector<int> geometry_outputs;
+  Vector<int> data_outputs;
   for (const int i : declaration_.outputs.index_range()) {
     if (is_data_socket_decl(declaration_.outputs[i])) {
-      geometry_outputs.append(i);
+      data_outputs.append(i);
     }
   }
 
@@ -57,8 +60,8 @@ void NodeDeclarationBuilder::build_remaining_anonymous_attribute_relations()
     if (socket_builder->field_on_all_) {
       aal::RelationsInNode &relations = this->get_anonymous_attribute_relations();
       const int field_input = socket_builder->decl_base_->index;
-      for (const int geometry_input : geometry_inputs) {
-        relations.eval_relations.append({field_input, geometry_input});
+      for (const int data_input : data_inputs) {
+        relations.eval_relations.append({field_input, data_input});
       }
     }
   }
@@ -66,8 +69,8 @@ void NodeDeclarationBuilder::build_remaining_anonymous_attribute_relations()
     if (socket_builder->field_on_all_) {
       aal::RelationsInNode &relations = this->get_anonymous_attribute_relations();
       const int field_output = socket_builder->decl_base_->index;
-      for (const int geometry_output : geometry_outputs) {
-        relations.available_relations.append({field_output, geometry_output});
+      for (const int data_output : data_outputs) {
+        relations.available_relations.append({field_output, data_output});
       }
     }
     if (socket_builder->reference_pass_all_) {
@@ -75,16 +78,18 @@ void NodeDeclarationBuilder::build_remaining_anonymous_attribute_relations()
       const int field_output = socket_builder->decl_base_->index;
       for (const int input_i : declaration_.inputs.index_range()) {
         SocketDeclaration &input_socket_decl = *declaration_.inputs[input_i];
-        if (input_socket_decl.input_field_type != InputSocketFieldType::None) {
+        if (input_socket_decl.input_field_type != InputSocketFieldType::None ||
+            ELEM(input_socket_decl.socket_type, SOCK_BUNDLE, SOCK_CLOSURE))
+        {
           relations.reference_relations.append({input_i, field_output});
         }
       }
     }
     if (socket_builder->propagate_from_all_) {
       aal::RelationsInNode &relations = this->get_anonymous_attribute_relations();
-      const int geometry_output = socket_builder->decl_base_->index;
-      for (const int geometry_input : geometry_inputs) {
-        relations.propagate_relations.append({geometry_input, geometry_output});
+      const int data_output = socket_builder->decl_base_->index;
+      for (const int data_input : data_inputs) {
+        relations.propagate_relations.append({data_input, data_output});
       }
     }
   }
@@ -93,6 +98,14 @@ void NodeDeclarationBuilder::build_remaining_anonymous_attribute_relations()
 void NodeDeclarationBuilder::finalize()
 {
   this->build_remaining_anonymous_attribute_relations();
+  if (is_function_node_) {
+    for (SocketDeclaration *socket_decl : declaration_.inputs) {
+      socket_decl->structure_type = StructureType::Dynamic;
+    }
+    for (SocketDeclaration *socket_decl : declaration_.outputs) {
+      socket_decl->structure_type = StructureType::Dynamic;
+    }
+  }
 #ifndef NDEBUG
   declaration_.assert_valid();
 #endif
@@ -270,11 +283,8 @@ bNodeSocket &SocketDeclaration::update_or_build(bNodeTree &ntree,
 
 void SocketDeclaration::set_common_flags(bNodeSocket &socket) const
 {
-  SET_FLAG_FROM_TEST(socket.flag, compact, SOCK_COMPACT);
   SET_FLAG_FROM_TEST(socket.flag, hide_value, SOCK_HIDE_VALUE);
-  SET_FLAG_FROM_TEST(socket.flag, hide_label, SOCK_HIDE_LABEL);
   SET_FLAG_FROM_TEST(socket.flag, is_multi_input, SOCK_MULTI_INPUT);
-  SET_FLAG_FROM_TEST(socket.flag, no_mute_links, SOCK_NO_INTERNAL_LINK);
   SET_FLAG_FROM_TEST(socket.flag, !is_available, SOCK_UNAVAIL);
 }
 
@@ -286,21 +296,12 @@ bool SocketDeclaration::matches_common_data(const bNodeSocket &socket) const
   if (socket.identifier != this->identifier) {
     return false;
   }
-  if (((socket.flag & SOCK_COMPACT) != 0) != this->compact) {
-    return false;
-  }
   if (((socket.flag & SOCK_HIDE_VALUE) != 0) != this->hide_value) {
     return false;
   }
-  if (((socket.flag & SOCK_HIDE_LABEL) != 0) != this->hide_label) {
-    return false;
-  }
   if (((socket.flag & SOCK_MULTI_INPUT) != 0) != this->is_multi_input) {
     return false;
   }
-  if (((socket.flag & SOCK_NO_INTERNAL_LINK) != 0) != this->no_mute_links) {
-    return false;
-  }
   if (((socket.flag & SOCK_UNAVAIL) != 0) != !this->is_available) {
     return false;
   }
@@ -320,6 +321,9 @@ static bool socket_type_to_static_decl_type(const eNodeSocketDatatype socket_typ
     case SOCK_RGBA:
       fn(TypeTag<decl::Color>());
       return true;
+    case SOCK_SHADER:
+      fn(TypeTag<decl::Shader>());
+      return true;
     case SOCK_BOOLEAN:
       fn(TypeTag<decl::Bool>());
       return true;
@@ -353,6 +357,12 @@ static bool socket_type_to_static_decl_type(const eNodeSocketDatatype socket_typ
     case SOCK_MENU:
       fn(TypeTag<decl::Menu>());
       return true;
+    case SOCK_BUNDLE:
+      fn(TypeTag<decl::Bundle>());
+      return true;
+    case SOCK_CLOSURE:
+      fn(TypeTag<decl::Closure>());
+      return true;
     default:
       return false;
   }
@@ -495,10 +505,27 @@ int PanelDeclaration::depth() const
   return count;
 }
 
+const nodes::SocketDeclaration *PanelDeclaration::panel_input_decl() const
+{
+  if (this->items.is_empty()) {
+    return nullptr;
+  }
+  const nodes::ItemDeclaration *item_decl = this->items.first();
+  if (const auto *socket_decl = dynamic_cast<const nodes::SocketDeclaration *>(item_decl)) {
+    if (socket_decl->is_panel_toggle && (socket_decl->in_out & SOCK_IN) &&
+        (socket_decl->socket_type & SOCK_BOOLEAN))
+    {
+      return socket_decl;
+    }
+  }
+  return nullptr;
+}
+
 BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::supports_field()
 {
   BLI_assert(this->is_input());
   decl_base_->input_field_type = InputSocketFieldType::IsSupported;
+  this->structure_type(StructureType::Field);
   return *this;
 }
 
@@ -509,12 +536,13 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::dependent_field(
   this->reference_pass(input_dependencies);
   decl_base_->output_field_dependency = OutputFieldDependency::ForPartiallyDependentField(
       std::move(input_dependencies));
+  this->structure_type(StructureType::Dynamic);
   return *this;
 }
 
-BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::hide_label(bool value)
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::optional_label(bool value)
 {
-  decl_base_->hide_label = value;
+  decl_base_->optional_label = value;
   return *this;
 }
 
@@ -572,6 +600,7 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::field_on(const Span<
       relations.available_relations.append(relation);
     }
   }
+  this->structure_type(StructureType::Field);
   return *this;
 }
 
@@ -618,6 +647,13 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::is_default_link_sock
   return *this;
 }
 
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::default_input_type(
+    const NodeDefaultInputType value)
+{
+  decl_base_->default_input_type = value;
+  return *this;
+}
+
 BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::field_on_all()
 {
   if (this->is_input()) {
@@ -627,6 +663,7 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::field_on_all()
     this->field_source();
   }
   field_on_all_ = true;
+  this->structure_type(StructureType::Field);
   return *this;
 }
 
@@ -634,31 +671,36 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::field_source()
 {
   BLI_assert(this->is_output());
   decl_base_->output_field_dependency = OutputFieldDependency::ForFieldSource();
+  this->structure_type(StructureType::Field);
   return *this;
 }
 
-BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::implicit_field(ImplicitInputValueFn fn)
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::implicit_field(
+    const NodeDefaultInputType default_input_type)
 {
   BLI_assert(this->is_input());
   this->hide_value();
+  this->structure_type(StructureType::Dynamic);
   decl_base_->input_field_type = InputSocketFieldType::Implicit;
-  decl_base_->implicit_input_fn = std::make_unique<ImplicitInputValueFn>(std::move(fn));
+  decl_base_->default_input_type = default_input_type;
   return *this;
 }
 
 BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::implicit_field_on_all(
-    ImplicitInputValueFn fn)
+    const NodeDefaultInputType default_input_type)
 {
-  this->implicit_field(fn);
+  this->implicit_field(default_input_type);
   field_on_all_ = true;
+  this->structure_type(StructureType::Field);
   return *this;
 }
 
 BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::implicit_field_on(
-    ImplicitInputValueFn fn, const Span<int> input_indices)
+    const NodeDefaultInputType default_input_type, const Span<int> input_indices)
 {
   this->field_on(input_indices);
-  this->implicit_field(fn);
+  this->implicit_field(default_input_type);
+  this->structure_type(StructureType::Field);
   return *this;
 }
 
@@ -666,6 +708,7 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::dependent_field()
 {
   BLI_assert(this->is_output());
   decl_base_->output_field_dependency = OutputFieldDependency::ForDependentField();
+  this->structure_type(StructureType::Dynamic);
   this->reference_pass_all();
   return *this;
 }
@@ -689,6 +732,14 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::propagate_all()
   return *this;
 }
 
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::propagate_all_instance_attributes()
+{
+  /* We can't distinguish between actually propagating everything or just instance attributes
+   * currently. It's still nice to be more explicit at the node declaration level. */
+  this->propagate_all();
+  return *this;
+}
+
 BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::compositor_realization_mode(
     CompositorInputRealizationMode value)
 {
@@ -699,17 +750,11 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::compositor_realizati
 BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::compositor_domain_priority(
     int priority)
 {
+  BLI_assert(priority >= 0);
   decl_base_->compositor_domain_priority_ = priority;
   return *this;
 }
 
-BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::compositor_expects_single_value(
-    bool value)
-{
-  decl_base_->compositor_expects_single_value_ = value;
-  return *this;
-}
-
 BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::make_available(
     std::function<void(bNode &)> fn)
 {
@@ -717,12 +762,152 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::make_available(
   return *this;
 }
 
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::custom_draw(CustomSocketDrawFn fn)
+{
+  decl_base_->custom_draw_fn = std::make_unique<CustomSocketDrawFn>(std::move(fn));
+  return *this;
+}
+
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::usage_inference(
+    InputSocketUsageInferenceFn fn)
+{
+  decl_base_->usage_inference_fn = std::make_unique<InputSocketUsageInferenceFn>(std::move(fn));
+  return *this;
+}
+
+static const bNodeSocket &find_single_menu_input(const bNode &node)
+{
+#ifndef NDEBUG
+  int menu_input_count = 0;
+  /* Topology cache may not be available here and this function may be called while doing tree
+   * modifications. */
+  LISTBASE_FOREACH (bNodeSocket *, socket, &node.inputs) {
+    if (socket->type == SOCK_MENU) {
+      menu_input_count++;
+    }
+  }
+  BLI_assert(menu_input_count == 1);
+#endif
+
+  LISTBASE_FOREACH (bNodeSocket *, socket, &node.inputs) {
+    if (!socket->is_available()) {
+      continue;
+    }
+    if (socket->type != SOCK_MENU) {
+      continue;
+    }
+    return *socket;
+  }
+  BLI_assert_unreachable();
+  return node.input_socket(0);
+}
+
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::usage_by_single_menu(
+    const int menu_value)
+{
+  this->make_available([menu_value](bNode &node) {
+    bNodeSocket &socket = const_cast<bNodeSocket &>(find_single_menu_input(node));
+    bNodeSocketValueMenu *value = socket.default_value_typed<bNodeSocketValueMenu>();
+    value->value = menu_value;
+  });
+  this->usage_inference([menu_value](const socket_usage_inference::InputSocketUsageParams &params)
+                            -> std::optional<bool> {
+    const bNodeSocket &socket = find_single_menu_input(params.node);
+    if (const std::optional<bool> any_output_used = params.any_output_is_used()) {
+      if (!*any_output_used) {
+        /* If no output is used, none of the inputs is used either. */
+        return false;
+      }
+    }
+    else {
+      /* It's not known if any output is used yet. This function will be called again once new
+       * information about output usages is available. */
+      return std::nullopt;
+    }
+    return params.menu_input_may_be(socket.identifier, menu_value);
+  });
+  return *this;
+}
+
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::usage_by_menu(
+    const StringRef menu_input_identifier, const int menu_value)
+{
+  Array<int> menu_values = {menu_value};
+  this->usage_by_menu(menu_input_identifier, menu_values);
+  return *this;
+}
+
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::usage_by_menu(
+    const StringRef menu_input_identifier, const Array<int> menu_values)
+{
+  this->make_available([menu_input_identifier, menu_values](bNode &node) {
+    bNodeSocket &menu_socket = *blender::bke::node_find_socket(
+        node, SOCK_IN, menu_input_identifier);
+    const SocketDeclaration &socket_declaration = *menu_socket.runtime->declaration;
+    socket_declaration.make_available(node);
+    bNodeSocketValueMenu *value = menu_socket.default_value_typed<bNodeSocketValueMenu>();
+    value->value = menu_values[0];
+  });
+  this->usage_inference(
+      [menu_input_identifier, menu_values](
+          const socket_usage_inference::InputSocketUsageParams &params) -> std::optional<bool> {
+        if (const std::optional<bool> any_output_used = params.any_output_is_used()) {
+          if (!*any_output_used) {
+            /* If no output is used, none of the inputs is used either. */
+            return false;
+          }
+        }
+        else {
+          /* It's not known if any output is used yet. This function will be called again once new
+           * information about output usages is available. */
+          return std::nullopt;
+        }
+
+        /* Check if the menu might be any of the given values. */
+        bool menu_might_be_any_value = false;
+        for (const int menu_value : menu_values) {
+          menu_might_be_any_value = params.menu_input_may_be(menu_input_identifier, menu_value);
+          if (menu_might_be_any_value) {
+            break;
+          }
+        }
+
+        const bNodeSocket &menu_socket = *blender::bke::node_find_socket(
+            params.node, SOCK_IN, menu_input_identifier);
+        const SocketDeclaration &menu_socket_declaration = *menu_socket.runtime->declaration;
+        if (!menu_socket_declaration.usage_inference_fn) {
+          return menu_might_be_any_value;
+        }
+
+        /* If the menu socket has a usage inference function, check if it might be used. */
+        const std::optional<bool> menu_might_be_used =
+            (*menu_socket_declaration.usage_inference_fn)(params);
+        if (!menu_might_be_used.has_value()) {
+          return menu_might_be_any_value;
+        }
+
+        /* The input is only used if the menu might be any of the values and the menu itself is
+         * used. */
+        return *menu_might_be_used && menu_might_be_any_value;
+      });
+  return *this;
+}
+
 BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::align_with_previous(const bool value)
 {
   decl_base_->align_with_previous_socket = value;
   return *this;
 }
 
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::structure_type(
+    const StructureType structure_type)
+{
+  BLI_assert(NodeSocketInterfaceStructureType(structure_type) !=
+             NODE_INTERFACE_SOCKET_STRUCTURE_TYPE_AUTO);
+  decl_base_->structure_type = structure_type;
+  return *this;
+}
+
 BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder ::socket_name_ptr(
     const PointerRNA ptr, const StringRef property_name)
 {
@@ -743,6 +928,24 @@ BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::socket_name_ptr(
                                property_name);
 }
 
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::panel_toggle(const bool value)
+{
+  decl_base_->is_panel_toggle = value;
+  return *this;
+}
+
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::is_layer_name(const bool value)
+{
+  decl_base_->is_layer_name = value;
+  return *this;
+}
+
+BaseSocketDeclarationBuilder &BaseSocketDeclarationBuilder::is_volume_grid_name(const bool value)
+{
+  decl_base_->is_volume_grid_name = value;
+  return *this;
+}
+
 OutputFieldDependency OutputFieldDependency::ForFieldSource()
 {
   OutputFieldDependency field_dependency;
@@ -797,11 +1000,6 @@ int SocketDeclaration::compositor_domain_priority() const
   return compositor_domain_priority_;
 }
 
-bool SocketDeclaration::compositor_expects_single_value() const
-{
-  return compositor_expects_single_value_;
-}
-
 void SocketDeclaration::make_available(bNode &node) const
 {
   if (make_available_fn_) {
@@ -815,6 +1013,13 @@ PanelDeclarationBuilder &PanelDeclarationBuilder::description(std::string value)
   return *this;
 }
 
+PanelDeclarationBuilder &PanelDeclarationBuilder::translation_context(
+    std::optional<std::string> value)
+{
+  decl_->translation_context = value;
+  return *this;
+}
+
 PanelDeclarationBuilder &PanelDeclarationBuilder::default_closed(bool closed)
 {
   decl_->default_collapsed = closed;
@@ -823,34 +1028,102 @@ PanelDeclarationBuilder &PanelDeclarationBuilder::default_closed(bool closed)
 
 namespace implicit_field_inputs {
 
-void position(const bNode & /*node*/, void *r_value)
+static void position(const bNode & /*node*/, void *r_value)
 {
-  new (r_value) bke::SocketValueVariant(bke::AttributeFieldInput::Create<float3>("position"));
+  bke::SocketValueVariant::ConstructIn(r_value,
+                                       bke::AttributeFieldInput::from<float3>("position"));
 }
 
-void normal(const bNode & /*node*/, void *r_value)
+static void normal(const bNode & /*node*/, void *r_value)
 {
-  new (r_value)
-      bke::SocketValueVariant(fn::Field<float3>(std::make_shared<bke::NormalFieldInput>()));
+  bke::SocketValueVariant::ConstructIn(
+      r_value, fn::Field<float3>(std::make_shared<bke::NormalFieldInput>()));
 }
 
-void index(const bNode & /*node*/, void *r_value)
+static void index(const bNode & /*node*/, void *r_value)
 {
-  new (r_value) bke::SocketValueVariant(fn::Field<int>(std::make_shared<fn::IndexFieldInput>()));
+  bke::SocketValueVariant::ConstructIn(r_value,
+                                       fn::Field<int>(std::make_shared<fn::IndexFieldInput>()));
 }
 
-void id_or_index(const bNode & /*node*/, void *r_value)
+static void id_or_index(const bNode & /*node*/, void *r_value)
 {
-  new (r_value)
-      bke::SocketValueVariant(fn::Field<int>(std::make_shared<bke::IDAttributeFieldInput>()));
+  bke::SocketValueVariant::ConstructIn(
+      r_value, fn::Field<int>(std::make_shared<bke::IDAttributeFieldInput>()));
 }
 
-void instance_transform(const bNode & /*node*/, void *r_value)
+static void instance_transform(const bNode & /*node*/, void *r_value)
 {
-  new (r_value)
-      bke::SocketValueVariant(bke::AttributeFieldInput::Create<float4x4>("instance_transform"));
+  bke::SocketValueVariant::ConstructIn(
+      r_value, bke::AttributeFieldInput::from<float4x4>("instance_transform"));
+}
+
+static void handle_left(const bNode & /*node*/, void *r_value)
+{
+  bke::SocketValueVariant::ConstructIn(r_value,
+                                       bke::AttributeFieldInput::from<float3>("handle_left"));
+}
+
+static void handle_right(const bNode & /*node*/, void *r_value)
+{
+  bke::SocketValueVariant::ConstructIn(r_value,
+                                       bke::AttributeFieldInput::from<float3>("handle_right"));
 }
 
 }  // namespace implicit_field_inputs
 
+std::optional<ImplicitInputValueFn> get_implicit_input_value_fn(const NodeDefaultInputType type)
+{
+  switch (type) {
+    case NODE_DEFAULT_INPUT_VALUE:
+      return std::nullopt;
+    case NODE_DEFAULT_INPUT_INDEX_FIELD:
+      return std::make_optional(implicit_field_inputs::index);
+    case NODE_DEFAULT_INPUT_ID_INDEX_FIELD:
+      return std::make_optional(implicit_field_inputs::id_or_index);
+    case NODE_DEFAULT_INPUT_NORMAL_FIELD:
+      return std::make_optional(implicit_field_inputs::normal);
+    case NODE_DEFAULT_INPUT_POSITION_FIELD:
+      return std::make_optional(implicit_field_inputs::position);
+    case NODE_DEFAULT_INPUT_INSTANCE_TRANSFORM_FIELD:
+      return std::make_optional(implicit_field_inputs::instance_transform);
+    case NODE_DEFAULT_INPUT_HANDLE_LEFT_FIELD:
+      return std::make_optional(implicit_field_inputs::handle_left);
+    case NODE_DEFAULT_INPUT_HANDLE_RIGHT_FIELD:
+      return std::make_optional(implicit_field_inputs::handle_right);
+  }
+  return std::nullopt;
+}
+
+bool socket_type_supports_default_input_type(const bke::bNodeSocketType &socket_type,
+                                             const NodeDefaultInputType input_type)
+{
+  const eNodeSocketDatatype stype = socket_type.type;
+  switch (input_type) {
+    case NODE_DEFAULT_INPUT_VALUE:
+      return true;
+    case NODE_DEFAULT_INPUT_ID_INDEX_FIELD:
+    case NODE_DEFAULT_INPUT_INDEX_FIELD:
+      return stype == SOCK_INT;
+    case NODE_DEFAULT_INPUT_NORMAL_FIELD:
+    case NODE_DEFAULT_INPUT_POSITION_FIELD:
+    case NODE_DEFAULT_INPUT_HANDLE_LEFT_FIELD:
+    case NODE_DEFAULT_INPUT_HANDLE_RIGHT_FIELD:
+      return stype == SOCK_VECTOR;
+    case NODE_DEFAULT_INPUT_INSTANCE_TRANSFORM_FIELD:
+      return stype == SOCK_MATRIX;
+  }
+  return false;
+}
+
+void CustomSocketDrawParams::draw_standard(uiLayout &layout,
+                                           const std::optional<StringRefNull> label_override)
+{
+  this->socket.typeinfo->draw(const_cast<bContext *>(&this->C),
+                              &layout,
+                              &this->socket_ptr,
+                              &this->node_ptr,
+                              label_override.has_value() ? *label_override : this->label);
+}
+
 }  // namespace blender::nodes
