@@ -50,30 +50,6 @@ void RE_texture_rng_exit(void);
 
 void ibuf_sample(struct ImBuf *ibuf, float fx, float fy, float dx, float dy, float result[4]);
 
-/* `texture_pointdensity.cc` */
-
-struct PointDensity;
-
-void RE_point_density_cache(struct Depsgraph *depsgraph, struct PointDensity *pd);
-
-void RE_point_density_minmax(struct Depsgraph *depsgraph,
-                             struct PointDensity *pd,
-                             float r_min[3],
-                             float r_max[3]);
-
-/**
- * \note Requires #RE_point_density_cache() to be called first.
- * \note Frees point density structure after sampling.
- */
-void RE_point_density_sample(struct Depsgraph *depsgraph,
-                             struct PointDensity *pd,
-                             int resolution,
-                             float *values);
-
-void RE_point_density_free(struct PointDensity *pd);
-
-void RE_point_density_fix_linking(void);
-
 /* `texture_procedural.cc` */
 
 /**
@@ -97,9 +73,6 @@ struct TexResult {
  */
 int multitex_ext(struct Tex *tex,
                  const float texvec[3],
-                 float dxt[3],
-                 float dyt[3],
-                 int osatex,
                  struct TexResult *texres,
                  short thread,
                  struct ImagePool *pool,
@@ -127,9 +100,6 @@ int multitex_ext_safe(struct Tex *tex,
  */
 int multitex_nodes(struct Tex *tex,
                    const float texvec[3],
-                   float dxt[3],
-                   float dyt[3],
-                   int osatex,
                    struct TexResult *texres,
                    short thread,
                    short which_output,
